@@ -15,10 +15,12 @@
 
 package org.apache.pylucene.util;
 
+import org.apache.pylucene.ref.PythonObject;
+import org.apache.pylucene.ref.PythonObjectManager;
 
 public class PythonComparable implements Comparable<Object> {
 
-    private long pythonObject;
+    private PythonObject pythonObject;
 
     public PythonComparable()
     {
@@ -26,21 +28,13 @@ public class PythonComparable implements Comparable<Object> {
 
     public void pythonExtension(long pythonObject)
     {
-        this.pythonObject = pythonObject;
+        this.pythonObject = PythonObjectManager.register(pythonObject);
     }
     public long pythonExtension()
     {
-        return this.pythonObject;
+        return this.pythonObject.pythonObject;
     }
 
-    public void finalize()
-        throws Throwable
-    {
-        pythonDecRef();
-    }
-
-    public native void pythonDecRef();
-
     @Override
     public native int compareTo(Object o);
 }
