@@ -18,7 +18,7 @@ from itertools import izip
 from cpp import PRIMITIVES, INDENT, HALF_INDENT
 from cpp import RENAME_METHOD_SUFFIX, RENAME_FIELD_SUFFIX
 from cpp import cppname, cppnames, absname, typename, findClass
-from cpp import line, signature, find_method, split_pkg, sort
+from cpp import line, signature, find_method, split_pkg, sort, sortSignatures
 from cpp import Modifier, Class, Method
 from cpp import getActualTypeArguments, getTypeParameters
 from config import INCLUDES, CFLAGS, DEBUG_CFLAGS, LFLAGS, IMPLIB_LFLAGS, \
@@ -747,7 +747,7 @@ def python(env, out_h, out, cls, superCls, names, superNames,
 
     for name, methods in allMethods:
         args, x, cardinality = methodargs(methods, superMethods)
-        sort(methods, key=lambda x: len(x.getParameterTypes()))
+        sortSignatures(methods)
         method = methods[0]
         modifiers = method.getModifiers()
         if name == 'iterator' and iteratorMethod is None:
@@ -785,7 +785,7 @@ def python(env, out_h, out, cls, superCls, names, superNames,
 
     for name, methods in extMethods:
         args, x, cardinality = methodargs(methods, superMethods)
-        sort(methods, key=lambda x: len(x.getParameterTypes()))
+        sortSignatures(methods)
         method = methods[0]
         modifiers = method.getModifiers()
         if name == 'iterator' and iteratorMethod is None:
@@ -1438,7 +1438,7 @@ def package(out, allInOne, cppdir, namespace, names, use_full_names):
         line(out, 0, '#include "JCCEnv.h"')
         line(out, 0, '#include "functions.h"')
 
-    if not names:
+    if not names or not allInOne:
         line(out)
         line(out, 0, 'PyObject *initVM(PyObject *module, PyObject *args, PyObject *kwds);')
         if use_full_names:
@@ -1613,6 +1613,10 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
         with_setuptools = False
 
     extname = '_%s' %(moduleName)
+    if not os.path.isdir(os.path.join(output, extname)):
+        raise ValueError, \
+            "%s directory not found, module %s needs to be compiled first" %(
+                os.path.join(output, extname), extname)
 
     modulePath = os.path.join(output, moduleName)
     if not os.path.isdir(modulePath):
@@ -1620,18 +1624,15 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
 
     out = file(os.path.join(modulePath, '__init__.py'), 'w')
     line(out)
-    if shared:
-        line(out, 0, "import os, sys")
-        line(out)
+    line(out, 0, "import os")
+    if find_jvm_dll:
+        line(out, 0, "import sys")
         line(out, 0, "if sys.platform == 'win32':")
-        if find_jvm_dll:
-            line(out, 1, "from jcc.windows import add_jvm_dll_directory_to_path")
-            line(out, 1, "add_jvm_dll_directory_to_path()")
-        line(out, 1, "import jcc, %s", extname)
-        line(out, 0, "else:")
-        line(out, 1, "import %s", extname)
-    else:
-        line(out, 0, 'import os, %s', extname)
+        line(out, 1, "from windows import add_jvm_dll_directory_to_path")
+        line(out, 1, "add_jvm_dll_directory_to_path()")
+        modules.append(os.path.join(
+            os.path.dirname(os.path.abspath(__file__)), 'windows.py'))
+    line(out, 0, "import %s", extname)
     line(out)
     line(out, 0, '__dir__ = os.path.abspath(os.path.dirname(__file__))')
 
@@ -1884,10 +1885,11 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
         'packages': packages,
         'package_dir': {moduleName: modulePath},
         'package_data': {moduleName: package_data},
-        'version': version,
         'ext_modules': extensions,
         'script_args': script_args
     }
+    if version:
+        args['version'] = version
     if with_setuptools:
         args['zip_safe'] = False
 
