@@ -424,6 +424,7 @@ def jcc(args):
     classpath = [_jcc3.CLASSPATH]
     libpath = []
     vmargs = ['-Djava.awt.headless=true']
+    builtin_vmargs = []
     moduleName = None
     modules = []
     generate = False
@@ -484,6 +485,9 @@ def jcc(args):
             elif arg == '--vmarg':
                 i += 1
                 vmargs.append(args[i])
+            elif arg == '--builtin-vmarg':
+                i += 1
+                builtin_vmargs.append(args[i])
             elif arg == '--maxheap':
                 i += 1
                 initvm_args['maxheap'] = args[i]
@@ -626,7 +630,7 @@ def jcc(args):
                     prefix, root, install_dir, home_dir, use_distutils,
                     shared, compiler, modules, wheel, wininst, find_jvm_dll,
                     arch, generics, resources, imports, use_full_names,
-                    egg_info, extra_setup_args)
+                    egg_info, builtin_vmargs, extra_setup_args)
     else:
         if imports:
             for import_, importset in imports.items():
@@ -782,7 +786,7 @@ def jcc(args):
                         prefix, root, install_dir, home_dir, use_distutils,
                         shared, compiler, modules, wheel, wininst, find_jvm_dll,
                         arch, generics, resources, imports, use_full_names,
-                        egg_info, extra_setup_args)
+                        egg_info, builtin_vmargs, extra_setup_args)
 
 
 def header(env, out, cls, typeset, packages, excludes, generics,
