@@ -132,9 +132,10 @@ static void t_jccenv_dealloc(t_jccenv *self)
 
 static void add_option(char *name, const char *value, JavaVMOption *option)
 {
-    char *buf = (char *) malloc(strlen(name) + strlen(value) + 1);
+    size_t size = strlen(name) + strlen(value) + 1;
+    char *buf = (char *) malloc(size);
 
-    sprintf(buf, "%s%s", name, value);
+    snprintf(buf, size, "%s%s", name, value);
     option->optionString = buf;
 }
 
@@ -142,14 +143,15 @@ static void add_option(char *name, const char *value, JavaVMOption *option)
 static void add_paths(char *name, const char *p0, const char *p1,
                       JavaVMOption *option)
 {
+    size_t size = strlen(name) + strlen(p0) + strlen(p1) + 4;
 #if defined(_MSC_VER) || defined(__WIN32)
     char pathsep = ';';
 #else
     char pathsep = ':';
 #endif
-    char *buf = (char *) malloc(strlen(name) + strlen(p0) + strlen(p1) + 4);
+    char *buf = (char *) malloc(size);
 
-    sprintf(buf, "%s%s%c%s", name, p0, pathsep, p1);
+    snprintf(buf, size, "%s%s%c%s", name, p0, pathsep, p1);
     option->optionString = buf;
 }
 #endif
@@ -211,7 +213,7 @@ static PyObject *t_jccenv_strhash(PyObject *self, PyObject *arg)
     uintmax_t hash = (uintmax_t) PyObject_Hash(arg);
     char buffer[hexdig + 1];
 
-    sprintf(buffer, "%0*" PRIxMAX, (int) hexdig, hash);
+    snprintf(buffer, sizeof(buffer), "%0*" PRIxMAX, (int) hexdig, hash);
     return PyUnicode_FromStringAndSize(buffer, hexdig);
 }
 #endif
