@@ -12,7 +12,7 @@
 
 import os, sys, platform, subprocess
 
-jcc_ver = '3.14'
+jcc_ver = '3.15'
 machine = platform.machine()
 using_python2 = sys.version_info < (3,)
 
@@ -245,7 +245,12 @@ try:
             from pkg_resources.extern.packaging.version import Version
             with_modern_setuptools = True
         except ImportError:
-            with_modern_setuptools = False
+            try:
+                from pkg_resources import packaging
+                from packaging.version import Version
+                with_modern_setuptools = True
+            except ImportError:
+                with_modern_setuptools = False
 
     enable_shared = False
 
