@@ -29,12 +29,14 @@ import java.util.List;
 import org.apache.lucene.codecs.CodecUtil;
 import org.apache.lucene.codecs.KnnFieldVectorsWriter;
 import org.apache.lucene.codecs.KnnVectorsWriter;
+import org.apache.lucene.codecs.hnsw.DefaultFlatVectorScorer;
 import org.apache.lucene.codecs.lucene90.IndexedDISI;
 import org.apache.lucene.index.ByteVectorValues;
 import org.apache.lucene.index.DocsWithFieldSet;
 import org.apache.lucene.index.FieldInfo;
 import org.apache.lucene.index.FloatVectorValues;
 import org.apache.lucene.index.IndexFileNames;
+import org.apache.lucene.index.KnnVectorValues;
 import org.apache.lucene.index.MergeState;
 import org.apache.lucene.index.SegmentWriteState;
 import org.apache.lucene.index.Sorter;
@@ -51,7 +53,7 @@ import org.apache.lucene.util.hnsw.HnswGraph.NodesIterator;
 import org.apache.lucene.util.hnsw.HnswGraphBuilder;
 import org.apache.lucene.util.hnsw.NeighborArray;
 import org.apache.lucene.util.hnsw.OnHeapHnswGraph;
-import org.apache.lucene.util.hnsw.RandomAccessVectorValues;
+import org.apache.lucene.util.hnsw.RandomVectorScorerSupplier;
 import org.apache.lucene.util.packed.DirectMonotonicWriter;
 
 /**
@@ -171,12 +173,8 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     // write vector values
     long vectorDataOffset = vectorData.alignFilePointer(Float.BYTES);
     switch (fieldData.fieldInfo.getVectorEncoding()) {
-      case BYTE:
-        writeByteVectors(fieldData);
-        break;
-      default:
-      case FLOAT32:
-        writeFloat32Vectors(fieldData);
+      case BYTE -> writeByteVectors(fieldData);
+      case FLOAT32 -> writeFloat32Vectors(fieldData);
     }
     long vectorDataLength = vectorData.getFilePointer() - vectorDataOffset;
 
@@ -218,9 +216,7 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     final int[] docIdOffsets = new int[sortMap.size()];
     int offset = 1; // 0 means no vector for this (field, document)
     DocIdSetIterator iterator = fieldData.docsWithField.iterator();
-    for (int docID = iterator.nextDoc();
-        docID != DocIdSetIterator.NO_MORE_DOCS;
-        docID = iterator.nextDoc()) {
+    for (int docID = iterator.nextDoc(); docID != NO_MORE_DOCS; docID = iterator.nextDoc()) {
       int newDocID = sortMap.oldToNew(docID);
       docIdOffsets[newDocID] = offset++;
     }
@@ -240,17 +236,11 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     }
 
     // write vector values
-    long vectorDataOffset;
-    switch (fieldData.fieldInfo.getVectorEncoding()) {
-      case BYTE:
-        vectorDataOffset = writeSortedByteVectors(fieldData, ordMap);
-        break;
-      default:
-      case FLOAT32:
-        vectorDataOffset = writeSortedFloat32Vectors(fieldData, ordMap);
-        break;
-    }
-    ;
+    long vectorDataOffset =
+        switch (fieldData.fieldInfo.getVectorEncoding()) {
+          case BYTE -> writeSortedByteVectors(fieldData, ordMap);
+          case FLOAT32 -> writeSortedFloat32Vectors(fieldData, ordMap);
+        };
     long vectorDataLength = vectorData.getFilePointer() - vectorDataOffset;
 
     // write graph
@@ -313,9 +303,8 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     for (int level = 1; level < graph.numLevels(); level++) {
       NodesIterator nodesOnLevel = graph.getNodesOnLevel(level);
       int[] newNodes = new int[nodesOnLevel.size()];
-      int n = 0;
-      while (nodesOnLevel.hasNext()) {
-        newNodes[n++] = oldToNewMap[nodesOnLevel.nextInt()];
+      for (int n = 0; nodesOnLevel.hasNext(); n++) {
+        newNodes[n] = oldToNewMap[nodesOnLevel.nextInt()];
       }
       Arrays.sort(newNodes);
       nodesByLevel.add(newNodes);
@@ -368,7 +357,7 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     vectorIndex.writeInt(size);
 
     // Destructively modify; it's ok we are discarding it after this
-    int[] nnodes = neighbors.node();
+    int[] nnodes = neighbors.nodes();
     for (int i = 0; i < size; i++) {
       nnodes[i] = oldToNewMap[nnodes[i]];
     }
@@ -395,22 +384,17 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     boolean success = false;
     try {
       // write the vector data to a temporary file
-      final DocsWithFieldSet docsWithField;
-      switch (fieldInfo.getVectorEncoding()) {
-        case BYTE:
-          docsWithField =
-              writeByteVectorData(
-                  tempVectorData, MergedVectorValues.mergeByteVectorValues(fieldInfo, mergeState));
-          break;
-        case FLOAT32:
-          docsWithField =
-              writeVectorData(
-                  tempVectorData, MergedVectorValues.mergeFloatVectorValues(fieldInfo, mergeState));
-          break;
-        default:
-          throw new IllegalArgumentException(
-              "unknown vector encoding=" + fieldInfo.getVectorEncoding());
-      }
+      DocsWithFieldSet docsWithField =
+          switch (fieldInfo.getVectorEncoding()) {
+            case BYTE ->
+                writeByteVectorData(
+                    tempVectorData,
+                    MergedVectorValues.mergeByteVectorValues(fieldInfo, mergeState));
+            case FLOAT32 ->
+                writeVectorData(
+                    tempVectorData,
+                    MergedVectorValues.mergeFloatVectorValues(fieldInfo, mergeState));
+          };
       CodecUtil.writeFooter(tempVectorData);
       IOUtils.close(tempVectorData);
 
@@ -428,49 +412,45 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
       // TODO: separate random access vector values from DocIdSetIterator?
       int byteSize = fieldInfo.getVectorDimension() * fieldInfo.getVectorEncoding().byteSize;
       OnHeapHnswGraph graph = null;
+      DefaultFlatVectorScorer defaultFlatVectorScorer = new DefaultFlatVectorScorer();
       if (docsWithField.cardinality() != 0) {
         // build graph
-        switch (fieldInfo.getVectorEncoding()) {
-          case BYTE:
-            OffHeapByteVectorValues.DenseOffHeapVectorValues byteVectorValues =
-                new OffHeapByteVectorValues.DenseOffHeapVectorValues(
-                    fieldInfo.getVectorDimension(),
-                    docsWithField.cardinality(),
-                    vectorDataInput,
-                    byteSize);
-            HnswGraphBuilder<byte[]> bytesRefHnswGraphBuilder =
-                HnswGraphBuilder.create(
-                    byteVectorValues,
-                    fieldInfo.getVectorEncoding(),
-                    fieldInfo.getVectorSimilarityFunction(),
-                    M,
-                    beamWidth,
-                    HnswGraphBuilder.randSeed);
-            bytesRefHnswGraphBuilder.setInfoStream(segmentWriteState.infoStream);
-            graph = bytesRefHnswGraphBuilder.build(byteVectorValues.copy());
-            break;
-          case FLOAT32:
-            OffHeapFloatVectorValues.DenseOffHeapVectorValues vectorValues =
-                new OffHeapFloatVectorValues.DenseOffHeapVectorValues(
-                    fieldInfo.getVectorDimension(),
-                    docsWithField.cardinality(),
-                    vectorDataInput,
-                    byteSize);
-            HnswGraphBuilder<float[]> hnswGraphBuilder =
-                HnswGraphBuilder.create(
-                    vectorValues,
-                    fieldInfo.getVectorEncoding(),
-                    fieldInfo.getVectorSimilarityFunction(),
-                    M,
-                    beamWidth,
-                    HnswGraphBuilder.randSeed);
-            hnswGraphBuilder.setInfoStream(segmentWriteState.infoStream);
-            graph = hnswGraphBuilder.build(vectorValues.copy());
-            break;
-          default:
-            throw new IllegalArgumentException(
-                "unknown vector encoding=" + fieldInfo.getVectorEncoding());
-        }
+        graph =
+            switch (fieldInfo.getVectorEncoding()) {
+              case BYTE -> {
+                OffHeapByteVectorValues.DenseOffHeapVectorValues vectorValues =
+                    new OffHeapByteVectorValues.DenseOffHeapVectorValues(
+                        fieldInfo.getVectorDimension(),
+                        docsWithField.cardinality(),
+                        vectorDataInput,
+                        fieldInfo.getVectorSimilarityFunction(),
+                        byteSize);
+                RandomVectorScorerSupplier scorerSupplier =
+                    defaultFlatVectorScorer.getRandomVectorScorerSupplier(
+                        fieldInfo.getVectorSimilarityFunction(), vectorValues);
+                HnswGraphBuilder hnswGraphBuilder =
+                    HnswGraphBuilder.create(
+                        scorerSupplier, M, beamWidth, HnswGraphBuilder.randSeed);
+                hnswGraphBuilder.setInfoStream(segmentWriteState.infoStream);
+                yield hnswGraphBuilder.build(vectorValues.size());
+              }
+              case FLOAT32 -> {
+                OffHeapFloatVectorValues.DenseOffHeapVectorValues vectorValues =
+                    new OffHeapFloatVectorValues.DenseOffHeapVectorValues(
+                        fieldInfo.getVectorDimension(),
+                        docsWithField.cardinality(),
+                        vectorDataInput,
+                        fieldInfo.getVectorSimilarityFunction(),
+                        byteSize);
+                RandomVectorScorerSupplier scorerSupplier =
+                    defaultFlatVectorScorer.getRandomVectorScorerSupplier(
+                        fieldInfo.getVectorSimilarityFunction(), vectorValues);
+                HnswGraphBuilder hnswGraphBuilder =
+                    HnswGraphBuilder.create(
+                        scorerSupplier, M, beamWidth, HnswGraphBuilder.randSeed);
+                yield hnswGraphBuilder.build(vectorValues.size());
+              }
+            };
         writeGraph(graph);
       }
       long vectorIndexLength = vectorIndex.getFilePointer() - vectorIndexOffset;
@@ -502,14 +482,13 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     int countOnLevel0 = graph.size();
     for (int level = 0; level < graph.numLevels(); level++) {
       int maxConnOnLevel = level == 0 ? (M * 2) : M;
-      NodesIterator nodesOnLevel = graph.getNodesOnLevel(level);
-      while (nodesOnLevel.hasNext()) {
-        int node = nodesOnLevel.nextInt();
+      int[] sortedNodes = HnswGraph.NodesIterator.getSortedNodes(graph.getNodesOnLevel(level));
+      for (int node : sortedNodes) {
         NeighborArray neighbors = graph.getNeighbors(level, node);
         int size = neighbors.size();
         vectorIndex.writeInt(size);
         // Destructively modify; it's ok we are discarding it after this
-        int[] nnodes = neighbors.node();
+        int[] nnodes = neighbors.nodes();
         Arrays.sort(nnodes, 0, size);
         for (int i = 0; i < size; i++) {
           int nnode = nnodes[i];
@@ -575,9 +554,7 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
       final DirectMonotonicWriter ordToDocWriter =
           DirectMonotonicWriter.getInstance(meta, vectorData, count, DIRECT_MONOTONIC_BLOCK_SHIFT);
       DocIdSetIterator iterator = docsWithField.iterator();
-      for (int doc = iterator.nextDoc();
-          doc != DocIdSetIterator.NO_MORE_DOCS;
-          doc = iterator.nextDoc()) {
+      for (int doc = iterator.nextDoc(); doc != NO_MORE_DOCS; doc = iterator.nextDoc()) {
         ordToDocWriter.add(doc);
       }
       ordToDocWriter.finish();
@@ -591,11 +568,10 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     } else {
       meta.writeInt(graph.numLevels());
       for (int level = 0; level < graph.numLevels(); level++) {
-        NodesIterator nodesOnLevel = graph.getNodesOnLevel(level);
-        meta.writeInt(nodesOnLevel.size()); // number of nodes on a level
+        int[] sortedNodes = HnswGraph.NodesIterator.getSortedNodes(graph.getNodesOnLevel(level));
+        meta.writeInt(sortedNodes.length); // number of nodes on a level
         if (level > 0) {
-          while (nodesOnLevel.hasNext()) {
-            int node = nodesOnLevel.nextInt();
+          for (int node : sortedNodes) {
             meta.writeInt(node); // list of nodes on a level
           }
         }
@@ -610,11 +586,10 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
   private static DocsWithFieldSet writeByteVectorData(
       IndexOutput output, ByteVectorValues byteVectorValues) throws IOException {
     DocsWithFieldSet docsWithField = new DocsWithFieldSet();
-    for (int docV = byteVectorValues.nextDoc();
-        docV != NO_MORE_DOCS;
-        docV = byteVectorValues.nextDoc()) {
+    KnnVectorValues.DocIndexIterator iter = byteVectorValues.iterator();
+    for (int docV = iter.nextDoc(); docV != NO_MORE_DOCS; docV = iter.nextDoc()) {
       // write vector
-      byte[] binaryValue = byteVectorValues.vectorValue();
+      byte[] binaryValue = byteVectorValues.vectorValue(iter.index());
       assert binaryValue.length == byteVectorValues.dimension() * VectorEncoding.BYTE.byteSize;
       output.writeBytes(binaryValue, binaryValue.length);
       docsWithField.add(docV);
@@ -628,14 +603,13 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
   private static DocsWithFieldSet writeVectorData(
       IndexOutput output, FloatVectorValues floatVectorValues) throws IOException {
     DocsWithFieldSet docsWithField = new DocsWithFieldSet();
+    KnnVectorValues.DocIndexIterator iter = floatVectorValues.iterator();
     ByteBuffer binaryVector =
         ByteBuffer.allocate(floatVectorValues.dimension() * VectorEncoding.FLOAT32.byteSize)
             .order(ByteOrder.LITTLE_ENDIAN);
-    for (int docV = floatVectorValues.nextDoc();
-        docV != NO_MORE_DOCS;
-        docV = floatVectorValues.nextDoc()) {
+    for (int docV = iter.nextDoc(); docV != NO_MORE_DOCS; docV = iter.nextDoc()) {
       // write vector
-      float[] vectorValue = floatVectorValues.vectorValue();
+      float[] vectorValue = floatVectorValues.vectorValue(iter.index());
       binaryVector.asFloatBuffer().put(vectorValue);
       output.writeBytes(binaryVector.array(), binaryVector.limit());
       docsWithField.add(docV);
@@ -653,7 +627,7 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     private final int dim;
     private final DocsWithFieldSet docsWithField;
     private final List<T> vectors;
-    private final HnswGraphBuilder<T> hnswGraphBuilder;
+    private final HnswGraphBuilder hnswGraphBuilder;
 
     private int lastDocID = -1;
     private int node = 0;
@@ -661,40 +635,45 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
     static FieldWriter<?> create(FieldInfo fieldInfo, int M, int beamWidth, InfoStream infoStream)
         throws IOException {
       int dim = fieldInfo.getVectorDimension();
-      switch (fieldInfo.getVectorEncoding()) {
-        case BYTE:
-          return new FieldWriter<byte[]>(fieldInfo, M, beamWidth, infoStream) {
-            @Override
-            public byte[] copyValue(byte[] value) {
-              return ArrayUtil.copyOfSubArray(value, 0, dim);
-            }
-          };
-        default:
-        case FLOAT32:
-          return new FieldWriter<float[]>(fieldInfo, M, beamWidth, infoStream) {
-            @Override
-            public float[] copyValue(float[] value) {
-              return ArrayUtil.copyOfSubArray(value, 0, dim);
-            }
-          };
-      }
+      return switch (fieldInfo.getVectorEncoding()) {
+        case BYTE ->
+            new FieldWriter<byte[]>(fieldInfo, M, beamWidth, infoStream) {
+              @Override
+              public byte[] copyValue(byte[] value) {
+                return ArrayUtil.copyOfSubArray(value, 0, dim);
+              }
+            };
+        case FLOAT32 ->
+            new FieldWriter<float[]>(fieldInfo, M, beamWidth, infoStream) {
+              @Override
+              public float[] copyValue(float[] value) {
+                return ArrayUtil.copyOfSubArray(value, 0, dim);
+              }
+            };
+      };
     }
 
+    @SuppressWarnings("unchecked")
     FieldWriter(FieldInfo fieldInfo, int M, int beamWidth, InfoStream infoStream)
         throws IOException {
       this.fieldInfo = fieldInfo;
       this.dim = fieldInfo.getVectorDimension();
       this.docsWithField = new DocsWithFieldSet();
       vectors = new ArrayList<>();
-      RAVectorValues<T> raVectorValues = new RAVectorValues<>(vectors, dim);
+      DefaultFlatVectorScorer defaultFlatVectorScorer = new DefaultFlatVectorScorer();
+      RandomVectorScorerSupplier scorerSupplier =
+          switch (fieldInfo.getVectorEncoding()) {
+            case BYTE ->
+                defaultFlatVectorScorer.getRandomVectorScorerSupplier(
+                    fieldInfo.getVectorSimilarityFunction(),
+                    ByteVectorValues.fromBytes((List<byte[]>) vectors, dim));
+            case FLOAT32 ->
+                defaultFlatVectorScorer.getRandomVectorScorerSupplier(
+                    fieldInfo.getVectorSimilarityFunction(),
+                    FloatVectorValues.fromFloats((List<float[]>) vectors, dim));
+          };
       hnswGraphBuilder =
-          HnswGraphBuilder.create(
-              raVectorValues,
-              fieldInfo.getVectorEncoding(),
-              fieldInfo.getVectorSimilarityFunction(),
-              M,
-              beamWidth,
-              HnswGraphBuilder.randSeed);
+          HnswGraphBuilder.create(scorerSupplier, M, beamWidth, HnswGraphBuilder.randSeed);
       hnswGraphBuilder.setInfoStream(infoStream);
     }
 
@@ -711,14 +690,14 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
       assert docID > lastDocID;
       docsWithField.add(docID);
       vectors.add(copyValue(vectorValue));
-      hnswGraphBuilder.addGraphNode(node, vectorValue);
+      hnswGraphBuilder.addGraphNode(node);
       node++;
       lastDocID = docID;
     }
 
-    OnHeapHnswGraph getGraph() {
+    OnHeapHnswGraph getGraph() throws IOException {
       if (vectors.size() > 0) {
-        return hnswGraphBuilder.getGraph();
+        return hnswGraphBuilder.getCompletedGraph();
       } else {
         return null;
       }
@@ -735,34 +714,4 @@ public final class Lucene94HnswVectorsWriter extends KnnVectorsWriter {
           + hnswGraphBuilder.getGraph().ramBytesUsed();
     }
   }
-
-  private static class RAVectorValues<T> implements RandomAccessVectorValues<T> {
-    private final List<T> vectors;
-    private final int dim;
-
-    RAVectorValues(List<T> vectors, int dim) {
-      this.vectors = vectors;
-      this.dim = dim;
-    }
-
-    @Override
-    public int size() {
-      return vectors.size();
-    }
-
-    @Override
-    public int dimension() {
-      return dim;
-    }
-
-    @Override
-    public T vectorValue(int targetOrd) throws IOException {
-      return vectors.get(targetOrd);
-    }
-
-    @Override
-    public RAVectorValues<T> copy() throws IOException {
-      return this;
-    }
-  }
 }
