@@ -37,10 +37,8 @@ import org.apache.lucene.index.IndexFileNames;
 import org.apache.lucene.index.SegmentReadState;
 import org.apache.lucene.index.VectorSimilarityFunction;
 import org.apache.lucene.search.DocIdSetIterator;
-import org.apache.lucene.search.HitQueue;
-import org.apache.lucene.search.ScoreDoc;
-import org.apache.lucene.search.TopDocs;
-import org.apache.lucene.search.TotalHits;
+import org.apache.lucene.search.KnnCollector;
+import org.apache.lucene.search.VectorScorer;
 import org.apache.lucene.store.BufferedChecksumIndexInput;
 import org.apache.lucene.store.ChecksumIndexInput;
 import org.apache.lucene.store.IOContext;
@@ -49,9 +47,7 @@ import org.apache.lucene.util.Bits;
 import org.apache.lucene.util.BytesRef;
 import org.apache.lucene.util.BytesRefBuilder;
 import org.apache.lucene.util.IOUtils;
-import org.apache.lucene.util.RamUsageEstimator;
 import org.apache.lucene.util.StringHelper;
-import org.apache.lucene.util.hnsw.RandomAccessVectorValues;
 
 /**
  * Reads vector values from a simple text format. All vectors are read up front and cached in RAM in
@@ -61,9 +57,6 @@ import org.apache.lucene.util.hnsw.RandomAccessVectorValues;
  */
 public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
   // shallowSizeOfInstance for fieldEntries map is included in ramBytesUsed() calculation
-  private static final long BASE_RAM_BYTES_USED =
-      RamUsageEstimator.shallowSizeOfInstance(SimpleTextKnnVectorsReader.class)
-          + RamUsageEstimator.shallowSizeOfInstance(BytesRef.class);
 
   private static final BytesRef EMPTY = new BytesRef("");
 
@@ -86,8 +79,7 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
             SimpleTextKnnVectorsFormat.VECTOR_EXTENSION);
 
     boolean success = false;
-    try (ChecksumIndexInput in =
-        readState.directory.openChecksumInput(metaFileName, IOContext.DEFAULT)) {
+    try (ChecksumIndexInput in = readState.directory.openChecksumInput(metaFileName)) {
       int fieldNumber = readInt(in, FIELD_NUMBER);
       while (fieldNumber != -1) {
         String fieldName = readString(in, FIELD_NAME);
@@ -101,7 +93,13 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
         }
         assert fieldEntries.containsKey(fieldName) == false;
         fieldEntries.put(
-            fieldName, new FieldEntry(dimension, vectorDataOffset, vectorDataLength, docIds));
+            fieldName,
+            new FieldEntry(
+                dimension,
+                vectorDataOffset,
+                vectorDataLength,
+                docIds,
+                readState.fieldInfos.fieldInfo(fieldName).getVectorSimilarityFunction()));
         fieldNumber = readInt(in, FIELD_NUMBER);
       }
       SimpleTextUtil.checkFooter(in);
@@ -182,7 +180,7 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
   }
 
   @Override
-  public TopDocs search(String field, float[] target, int k, Bits acceptDocs, int visitedLimit)
+  public void search(String field, float[] target, KnnCollector knnCollector, Bits acceptDocs)
       throws IOException {
     FloatVectorValues values = getFloatVectorValues(field);
     if (target.length != values.dimension()) {
@@ -194,36 +192,25 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
     }
     FieldInfo info = readState.fieldInfos.fieldInfo(field);
     VectorSimilarityFunction vectorSimilarity = info.getVectorSimilarityFunction();
-    HitQueue topK = new HitQueue(k, false);
-
-    int numVisited = 0;
-    TotalHits.Relation relation = TotalHits.Relation.EQUAL_TO;
-
-    int doc;
-    while ((doc = values.nextDoc()) != DocIdSetIterator.NO_MORE_DOCS) {
+    for (int ord = 0; ord < values.size(); ord++) {
+      int doc = values.ordToDoc(ord);
       if (acceptDocs != null && acceptDocs.get(doc) == false) {
         continue;
       }
 
-      if (numVisited >= visitedLimit) {
-        relation = TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO;
+      if (knnCollector.earlyTerminated()) {
         break;
       }
 
-      float[] vector = values.vectorValue();
+      float[] vector = values.vectorValue(ord);
       float score = vectorSimilarity.compare(vector, target);
-      topK.insertWithOverflow(new ScoreDoc(doc, score));
-      numVisited++;
+      knnCollector.collect(doc, score);
+      knnCollector.incVisitedCount(1);
     }
-    ScoreDoc[] topScoreDocs = new ScoreDoc[topK.size()];
-    for (int i = topScoreDocs.length - 1; i >= 0; i--) {
-      topScoreDocs[i] = topK.pop();
-    }
-    return new TopDocs(new TotalHits(numVisited, relation), topScoreDocs);
   }
 
   @Override
-  public TopDocs search(String field, byte[] target, int k, Bits acceptDocs, int visitedLimit)
+  public void search(String field, byte[] target, KnnCollector knnCollector, Bits acceptDocs)
       throws IOException {
     ByteVectorValues values = getByteVectorValues(field);
     if (target.length != values.dimension()) {
@@ -235,32 +222,22 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
     }
     FieldInfo info = readState.fieldInfos.fieldInfo(field);
     VectorSimilarityFunction vectorSimilarity = info.getVectorSimilarityFunction();
-    HitQueue topK = new HitQueue(k, false);
 
-    int numVisited = 0;
-    TotalHits.Relation relation = TotalHits.Relation.EQUAL_TO;
-
-    int doc;
-    while ((doc = values.nextDoc()) != DocIdSetIterator.NO_MORE_DOCS) {
+    for (int ord = 0; ord < values.size(); ord++) {
+      int doc = values.ordToDoc(ord);
       if (acceptDocs != null && acceptDocs.get(doc) == false) {
         continue;
       }
 
-      if (numVisited >= visitedLimit) {
-        relation = TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO;
+      if (knnCollector.earlyTerminated()) {
         break;
       }
 
-      byte[] vector = values.vectorValue();
+      byte[] vector = values.vectorValue(ord);
       float score = vectorSimilarity.compare(vector, target);
-      topK.insertWithOverflow(new ScoreDoc(doc, score));
-      numVisited++;
+      knnCollector.collect(doc, score);
+      knnCollector.incVisitedCount(1);
     }
-    ScoreDoc[] topScoreDocs = new ScoreDoc[topK.size()];
-    for (int i = topScoreDocs.length - 1; i >= 0; i--) {
-      topScoreDocs[i] = topK.pop();
-    }
-    return new TopDocs(new TotalHits(numVisited, relation), topScoreDocs);
   }
 
   @Override
@@ -299,47 +276,23 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
     }
   }
 
-  @Override
-  public long ramBytesUsed() {
-    // mirror implementation of Lucene90VectorReader#ramBytesUsed
-    long totalBytes = BASE_RAM_BYTES_USED;
-    totalBytes += RamUsageEstimator.sizeOf(scratch.bytes());
-    totalBytes +=
-        RamUsageEstimator.sizeOfMap(
-            fieldEntries, RamUsageEstimator.shallowSizeOfInstance(FieldEntry.class));
-    for (FieldEntry entry : fieldEntries.values()) {
-      totalBytes += RamUsageEstimator.sizeOf(entry.ordToDoc);
-    }
-    return totalBytes;
-  }
-
   @Override
   public void close() throws IOException {
     dataIn.close();
   }
 
-  private static class FieldEntry {
-
-    final int dimension;
-
-    final long vectorDataOffset;
-    final long vectorDataLength;
-    final int[] ordToDoc;
-
-    FieldEntry(int dimension, long vectorDataOffset, long vectorDataLength, int[] ordToDoc) {
-      this.dimension = dimension;
-      this.vectorDataOffset = vectorDataOffset;
-      this.vectorDataLength = vectorDataLength;
-      this.ordToDoc = ordToDoc;
-    }
-
+  private record FieldEntry(
+      int dimension,
+      long vectorDataOffset,
+      long vectorDataLength,
+      int[] ordToDoc,
+      VectorSimilarityFunction similarityFunction) {
     int size() {
       return ordToDoc.length;
     }
   }
 
-  private static class SimpleTextFloatVectorValues extends FloatVectorValues
-      implements RandomAccessVectorValues<float[]> {
+  private static class SimpleTextFloatVectorValues extends FloatVectorValues {
 
     private final BytesRefBuilder scratch = new BytesRefBuilder();
     private final FieldEntry entry;
@@ -356,6 +309,13 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
       readAllVectors();
     }
 
+    private SimpleTextFloatVectorValues(SimpleTextFloatVectorValues other) {
+      this.entry = other.entry;
+      this.in = other.in.clone();
+      this.values = other.values;
+      this.curOrd = other.curOrd;
+    }
+
     @Override
     public int dimension() {
       return entry.dimension;
@@ -367,40 +327,42 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
     }
 
     @Override
-    public float[] vectorValue() {
-      return values[curOrd];
+    public float[] vectorValue(int ord) {
+      return values[ord];
     }
 
     @Override
-    public RandomAccessVectorValues<float[]> copy() {
-      return this;
+    public int ordToDoc(int ord) {
+      return entry.ordToDoc[ord];
     }
 
     @Override
-    public int docID() {
-      if (curOrd == -1) {
-        return -1;
-      } else if (curOrd >= entry.size()) {
-        // when call to advance / nextDoc below already returns NO_MORE_DOCS, calling docID
-        // immediately afterward should also return NO_MORE_DOCS
-        // this is needed for TestSimpleTextKnnVectorsFormat.testAdvance test case
-        return NO_MORE_DOCS;
+    public DocIndexIterator iterator() {
+      return createSparseIterator();
+    }
+
+    @Override
+    public VectorScorer scorer(float[] target) {
+      if (size() == 0) {
+        return null;
       }
+      SimpleTextFloatVectorValues simpleTextFloatVectorValues =
+          new SimpleTextFloatVectorValues(this);
+      DocIndexIterator iterator = simpleTextFloatVectorValues.iterator();
+      return new VectorScorer() {
+        @Override
+        public float score() throws IOException {
+          int ord = iterator.index();
+          return entry
+              .similarityFunction()
+              .compare(simpleTextFloatVectorValues.vectorValue(ord), target);
+        }
 
-      return entry.ordToDoc[curOrd];
-    }
-
-    @Override
-    public int nextDoc() throws IOException {
-      if (++curOrd < entry.size()) {
-        return docID();
-      }
-      return NO_MORE_DOCS;
-    }
-
-    @Override
-    public int advance(int target) throws IOException {
-      return slowAdvance(target);
+        @Override
+        public DocIdSetIterator iterator() {
+          return iterator;
+        }
+      };
     }
 
     private void readAllVectors() throws IOException {
@@ -422,13 +384,12 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
     }
 
     @Override
-    public float[] vectorValue(int targetOrd) throws IOException {
-      return values[targetOrd];
+    public SimpleTextFloatVectorValues copy() {
+      return this;
     }
   }
 
-  private static class SimpleTextByteVectorValues extends ByteVectorValues
-      implements RandomAccessVectorValues<BytesRef> {
+  private static class SimpleTextByteVectorValues extends ByteVectorValues {
 
     private final BytesRefBuilder scratch = new BytesRefBuilder();
     private final FieldEntry entry;
@@ -448,6 +409,15 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
       readAllVectors();
     }
 
+    private SimpleTextByteVectorValues(SimpleTextByteVectorValues other) {
+      this.entry = other.entry;
+      this.in = other.in.clone();
+      this.values = other.values;
+      this.binaryValue = new BytesRef(entry.dimension);
+      this.binaryValue.length = binaryValue.bytes.length;
+      this.curOrd = other.curOrd;
+    }
+
     @Override
     public int dimension() {
       return entry.dimension;
@@ -459,41 +429,43 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
     }
 
     @Override
-    public byte[] vectorValue() {
-      binaryValue.bytes = values[curOrd];
+    public byte[] vectorValue(int ord) {
+      binaryValue.bytes = values[ord];
       return binaryValue.bytes;
     }
 
     @Override
-    public RandomAccessVectorValues<BytesRef> copy() {
-      return this;
+    public int ordToDoc(int ord) {
+      return entry.ordToDoc[ord];
     }
 
     @Override
-    public int docID() {
-      if (curOrd == -1) {
-        return -1;
-      } else if (curOrd >= entry.size()) {
-        // when call to advance / nextDoc below already returns NO_MORE_DOCS, calling docID
-        // immediately afterward should also return NO_MORE_DOCS
-        // this is needed for TestSimpleTextKnnVectorsFormat.testAdvance test case
-        return NO_MORE_DOCS;
+    public DocIndexIterator iterator() {
+      return createSparseIterator();
+    }
+
+    @Override
+    public VectorScorer scorer(byte[] target) {
+      if (size() == 0) {
+        return null;
       }
+      SimpleTextByteVectorValues simpleTextByteVectorValues = new SimpleTextByteVectorValues(this);
+      return new VectorScorer() {
+        DocIndexIterator it = simpleTextByteVectorValues.iterator();
 
-      return entry.ordToDoc[curOrd];
-    }
+        @Override
+        public float score() throws IOException {
+          int ord = it.index();
+          return entry
+              .similarityFunction()
+              .compare(simpleTextByteVectorValues.vectorValue(ord), target);
+        }
 
-    @Override
-    public int nextDoc() throws IOException {
-      if (++curOrd < entry.size()) {
-        return docID();
-      }
-      return NO_MORE_DOCS;
-    }
-
-    @Override
-    public int advance(int target) throws IOException {
-      return slowAdvance(target);
+        @Override
+        public DocIdSetIterator iterator() {
+          return it;
+        }
+      };
     }
 
     private void readAllVectors() throws IOException {
@@ -515,9 +487,8 @@ public class SimpleTextKnnVectorsReader extends KnnVectorsReader {
     }
 
     @Override
-    public BytesRef vectorValue(int targetOrd) throws IOException {
-      binaryValue.bytes = values[curOrd];
-      return binaryValue;
+    public SimpleTextByteVectorValues copy() {
+      return this;
     }
   }
 
