@@ -16,6 +16,8 @@
  */
 package org.apache.lucene.codecs.lucene90;
 
+import static org.apache.lucene.codecs.lucene90.Lucene90DocValuesFormat.SKIP_INDEX_JUMP_LENGTH_PER_LEVEL;
+import static org.apache.lucene.codecs.lucene90.Lucene90DocValuesFormat.SKIP_INDEX_MAX_LEVEL;
 import static org.apache.lucene.codecs.lucene90.Lucene90DocValuesFormat.TERMS_DICT_BLOCK_LZ4_SHIFT;
 
 import java.io.IOException;
@@ -27,6 +29,8 @@ import org.apache.lucene.index.BaseTermsEnum;
 import org.apache.lucene.index.BinaryDocValues;
 import org.apache.lucene.index.CorruptIndexException;
 import org.apache.lucene.index.DocValues;
+import org.apache.lucene.index.DocValuesSkipIndexType;
+import org.apache.lucene.index.DocValuesSkipper;
 import org.apache.lucene.index.FieldInfo;
 import org.apache.lucene.index.FieldInfos;
 import org.apache.lucene.index.ImpactsEnum;
@@ -39,12 +43,13 @@ import org.apache.lucene.index.SortedNumericDocValues;
 import org.apache.lucene.index.SortedSetDocValues;
 import org.apache.lucene.index.TermsEnum;
 import org.apache.lucene.index.TermsEnum.SeekStatus;
+import org.apache.lucene.search.DocIdSetIterator;
 import org.apache.lucene.store.ByteArrayDataInput;
 import org.apache.lucene.store.ChecksumIndexInput;
 import org.apache.lucene.store.DataInput;
-import org.apache.lucene.store.IOContext;
 import org.apache.lucene.store.IndexInput;
 import org.apache.lucene.store.RandomAccessInput;
+import org.apache.lucene.store.ReadAdvice;
 import org.apache.lucene.util.BytesRef;
 import org.apache.lucene.util.IOUtils;
 import org.apache.lucene.util.LongValues;
@@ -59,6 +64,7 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
   private final Map<String, SortedEntry> sorted;
   private final Map<String, SortedSetEntry> sortedSets;
   private final Map<String, SortedNumericEntry> sortedNumerics;
+  private final Map<String, DocValuesSkipperEntry> skippers;
   private final IndexInput data;
   private final int maxDoc;
   private int version = -1;
@@ -80,10 +86,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
     sorted = new HashMap<>();
     sortedSets = new HashMap<>();
     sortedNumerics = new HashMap<>();
+    skippers = new HashMap<>();
     merging = false;
 
     // read in the entries from the metadata file.
-    try (ChecksumIndexInput in = state.directory.openChecksumInput(metaName, IOContext.READONCE)) {
+    try (ChecksumIndexInput in = state.directory.openChecksumInput(metaName)) {
       Throwable priorE = null;
 
       try {
@@ -107,7 +114,10 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
 
     String dataName =
         IndexFileNames.segmentFileName(state.segmentInfo.name, state.segmentSuffix, dataExtension);
-    this.data = state.directory.openInput(dataName, state.context);
+    // Doc-values have a forward-only access pattern, so pass ReadAdvice.NORMAL to perform
+    // readahead.
+    this.data =
+        state.directory.openInput(dataName, state.context.withReadAdvice(ReadAdvice.NORMAL));
     boolean success = false;
     try {
       final int version2 =
@@ -144,6 +154,7 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
       Map<String, SortedEntry> sorted,
       Map<String, SortedSetEntry> sortedSets,
       Map<String, SortedNumericEntry> sortedNumerics,
+      Map<String, DocValuesSkipperEntry> skippers,
       IndexInput data,
       int maxDoc,
       int version,
@@ -153,6 +164,7 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
     this.sorted = sorted;
     this.sortedSets = sortedSets;
     this.sortedNumerics = sortedNumerics;
+    this.skippers = skippers;
     this.data = data.clone();
     this.maxDoc = maxDoc;
     this.version = version;
@@ -162,7 +174,16 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
   @Override
   public DocValuesProducer getMergeInstance() {
     return new Lucene90DocValuesProducer(
-        numerics, binaries, sorted, sortedSets, sortedNumerics, data, maxDoc, version, true);
+        numerics,
+        binaries,
+        sorted,
+        sortedSets,
+        sortedNumerics,
+        skippers,
+        data,
+        maxDoc,
+        version,
+        true);
   }
 
   private void readFields(IndexInput meta, FieldInfos infos) throws IOException {
@@ -172,6 +193,9 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
         throw new CorruptIndexException("Invalid field number: " + fieldNumber, meta);
       }
       byte type = meta.readByte();
+      if (info.docValuesSkipIndexType() != DocValuesSkipIndexType.NONE) {
+        skippers.put(info.name, readDocValueSkipperMeta(meta));
+      }
       if (type == Lucene90DocValuesFormat.NUMERIC) {
         numerics.put(info.name, readNumeric(meta));
       } else if (type == Lucene90DocValuesFormat.BINARY) {
@@ -194,6 +218,17 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
     return entry;
   }
 
+  private DocValuesSkipperEntry readDocValueSkipperMeta(IndexInput meta) throws IOException {
+    long offset = meta.readLong();
+    long length = meta.readLong();
+    long maxValue = meta.readLong();
+    long minValue = meta.readLong();
+    int docCount = meta.readInt();
+    int maxDocID = meta.readInt();
+
+    return new DocValuesSkipperEntry(offset, length, minValue, maxValue, docCount, maxDocID);
+  }
+
   private void readNumeric(IndexInput meta, NumericEntry entry) throws IOException {
     entry.docsWithFieldOffset = meta.readLong();
     entry.docsWithFieldLength = meta.readLong();
@@ -323,6 +358,9 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
     data.close();
   }
 
+  private record DocValuesSkipperEntry(
+      long offset, long length, long minValue, long maxValue, int docCount, int maxDocId) {}
+
   private static class NumericEntry {
     long[] table;
     int blockShift;
@@ -494,6 +532,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
       } else {
         final RandomAccessInput slice =
             data.randomAccessSlice(entry.valuesOffset, entry.valuesLength);
+        // Prefetch the first page of data. Following pages are expected to get prefetched through
+        // read-ahead.
+        if (slice.length() > 0) {
+          slice.prefetch(0, 1);
+        }
         if (entry.blockShift >= 0) {
           // dense but split into blocks of different bits per value
           return new DenseNumericDocValues(maxDoc) {
@@ -555,6 +598,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
       } else {
         final RandomAccessInput slice =
             data.randomAccessSlice(entry.valuesOffset, entry.valuesLength);
+        // Prefetch the first page of data. Following pages are expected to get prefetched through
+        // read-ahead.
+        if (slice.length() > 0) {
+          slice.prefetch(0, 1);
+        }
         if (entry.blockShift >= 0) {
           // sparse and split into blocks of different bits per value
           return new SparseNumericDocValues(disi) {
@@ -610,6 +658,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
     } else {
       final RandomAccessInput slice =
           data.randomAccessSlice(entry.valuesOffset, entry.valuesLength);
+      // Prefetch the first page of data. Following pages are expected to get prefetched through
+      // read-ahead.
+      if (slice.length() > 0) {
+        slice.prefetch(0, 1);
+      }
       if (entry.blockShift >= 0) {
         return new LongValues() {
           final VaryingBPVReader vBPVReader = new VaryingBPVReader(entry, slice);
@@ -740,6 +793,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
     }
 
     final IndexInput bytesSlice = data.slice("fixed-binary", entry.dataOffset, entry.dataLength);
+    // Prefetch the first page of data. Following pages are expected to get prefetched through
+    // read-ahead.
+    if (bytesSlice.length() > 0) {
+      bytesSlice.prefetch(0, 1);
+    }
 
     if (entry.docsWithFieldOffset == -1) {
       // dense
@@ -760,6 +818,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
         // variable length
         final RandomAccessInput addressesData =
             this.data.randomAccessSlice(entry.addressesOffset, entry.addressesLength);
+        // Prefetch the first page of data. Following pages are expected to get prefetched through
+        // read-ahead.
+        if (addressesData.length() > 0) {
+          addressesData.prefetch(0, 1);
+        }
         final LongValues addresses =
             DirectMonotonicReader.getInstance(entry.addressesMeta, addressesData, merging);
         return new DenseBinaryDocValues(maxDoc) {
@@ -802,6 +865,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
         // variable length
         final RandomAccessInput addressesData =
             this.data.randomAccessSlice(entry.addressesOffset, entry.addressesLength);
+        // Prefetch the first page of data. Following pages are expected to get prefetched through
+        // read-ahead.
+        if (addressesData.length() > 0) {
+          addressesData.prefetch(0, 1);
+        }
         final LongValues addresses =
             DirectMonotonicReader.getInstance(entry.addressesMeta, addressesData);
         return new SparseBinaryDocValues(disi) {
@@ -839,6 +907,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
 
       final RandomAccessInput slice =
           data.randomAccessSlice(ordsEntry.valuesOffset, ordsEntry.valuesLength);
+      // Prefetch the first page of data. Following pages are expected to get prefetched through
+      // read-ahead.
+      if (slice.length() > 0) {
+        slice.prefetch(0, 1);
+      }
       final LongValues values =
           getDirectReaderInstance(slice, ordsEntry.bitsPerValue, 0L, ordsEntry.numValues);
 
@@ -1154,6 +1227,7 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
       assert hi < 0 || getTermFromIndex(hi).compareTo(text) <= 0;
       assert hi == ((entry.termsDictSize - 1) >> entry.termsDictIndexShift)
           || getTermFromIndex(hi + 1).compareTo(text) > 0;
+      assert hi < 0 ^ entry.termsDictSize > 0; // return -1 iff empty term dict
 
       return hi;
     }
@@ -1170,7 +1244,9 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
     private long seekBlock(BytesRef text) throws IOException {
       long index = seekTermsIndex(text);
       if (index == -1L) {
-        return -1L;
+        // empty terms dict
+        this.ord = 0;
+        return -2L;
       }
 
       long ordLo = index << entry.termsDictIndexShift;
@@ -1194,26 +1270,30 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
       assert blockHi == ((entry.termsDictSize - 1) >>> TERMS_DICT_BLOCK_LZ4_SHIFT)
           || getFirstTermFromBlock(blockHi + 1).compareTo(text) > 0;
 
+      // read the block only if term dict is not empty
+      assert entry.termsDictSize > 0;
+      // reset ord and bytes to the ceiling block even if
+      // text is before the first term (blockHi == -1)
+      final long block = Math.max(blockHi, 0);
+      final long blockAddress = blockAddresses.get(block);
+      this.ord = block << TERMS_DICT_BLOCK_LZ4_SHIFT;
+      bytes.seek(blockAddress);
+      decompressBlock();
+
       return blockHi;
     }
 
     @Override
     public SeekStatus seekCeil(BytesRef text) throws IOException {
       final long block = seekBlock(text);
-      if (block == -1) {
-        // before the first term, or empty terms dict
-        if (entry.termsDictSize == 0) {
-          ord = 0;
-          return SeekStatus.END;
-        } else {
-          seekExact(0L);
-          return SeekStatus.NOT_FOUND;
-        }
+      if (block == -2) {
+        // empty terms dict
+        assert entry.termsDictSize == 0;
+        return SeekStatus.END;
+      } else if (block == -1) {
+        // before the first term
+        return SeekStatus.NOT_FOUND;
       }
-      final long blockAddress = blockAddresses.get(block);
-      this.ord = block << TERMS_DICT_BLOCK_LZ4_SHIFT;
-      bytes.seek(blockAddress);
-      decompressBlock();
 
       while (true) {
         int cmp = term.compareTo(text);
@@ -1299,6 +1379,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
 
     final RandomAccessInput addressesInput =
         data.randomAccessSlice(entry.addressesOffset, entry.addressesLength);
+    // Prefetch the first page of data. Following pages are expected to get prefetched through
+    // read-ahead.
+    if (addressesInput.length() > 0) {
+      addressesInput.prefetch(0, 1);
+    }
     final LongValues addresses =
         DirectMonotonicReader.getInstance(entry.addressesMeta, addressesInput, merging);
 
@@ -1442,11 +1527,21 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
 
       final RandomAccessInput addressesInput =
           data.randomAccessSlice(ordsEntry.addressesOffset, ordsEntry.addressesLength);
+      // Prefetch the first page of data. Following pages are expected to get prefetched through
+      // read-ahead.
+      if (addressesInput.length() > 0) {
+        addressesInput.prefetch(0, 1);
+      }
       final LongValues addresses =
           DirectMonotonicReader.getInstance(ordsEntry.addressesMeta, addressesInput);
 
       final RandomAccessInput slice =
           data.randomAccessSlice(ordsEntry.valuesOffset, ordsEntry.valuesLength);
+      // Prefetch the first page of data. Following pages are expected to get prefetched through
+      // read-ahead.
+      if (slice.length() > 0) {
+        slice.prefetch(0, 1);
+      }
       final LongValues values = DirectReader.getInstance(slice, ordsEntry.bitsPerValue);
 
       if (ordsEntry.docsWithFieldOffset == -1) { // dense
@@ -1454,23 +1549,19 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
 
           private final int maxDoc = Lucene90DocValuesProducer.this.maxDoc;
           private int doc = -1;
-          private long start;
-          private long end;
+          private long curr;
           private int count;
 
           @Override
           public long nextOrd() throws IOException {
-            if (start == end) {
-              return NO_MORE_ORDS;
-            }
-            return values.get(start++);
+            return values.get(curr++);
           }
 
           @Override
           public boolean advanceExact(int target) throws IOException {
-            start = addresses.get(target);
-            end = addresses.get(target + 1L);
-            count = (int) (end - start);
+            curr = addresses.get(target);
+            long end = addresses.get(target + 1L);
+            count = (int) (end - curr);
             doc = target;
             return true;
           }
@@ -1495,9 +1586,9 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
             if (target >= maxDoc) {
               return doc = NO_MORE_DOCS;
             }
-            start = addresses.get(target);
-            end = addresses.get(target + 1L);
-            count = (int) (end - start);
+            curr = addresses.get(target);
+            long end = addresses.get(target + 1L);
+            count = (int) (end - curr);
             return doc = target;
           }
 
@@ -1519,16 +1610,13 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
         return new BaseSortedSetDocValues(entry, data) {
 
           boolean set;
-          long start, end;
+          long curr;
           int count;
 
           @Override
           public long nextOrd() throws IOException {
             set();
-            if (start == end) {
-              return NO_MORE_ORDS;
-            }
-            return values.get(start++);
+            return values.get(curr++);
           }
 
           @Override
@@ -1568,9 +1656,9 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
           private void set() {
             if (set == false) {
               final int index = disi.index();
-              start = addresses.get(index);
-              end = addresses.get(index + 1L);
-              count = (int) (end - start);
+              curr = addresses.get(index);
+              long end = addresses.get(index + 1L);
+              count = (int) (end - curr);
               set = true;
             }
           }
@@ -1581,20 +1669,8 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
     final SortedNumericDocValues ords = getSortedNumeric(ordsEntry);
     return new BaseSortedSetDocValues(entry, data) {
 
-      int i = 0;
-      int count = 0;
-      boolean set = false;
-
       @Override
       public long nextOrd() throws IOException {
-        if (set == false) {
-          set = true;
-          i = 0;
-          count = ords.docValueCount();
-        }
-        if (i++ == count) {
-          return NO_MORE_ORDS;
-        }
         return ords.nextValue();
       }
 
@@ -1605,7 +1681,6 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
 
       @Override
       public boolean advanceExact(int target) throws IOException {
-        set = false;
         return ords.advanceExact(target);
       }
 
@@ -1616,13 +1691,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
 
       @Override
       public int nextDoc() throws IOException {
-        set = false;
         return ords.nextDoc();
       }
 
       @Override
       public int advance(int target) throws IOException {
-        set = false;
         return ords.advance(target);
       }
 
@@ -1667,6 +1740,11 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
               ? null
               : data.randomAccessSlice(
                   entry.valueJumpTableOffset, data.length() - entry.valueJumpTableOffset);
+      if (rankSlice != null && rankSlice.length() > 0) {
+        // Prefetch the first page of data. Following pages are expected to get prefetched through
+        // read-ahead.
+        rankSlice.prefetch(0, 1);
+      }
       shift = entry.blockShift;
       mul = entry.gcd;
       mask = (1 << shift) - 1;
@@ -1706,4 +1784,115 @@ final class Lucene90DocValuesProducer extends DocValuesProducer {
       return mul * values.get(index & mask) + delta;
     }
   }
+
+  @Override
+  public DocValuesSkipper getSkipper(FieldInfo field) throws IOException {
+    final DocValuesSkipperEntry entry = skippers.get(field.name);
+
+    final IndexInput input = data.slice("doc value skipper", entry.offset, entry.length);
+    // Prefetch the first page of data. Following pages are expected to get prefetched through
+    // read-ahead.
+    if (input.length() > 0) {
+      input.prefetch(0, 1);
+    }
+    // TODO: should we write to disk the actual max level for this segment?
+    return new DocValuesSkipper() {
+      final int[] minDocID = new int[SKIP_INDEX_MAX_LEVEL];
+      final int[] maxDocID = new int[SKIP_INDEX_MAX_LEVEL];
+
+      {
+        for (int i = 0; i < SKIP_INDEX_MAX_LEVEL; i++) {
+          minDocID[i] = maxDocID[i] = -1;
+        }
+      }
+
+      final long[] minValue = new long[SKIP_INDEX_MAX_LEVEL];
+      final long[] maxValue = new long[SKIP_INDEX_MAX_LEVEL];
+      final int[] docCount = new int[SKIP_INDEX_MAX_LEVEL];
+      int levels = 1;
+
+      @Override
+      public void advance(int target) throws IOException {
+        if (target > entry.maxDocId) {
+          // skipper is exhausted
+          for (int i = 0; i < SKIP_INDEX_MAX_LEVEL; i++) {
+            minDocID[i] = maxDocID[i] = DocIdSetIterator.NO_MORE_DOCS;
+          }
+        } else {
+          // find next interval
+          assert target > maxDocID[0] : "target must be bigger that current interval";
+          while (true) {
+            levels = input.readByte();
+            assert levels <= SKIP_INDEX_MAX_LEVEL && levels > 0
+                : "level out of range [" + levels + "]";
+            boolean valid = true;
+            // check if current interval is competitive or we can jump to the next position
+            for (int level = levels - 1; level >= 0; level--) {
+              if ((maxDocID[level] = input.readInt()) < target) {
+                input.skipBytes(SKIP_INDEX_JUMP_LENGTH_PER_LEVEL[level]); // the jump for the level
+                valid = false;
+                break;
+              }
+              minDocID[level] = input.readInt();
+              maxValue[level] = input.readLong();
+              minValue[level] = input.readLong();
+              docCount[level] = input.readInt();
+            }
+            if (valid) {
+              // adjust levels
+              while (levels < SKIP_INDEX_MAX_LEVEL && maxDocID[levels] >= target) {
+                levels++;
+              }
+              break;
+            }
+          }
+        }
+      }
+
+      @Override
+      public int numLevels() {
+        return levels;
+      }
+
+      @Override
+      public int minDocID(int level) {
+        return minDocID[level];
+      }
+
+      @Override
+      public int maxDocID(int level) {
+        return maxDocID[level];
+      }
+
+      @Override
+      public long minValue(int level) {
+        return minValue[level];
+      }
+
+      @Override
+      public long maxValue(int level) {
+        return maxValue[level];
+      }
+
+      @Override
+      public int docCount(int level) {
+        return docCount[level];
+      }
+
+      @Override
+      public long minValue() {
+        return entry.minValue;
+      }
+
+      @Override
+      public long maxValue() {
+        return entry.maxValue;
+      }
+
+      @Override
+      public int docCount() {
+        return entry.docCount;
+      }
+    };
+  }
 }
