@@ -16,23 +16,21 @@
  */
 package org.apache.lucene.search;
 
-
 import java.io.IOException;
 import java.util.Arrays;
 import java.util.Collection;
-
+import java.util.Objects;
 import org.apache.lucene.util.Bits;
-import org.apache.lucene.util.FutureObjects;
 import org.apache.lucene.util.PriorityQueue;
 
 /**
- * {@link BulkScorer} that is used for pure disjunctions and disjunctions
- * that have low values of {@link BooleanQuery.Builder#setMinimumNumberShouldMatch(int)}
- * and dense clauses. This scorer scores documents by batches of 2048 docs.
+ * {@link BulkScorer} that is used for pure disjunctions and disjunctions that have low values of
+ * {@link BooleanQuery.Builder#setMinimumNumberShouldMatch(int)} and dense clauses. This scorer
+ * scores documents by batches of 2048 docs.
  */
 final class BooleanScorer extends BulkScorer {
 
-  static final int SHIFT = 11;
+  static final int SHIFT = 12;
   static final int SIZE = 1 << SHIFT;
   static final int MASK = SIZE - 1;
   static final int SET_SIZE = 1 << (SHIFT - 6);
@@ -65,12 +63,13 @@ final class BooleanScorer extends BulkScorer {
 
   // See WANDScorer for an explanation
   private static long cost(Collection<BulkScorer> scorers, int minShouldMatch) {
-    final PriorityQueue<BulkScorer> pq = new PriorityQueue<BulkScorer>(scorers.size() - minShouldMatch + 1) {
-      @Override
-      protected boolean lessThan(BulkScorer a, BulkScorer b) {
-        return a.cost() > b.cost();
-      }
-    };
+    final PriorityQueue<BulkScorer> pq =
+        new PriorityQueue<BulkScorer>(scorers.size() - minShouldMatch + 1) {
+          @Override
+          protected boolean lessThan(BulkScorer a, BulkScorer b) {
+            return a.cost() > b.cost();
+          }
+        };
     for (BulkScorer scorer : scorers) {
       pq.insertWithOverflow(scorer);
     }
@@ -91,7 +90,6 @@ final class BooleanScorer extends BulkScorer {
     protected boolean lessThan(BulkScorerAndDoc a, BulkScorerAndDoc b) {
       return a.next < b.next;
     }
-
   }
 
   static final class TailPriorityQueue extends PriorityQueue<BulkScorerAndDoc> {
@@ -106,22 +104,24 @@ final class BooleanScorer extends BulkScorer {
     }
 
     public BulkScorerAndDoc get(int i) {
-      FutureObjects.checkIndex(i, size());
+      Objects.checkIndex(i, size());
       return (BulkScorerAndDoc) getHeapArray()[1 + i];
     }
-
   }
 
-  final Bucket[] buckets = new Bucket[SIZE];
+  // One bucket per doc ID in the window, non-null if scores are needed or if frequencies need to be
+  // counted
+  final Bucket[] buckets;
   // This is basically an inlined FixedBitSet... seems to help with bound checks
   final long[] matching = new long[SET_SIZE];
 
   final BulkScorerAndDoc[] leads;
   final HeadPriorityQueue head;
   final TailPriorityQueue tail;
-  final ScoreAndDoc scoreAndDoc = new ScoreAndDoc();
+  final Score score = new Score();
   final int minShouldMatch;
   final long cost;
+  final boolean needsScores;
 
   final class OrCollector implements LeafCollector {
     Scorable scorer;
@@ -136,34 +136,87 @@ final class BooleanScorer extends BulkScorer {
       final int i = doc & MASK;
       final int idx = i >>> 6;
       matching[idx] |= 1L << i;
-      final Bucket bucket = buckets[i];
-      bucket.freq++;
-      bucket.score += scorer.score();
+      if (buckets != null) {
+        final Bucket bucket = buckets[i];
+        bucket.freq++;
+        if (needsScores) {
+          bucket.score += scorer.score();
+        }
+      }
     }
   }
 
   final OrCollector orCollector = new OrCollector();
 
-  BooleanScorer(BooleanWeight weight, Collection<BulkScorer> scorers, int minShouldMatch, boolean needsScores) {
+  final class DocIdStreamView extends DocIdStream {
+
+    int base;
+
+    @Override
+    public void forEach(CheckedIntConsumer<IOException> consumer) throws IOException {
+      long[] matching = BooleanScorer.this.matching;
+      Bucket[] buckets = BooleanScorer.this.buckets;
+      int base = this.base;
+      for (int idx = 0; idx < matching.length; idx++) {
+        long bits = matching[idx];
+        while (bits != 0L) {
+          int ntz = Long.numberOfTrailingZeros(bits);
+          if (buckets != null) {
+            final int indexInWindow = (idx << 6) | ntz;
+            final Bucket bucket = buckets[indexInWindow];
+            if (bucket.freq >= minShouldMatch) {
+              score.score = (float) bucket.score;
+              consumer.accept(base | indexInWindow);
+            }
+            bucket.freq = 0;
+            bucket.score = 0;
+          } else {
+            consumer.accept(base | (idx << 6) | ntz);
+          }
+          bits ^= 1L << ntz;
+        }
+      }
+    }
+
+    @Override
+    public int count() throws IOException {
+      if (minShouldMatch > 1) {
+        // We can't just count bits in that case
+        return super.count();
+      }
+      int count = 0;
+      for (long l : matching) {
+        count += Long.bitCount(l);
+      }
+      return count;
+    }
+  }
+
+  private final DocIdStreamView docIdStreamView = new DocIdStreamView();
+
+  BooleanScorer(Collection<BulkScorer> scorers, int minShouldMatch, boolean needsScores) {
     if (minShouldMatch < 1 || minShouldMatch > scorers.size()) {
-      throw new IllegalArgumentException("minShouldMatch should be within 1..num_scorers. Got " + minShouldMatch);
+      throw new IllegalArgumentException(
+          "minShouldMatch should be within 1..num_scorers. Got " + minShouldMatch);
     }
     if (scorers.size() <= 1) {
-      throw new IllegalArgumentException("This scorer can only be used with two scorers or more, got " + scorers.size());
+      throw new IllegalArgumentException(
+          "This scorer can only be used with two scorers or more, got " + scorers.size());
     }
-    for (int i = 0; i < buckets.length; i++) {
-      buckets[i] = new Bucket();
+    if (needsScores || minShouldMatch > 1) {
+      buckets = new Bucket[SIZE];
+      for (int i = 0; i < buckets.length; i++) {
+        buckets[i] = new Bucket();
+      }
+    } else {
+      buckets = null;
     }
     this.leads = new BulkScorerAndDoc[scorers.size()];
     this.head = new HeadPriorityQueue(scorers.size() - minShouldMatch + 1);
     this.tail = new TailPriorityQueue(minShouldMatch - 1);
     this.minShouldMatch = minShouldMatch;
+    this.needsScores = needsScores;
     for (BulkScorer scorer : scorers) {
-      if (needsScores == false) {
-        // OrCollector calls score() all the time so we have to explicitly
-        // disable scoring in order to avoid decoding useless norms
-        scorer = BooleanWeight.disableScoring(scorer);
-      }
       final BulkScorerAndDoc evicted = tail.insertWithOverflow(new BulkScorerAndDoc(scorer));
       if (evicted != null) {
         head.add(evicted);
@@ -177,41 +230,24 @@ final class BooleanScorer extends BulkScorer {
     return cost;
   }
 
-  private void scoreDocument(LeafCollector collector, int base, int i) throws IOException {
-    final ScoreAndDoc scoreAndDoc = this.scoreAndDoc;
-    final Bucket bucket = buckets[i];
-    if (bucket.freq >= minShouldMatch) {
-      scoreAndDoc.score = (float) bucket.score;
-      final int doc = base | i;
-      scoreAndDoc.doc = doc;
-      collector.collect(doc);
-    }
-    bucket.freq = 0;
-    bucket.score = 0;
-  }
-
-  private void scoreMatches(LeafCollector collector, int base) throws IOException {
-    long matching[] = this.matching;
-    for (int idx = 0; idx < matching.length; idx++) {
-      long bits = matching[idx];
-      while (bits != 0L) {
-        int ntz = Long.numberOfTrailingZeros(bits);
-        int doc = idx << 6 | ntz;
-        scoreDocument(collector, base, doc);
-        bits ^= 1L << ntz;
-      }
-    }
-  }
-
-  private void scoreWindowIntoBitSetAndReplay(LeafCollector collector, Bits acceptDocs,
-      int base, int min, int max, BulkScorerAndDoc[] scorers, int numScorers) throws IOException {
+  private void scoreWindowIntoBitSetAndReplay(
+      LeafCollector collector,
+      Bits acceptDocs,
+      int base,
+      int min,
+      int max,
+      BulkScorerAndDoc[] scorers,
+      int numScorers)
+      throws IOException {
     for (int i = 0; i < numScorers; ++i) {
       final BulkScorerAndDoc scorer = scorers[i];
       assert scorer.next < max;
       scorer.score(orCollector, acceptDocs, min, max);
     }
 
-    scoreMatches(collector, base);
+    docIdStreamView.base = base;
+    collector.collect(docIdStreamView);
+
     Arrays.fill(matching, 0L);
   }
 
@@ -236,7 +272,14 @@ final class BooleanScorer extends BulkScorer {
     return headTop;
   }
 
-  private void scoreWindowMultipleScorers(LeafCollector collector, Bits acceptDocs, int windowBase, int windowMin, int windowMax, int maxFreq) throws IOException {
+  private void scoreWindowMultipleScorers(
+      LeafCollector collector,
+      Bits acceptDocs,
+      int windowBase,
+      int windowMin,
+      int windowMax,
+      int maxFreq)
+      throws IOException {
     while (maxFreq < minShouldMatch && maxFreq + tail.size() >= minShouldMatch) {
       // a match is still possible
       final BulkScorerAndDoc candidate = tail.pop();
@@ -255,7 +298,8 @@ final class BooleanScorer extends BulkScorer {
       }
       tail.clear();
 
-      scoreWindowIntoBitSetAndReplay(collector, acceptDocs, windowBase, windowMin, windowMax, leads, maxFreq);
+      scoreWindowIntoBitSetAndReplay(
+          collector, acceptDocs, windowBase, windowMin, windowMax, leads, maxFreq);
     }
 
     // Push back scorers into head and tail
@@ -267,8 +311,14 @@ final class BooleanScorer extends BulkScorer {
     }
   }
 
-  private void scoreWindowSingleScorer(BulkScorerAndDoc bulkScorer, LeafCollector collector,
-      Bits acceptDocs, int windowMin, int windowMax, int max) throws IOException {
+  private void scoreWindowSingleScorer(
+      BulkScorerAndDoc bulkScorer,
+      LeafCollector collector,
+      Bits acceptDocs,
+      int windowMin,
+      int windowMax,
+      int max)
+      throws IOException {
     assert tail.size() == 0;
     final int nextWindowBase = head.top().next & ~MASK;
     final int end = Math.max(windowMax, Math.min(max, nextWindowBase));
@@ -276,11 +326,12 @@ final class BooleanScorer extends BulkScorer {
     bulkScorer.score(collector, acceptDocs, windowMin, end);
 
     // reset the scorer that should be used for the general case
-    collector.setScorer(scoreAndDoc);
+    collector.setScorer(score);
   }
 
-  private BulkScorerAndDoc scoreWindow(BulkScorerAndDoc top, LeafCollector collector,
-      Bits acceptDocs, int min, int max) throws IOException {
+  private BulkScorerAndDoc scoreWindow(
+      BulkScorerAndDoc top, LeafCollector collector, Bits acceptDocs, int min, int max)
+      throws IOException {
     final int windowBase = top.next & ~MASK; // find the window that the next match belongs to
     final int windowMin = Math.max(min, windowBase);
     final int windowMax = Math.min(max, windowBase + SIZE);
@@ -307,8 +358,7 @@ final class BooleanScorer extends BulkScorer {
 
   @Override
   public int score(LeafCollector collector, Bits acceptDocs, int min, int max) throws IOException {
-    scoreAndDoc.doc = -1;
-    collector.setScorer(scoreAndDoc);
+    collector.setScorer(score);
 
     BulkScorerAndDoc top = advance(min);
     while (top.next < max) {
@@ -317,5 +367,4 @@ final class BooleanScorer extends BulkScorer {
 
     return top.next;
   }
-
 }
