@@ -25,7 +25,6 @@ import java.util.Map;
 import org.apache.lucene.index.LeafReaderContext;
 import org.apache.lucene.search.BooleanClause.Occur;
 import org.apache.lucene.search.similarities.Similarity;
-import org.apache.lucene.util.Bits;
 
 /** Expert: the Weight for BooleanQuery, used to normalize, score and explain these queries. */
 final class BooleanWeight extends Weight {
@@ -57,7 +56,7 @@ final class BooleanWeight extends Weight {
     for (BooleanClause c : query) {
       Weight w =
           searcher.createWeight(
-              c.getQuery(), c.isScoring() ? scoreMode : ScoreMode.COMPLETE_NO_SCORES, boost);
+              c.query(), c.isScoring() ? scoreMode : ScoreMode.COMPLETE_NO_SCORES, boost);
       weightedClauses.add(new WeightedBooleanClause(c, w));
     }
   }
@@ -85,20 +84,18 @@ final class BooleanWeight extends Weight {
                   e));
         } else if (c.isProhibited()) {
           subs.add(
-              Explanation.noMatch(
-                  "match on prohibited clause (" + c.getQuery().toString() + ")", e));
+              Explanation.noMatch("match on prohibited clause (" + c.query().toString() + ")", e));
           fail = true;
         }
         if (!c.isProhibited()) {
           matchCount++;
         }
-        if (c.getOccur() == Occur.SHOULD) {
+        if (c.occur() == Occur.SHOULD) {
           shouldMatchCount++;
         }
       } else if (c.isRequired()) {
         subs.add(
-            Explanation.noMatch(
-                "no match on required clause (" + c.getQuery().toString() + ")", e));
+            Explanation.noMatch("no match on required clause (" + c.query().toString() + ")", e));
         fail = true;
       }
     }
@@ -143,7 +140,7 @@ final class BooleanWeight extends Weight {
         }
         matches.add(m);
       }
-      if (bc.getOccur() == Occur.SHOULD) {
+      if (bc.occur() == Occur.SHOULD) {
         if (m != null) {
           matches.add(m);
           shouldMatchCount++;
@@ -156,43 +153,12 @@ final class BooleanWeight extends Weight {
     return MatchesUtils.fromSubMatches(matches);
   }
 
-  static BulkScorer disableScoring(final BulkScorer scorer) {
-    return new BulkScorer() {
-
-      @Override
-      public int score(final LeafCollector collector, Bits acceptDocs, int min, int max)
-          throws IOException {
-        final LeafCollector noScoreCollector =
-            new LeafCollector() {
-              ScoreAndDoc fake = new ScoreAndDoc();
-
-              @Override
-              public void setScorer(Scorable scorer) throws IOException {
-                collector.setScorer(fake);
-              }
-
-              @Override
-              public void collect(int doc) throws IOException {
-                fake.doc = doc;
-                collector.collect(doc);
-              }
-            };
-        return scorer.score(noScoreCollector, acceptDocs, min, max);
-      }
-
-      @Override
-      public long cost() {
-        return scorer.cost();
-      }
-    };
-  }
-
   // Return a BulkScorer for the optional clauses only,
   // or null if it is not applicable
   // pkg-private for forcing use of BooleanScorer in tests
   BulkScorer optionalBulkScorer(LeafReaderContext context) throws IOException {
     if (scoreMode == ScoreMode.TOP_SCORES) {
-      if (!query.isPureDisjunction() || weightedClauses.size() > 2) {
+      if (!query.isPureDisjunction()) {
         return null;
       }
 
@@ -200,7 +166,7 @@ final class BooleanWeight extends Weight {
       for (WeightedBooleanClause wc : weightedClauses) {
         Weight w = wc.weight;
         BooleanClause c = wc.clause;
-        if (c.getOccur() != Occur.SHOULD) {
+        if (c.occur() != Occur.SHOULD) {
           continue;
         }
         ScorerSupplier scorer = w.scorerSupplier(context);
@@ -218,41 +184,14 @@ final class BooleanWeight extends Weight {
         optionalScorers.add(ss.get(Long.MAX_VALUE));
       }
 
-      return new BulkScorer() {
-        final Scorer bmmScorer = new BlockMaxMaxscoreScorer(BooleanWeight.this, optionalScorers);
-        final DocIdSetIterator iterator = bmmScorer.iterator();
-
-        @Override
-        public int score(LeafCollector collector, Bits acceptDocs, int min, int max)
-            throws IOException {
-          collector.setScorer(bmmScorer);
-
-          int doc = bmmScorer.docID();
-          if (doc < min) {
-            doc = iterator.advance(min);
-          }
-          while (doc < max) {
-            if (acceptDocs == null || acceptDocs.get(doc)) {
-              collector.collect(doc);
-            }
-
-            doc = iterator.nextDoc();
-          }
-          return doc;
-        }
-
-        @Override
-        public long cost() {
-          return iterator.cost();
-        }
-      };
+      return new MaxScoreBulkScorer(context.reader().maxDoc(), optionalScorers);
     }
 
     List<BulkScorer> optional = new ArrayList<BulkScorer>();
     for (WeightedBooleanClause wc : weightedClauses) {
       Weight w = wc.weight;
       BooleanClause c = wc.clause;
-      if (c.getOccur() != Occur.SHOULD) {
+      if (c.occur() != Occur.SHOULD) {
         continue;
       }
       BulkScorer subScorer = w.bulkScorer(context);
@@ -275,125 +214,7 @@ final class BooleanWeight extends Weight {
     }
 
     return new BooleanScorer(
-        this, optional, Math.max(1, query.getMinimumNumberShouldMatch()), scoreMode.needsScores());
-  }
-
-  // Return a BulkScorer for the required clauses only,
-  // or null if it is not applicable
-  private BulkScorer requiredBulkScorer(LeafReaderContext context) throws IOException {
-    BulkScorer scorer = null;
-
-    for (WeightedBooleanClause wc : weightedClauses) {
-      Weight w = wc.weight;
-      BooleanClause c = wc.clause;
-      if (c.isRequired() == false) {
-        continue;
-      }
-      if (scorer != null) {
-        // we don't have a BulkScorer for conjunctions
-        return null;
-      }
-      scorer = w.bulkScorer(context);
-      if (scorer == null) {
-        // no matches
-        return null;
-      }
-      if (c.isScoring() == false && scoreMode.needsScores()) {
-        scorer = disableScoring(scorer);
-      }
-    }
-    return scorer;
-  }
-
-  /**
-   * Try to build a boolean scorer for this weight. Returns null if {@link BooleanScorer} cannot be
-   * used.
-   */
-  BulkScorer booleanScorer(LeafReaderContext context) throws IOException {
-    final int numOptionalClauses = query.getClauses(Occur.SHOULD).size();
-    final int numRequiredClauses =
-        query.getClauses(Occur.MUST).size() + query.getClauses(Occur.FILTER).size();
-
-    BulkScorer positiveScorer;
-    if (numRequiredClauses == 0) {
-      positiveScorer = optionalBulkScorer(context);
-      if (positiveScorer == null) {
-        return null;
-      }
-
-      // TODO: what is the right heuristic here?
-      final long costThreshold;
-      if (query.getMinimumNumberShouldMatch() <= 1) {
-        // when all clauses are optional, use BooleanScorer aggressively
-        // TODO: is there actually a threshold under which we should rather
-        // use the regular scorer?
-        costThreshold = -1;
-      } else {
-        // when a minimum number of clauses should match, BooleanScorer is
-        // going to score all windows that have at least minNrShouldMatch
-        // matches in the window. But there is no way to know if there is
-        // an intersection (all clauses might match a different doc ID and
-        // there will be no matches in the end) so we should only use
-        // BooleanScorer if matches are very dense
-        costThreshold = context.reader().maxDoc() / 3;
-      }
-
-      if (positiveScorer.cost() < costThreshold) {
-        return null;
-      }
-
-    } else if (numRequiredClauses == 1
-        && numOptionalClauses == 0
-        && query.getMinimumNumberShouldMatch() == 0) {
-      positiveScorer = requiredBulkScorer(context);
-    } else {
-      // TODO: there are some cases where BooleanScorer
-      // would handle conjunctions faster than
-      // BooleanScorer2...
-      return null;
-    }
-
-    if (positiveScorer == null) {
-      return null;
-    }
-
-    List<Scorer> prohibited = new ArrayList<>();
-    for (WeightedBooleanClause wc : weightedClauses) {
-      Weight w = wc.weight;
-      BooleanClause c = wc.clause;
-      if (c.isProhibited()) {
-        Scorer scorer = w.scorer(context);
-        if (scorer != null) {
-          prohibited.add(scorer);
-        }
-      }
-    }
-
-    if (prohibited.isEmpty()) {
-      return positiveScorer;
-    } else {
-      Scorer prohibitedScorer =
-          prohibited.size() == 1
-              ? prohibited.get(0)
-              : new DisjunctionSumScorer(this, prohibited, ScoreMode.COMPLETE_NO_SCORES);
-      if (prohibitedScorer.twoPhaseIterator() != null) {
-        // ReqExclBulkScorer can't deal efficiently with two-phased prohibited clauses
-        return null;
-      }
-      return new ReqExclBulkScorer(positiveScorer, prohibitedScorer.iterator());
-    }
-  }
-
-  @Override
-  public BulkScorer bulkScorer(LeafReaderContext context) throws IOException {
-    final BulkScorer bulkScorer = booleanScorer(context);
-    if (bulkScorer != null) {
-      // bulk scoring is applicable, use it
-      return bulkScorer;
-    } else {
-      // use a Scorer-based impl (BS2)
-      return super.bulkScorer(context);
-    }
+        optional, Math.max(1, query.getMinimumNumberShouldMatch()), scoreMode.needsScores());
   }
 
   @Override
@@ -472,7 +293,7 @@ final class BooleanWeight extends Weight {
     int optCount = 0;
     boolean unknownCount = false;
     for (WeightedBooleanClause weightedClause : weightedClauses) {
-      if (weightedClause.clause.getOccur() != occur) {
+      if (weightedClause.clause.occur() != occur) {
         continue;
       }
       int count = weightedClause.weight.count(context);
@@ -502,15 +323,6 @@ final class BooleanWeight extends Weight {
     return unknownCount ? -1 : optCount;
   }
 
-  @Override
-  public Scorer scorer(LeafReaderContext context) throws IOException {
-    ScorerSupplier scorerSupplier = scorerSupplier(context);
-    if (scorerSupplier == null) {
-      return null;
-    }
-    return scorerSupplier.get(Long.MAX_VALUE);
-  }
-
   @Override
   public boolean isCacheable(LeafReaderContext ctx) {
     if (query.clauses().size()
@@ -545,7 +357,7 @@ final class BooleanWeight extends Weight {
           return null;
         }
       } else {
-        scorers.get(c.getOccur()).add(subScorer);
+        scorers.get(c.occur()).add(subScorer);
       }
     }
 
@@ -577,6 +389,7 @@ final class BooleanWeight extends Weight {
       scorers.get(Occur.SHOULD).clear();
     }
 
-    return new Boolean2ScorerSupplier(this, scorers, scoreMode, minShouldMatch);
+    return new BooleanScorerSupplier(
+        this, scorers, scoreMode, minShouldMatch, context.reader().maxDoc());
   }
 }
