@@ -16,6 +16,8 @@
  */
 package org.apache.lucene.analysis;
 
+import static org.apache.lucene.util.automaton.Operations.DEFAULT_DETERMINIZE_WORK_LIMIT;
+
 import java.io.IOException;
 import java.io.PrintWriter;
 import java.io.StringWriter;
@@ -25,21 +27,25 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Random;
 import java.util.Set;
-
 import org.apache.lucene.analysis.tokenattributes.CharTermAttribute;
 import org.apache.lucene.analysis.tokenattributes.OffsetAttribute;
 import org.apache.lucene.analysis.tokenattributes.PositionIncrementAttribute;
 import org.apache.lucene.analysis.tokenattributes.PositionLengthAttribute;
+import org.apache.lucene.tests.analysis.BaseTokenStreamTestCase;
+import org.apache.lucene.tests.analysis.CannedTokenStream;
+import org.apache.lucene.tests.analysis.MockGraphTokenFilter;
+import org.apache.lucene.tests.analysis.MockHoleInjectingTokenFilter;
+import org.apache.lucene.tests.analysis.MockTokenizer;
+import org.apache.lucene.tests.analysis.Token;
+import org.apache.lucene.tests.analysis.TokenStreamToDot;
+import org.apache.lucene.tests.util.automaton.AutomatonTestUtil;
 import org.apache.lucene.util.BytesRefBuilder;
 import org.apache.lucene.util.IntsRef;
 import org.apache.lucene.util.automaton.Automata;
 import org.apache.lucene.util.automaton.Automaton;
-import org.apache.lucene.util.automaton.AutomatonTestUtil;
 import org.apache.lucene.util.automaton.Operations;
 import org.apache.lucene.util.fst.Util;
 
-import static org.apache.lucene.util.automaton.Operations.DEFAULT_DETERMINIZE_WORK_LIMIT;
-
 public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
   // Makes a graph TokenStream from the string; separate
@@ -51,7 +57,7 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   // So, offsets are computed based on the first token at a
   // given position.  NOTE: each token must be a single
   // character!  We assume this when computing offsets...
-  
+
   // NOTE: all input tokens must be length 1!!!  This means
   // you cannot turn on MockCharFilter when random
   // testing...
@@ -63,8 +69,10 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
     private final CharTermAttribute termAtt = addAttribute(CharTermAttribute.class);
     private final OffsetAttribute offsetAtt = addAttribute(OffsetAttribute.class);
-    private final PositionIncrementAttribute posIncrAtt = addAttribute(PositionIncrementAttribute.class);
-    private final PositionLengthAttribute posLengthAtt = addAttribute(PositionLengthAttribute.class);
+    private final PositionIncrementAttribute posIncrAtt =
+        addAttribute(PositionIncrementAttribute.class);
+    private final PositionLengthAttribute posLengthAtt =
+        addAttribute(PositionLengthAttribute.class);
 
     @Override
     public void reset() throws IOException {
@@ -78,13 +86,13 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
       if (tokens == null) {
         fillTokens();
       }
-      //System.out.println("graphTokenizer: incr upto=" + upto + " vs " + tokens.size());
+      // System.out.println("graphTokenizer: incr upto=" + upto + " vs " + tokens.size());
       if (upto == tokens.size()) {
-        //System.out.println("  END @ " + tokens.size());
+        // System.out.println("  END @ " + tokens.size());
         return false;
-      } 
+      }
       final Token t = tokens.get(upto++);
-      //System.out.println("  return token=" + t);
+      // System.out.println("  return token=" + t);
       clearAttributes();
       termAtt.append(t.toString());
       offsetAtt.setOffset(t.startOffset(), t.endOffset());
@@ -100,12 +108,11 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
       // satisfy BTSTC:
       final int lastOffset;
       if (tokens != null && !tokens.isEmpty()) {
-        lastOffset = tokens.get(tokens.size()-1).endOffset();
+        lastOffset = tokens.get(tokens.size() - 1).endOffset();
       } else {
         lastOffset = 0;
       }
-      offsetAtt.setOffset(correctOffset(lastOffset),
-                          correctOffset(inputLength));
+      offsetAtt.setOffset(correctOffset(lastOffset), correctOffset(inputLength));
     }
 
     private void fillTokens() throws IOException {
@@ -117,9 +124,9 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
           break;
         }
         sb.append(buffer, 0, count);
-        //System.out.println("got count=" + count);
+        // System.out.println("got count=" + count);
       }
-      //System.out.println("fillTokens: " + sb);
+      // System.out.println("fillTokens: " + sb);
 
       inputLength = sb.length();
 
@@ -129,41 +136,42 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
       int pos = 0;
       int maxPos = -1;
       int offset = 0;
-      //System.out.println("again");
-      for(String part : parts) {
+      // System.out.println("again");
+      for (String part : parts) {
         final String[] overlapped = part.split("/");
         boolean firstAtPos = true;
         int minPosLength = Integer.MAX_VALUE;
-        for(String part2 : overlapped) {
+        for (String part2 : overlapped) {
           final int colonIndex = part2.indexOf(':');
           final String token;
           final int posLength;
           if (colonIndex != -1) {
             token = part2.substring(0, colonIndex);
-            posLength = Integer.parseInt(part2.substring(1+colonIndex));
+            posLength = Integer.parseInt(part2.substring(1 + colonIndex));
           } else {
             token = part2;
             posLength = 1;
           }
           maxPos = Math.max(maxPos, pos + posLength);
           minPosLength = Math.min(minPosLength, posLength);
-          final Token t = new Token(token, offset, offset + 2*posLength - 1);
+          final Token t = new Token(token, offset, offset + 2 * posLength - 1);
           t.setPositionLength(posLength);
-          t.setPositionIncrement(firstAtPos ? 1:0);
+          t.setPositionIncrement(firstAtPos ? 1 : 0);
           firstAtPos = false;
-          //System.out.println("  add token=" + t + " startOff=" + t.startOffset() + " endOff=" + t.endOffset());
+          // System.out.println("  add token=" + t + " startOff=" + t.startOffset() + " endOff=" +
+          // t.endOffset());
           tokens.add(t);
         }
         pos += minPosLength;
         offset = 2 * pos;
       }
-      assert maxPos <= pos: "input string mal-formed: posLength>1 tokens hang over the end";
+      assert maxPos <= pos : "input string mal-formed: posLength>1 tokens hang over the end";
     }
   }
 
   public void testMockGraphTokenFilterBasic() throws Exception {
 
-    for(int iter=0;iter<10*RANDOM_MULTIPLIER;iter++) {
+    for (int iter = 0; iter < 10 * RANDOM_MULTIPLIER; iter++) {
 
       if (VERBOSE) {
         System.out.println("\nTEST: iter=" + iter);
@@ -171,21 +179,22 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
       // Make new analyzer each time, because MGTF has fixed
       // seed:
-      final Analyzer a = new Analyzer() {
-          @Override
-          protected TokenStreamComponents createComponents(String fieldName) {
-            final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
-            final TokenStream t2 = new MockGraphTokenFilter(random(), t);
-            return new TokenStreamComponents(t, t2);
-          }
-        };
-      
+      final Analyzer a =
+          new Analyzer() {
+            @Override
+            protected TokenStreamComponents createComponents(String fieldName) {
+              final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
+              final TokenStream t2 = new MockGraphTokenFilter(random(), t);
+              return new TokenStreamComponents(t, t2);
+            }
+          };
+
       checkAnalysisConsistency(random(), a, false, "a b c d e f g h i j k");
     }
   }
 
   public void testMockGraphTokenFilterOnGraphInput() throws Exception {
-    for(int iter=0;iter<100*RANDOM_MULTIPLIER;iter++) {
+    for (int iter = 0; iter < 100 * RANDOM_MULTIPLIER; iter++) {
 
       if (VERBOSE) {
         System.out.println("\nTEST: iter=" + iter);
@@ -193,25 +202,27 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
       // Make new analyzer each time, because MGTF has fixed
       // seed:
-      final Analyzer a = new Analyzer() {
-          @Override
-          protected TokenStreamComponents createComponents(String fieldName) {
-            final Tokenizer t = new GraphTokenizer();
-            final TokenStream t2 = new MockGraphTokenFilter(random(), t);
-            return new TokenStreamComponents(t, t2);
-          }
-        };
-      
+      final Analyzer a =
+          new Analyzer() {
+            @Override
+            protected TokenStreamComponents createComponents(String fieldName) {
+              final Tokenizer t = new GraphTokenizer();
+              final TokenStream t2 = new MockGraphTokenFilter(random(), t);
+              return new TokenStreamComponents(t, t2);
+            }
+          };
+
       checkAnalysisConsistency(random(), a, false, "a/x:3 c/y:2 d e f/z:4 g h i j k");
     }
   }
 
   // Just deletes (leaving hole) token 'a':
-  private final static class RemoveATokens extends TokenFilter {
+  private static final class RemoveATokens extends TokenFilter {
     private int pendingPosInc;
 
     private final CharTermAttribute termAtt = addAttribute(CharTermAttribute.class);
-    private final PositionIncrementAttribute posIncAtt = addAttribute(PositionIncrementAttribute.class);
+    private final PositionIncrementAttribute posIncAtt =
+        addAttribute(PositionIncrementAttribute.class);
 
     public RemoveATokens(TokenStream in) {
       super(in);
@@ -247,7 +258,7 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   }
 
   public void testMockGraphTokenFilterBeforeHoles() throws Exception {
-    for(int iter=0;iter<100*RANDOM_MULTIPLIER;iter++) {
+    for (int iter = 0; iter < 100 * RANDOM_MULTIPLIER; iter++) {
 
       if (VERBOSE) {
         System.out.println("\nTEST: iter=" + iter);
@@ -255,15 +266,16 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
       // Make new analyzer each time, because MGTF has fixed
       // seed:
-      final Analyzer a = new Analyzer() {
-          @Override
-          protected TokenStreamComponents createComponents(String fieldName) {
-            final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
-            final TokenStream t2 = new MockGraphTokenFilter(random(), t);
-            final TokenStream t3 = new RemoveATokens(t2);
-            return new TokenStreamComponents(t, t3);
-          }
-        };
+      final Analyzer a =
+          new Analyzer() {
+            @Override
+            protected TokenStreamComponents createComponents(String fieldName) {
+              final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
+              final TokenStream t2 = new MockGraphTokenFilter(random(), t);
+              final TokenStream t3 = new RemoveATokens(t2);
+              return new TokenStreamComponents(t, t3);
+            }
+          };
 
       Random random = random();
       checkAnalysisConsistency(random, a, false, "a b c d e f g h i j k");
@@ -274,7 +286,7 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   }
 
   public void testMockGraphTokenFilterAfterHoles() throws Exception {
-    for(int iter=0;iter<100*RANDOM_MULTIPLIER;iter++) {
+    for (int iter = 0; iter < 100 * RANDOM_MULTIPLIER; iter++) {
 
       if (VERBOSE) {
         System.out.println("\nTEST: iter=" + iter);
@@ -282,15 +294,16 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
       // Make new analyzer each time, because MGTF has fixed
       // seed:
-      final Analyzer a = new Analyzer() {
-          @Override
-          protected TokenStreamComponents createComponents(String fieldName) {
-            final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
-            final TokenStream t2 = new RemoveATokens(t);
-            final TokenStream t3 = new MockGraphTokenFilter(random(), t2);
-            return new TokenStreamComponents(t, t3);
-          }
-        };
+      final Analyzer a =
+          new Analyzer() {
+            @Override
+            protected TokenStreamComponents createComponents(String fieldName) {
+              final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
+              final TokenStream t2 = new RemoveATokens(t);
+              final TokenStream t3 = new MockGraphTokenFilter(random(), t2);
+              return new TokenStreamComponents(t, t3);
+            }
+          };
 
       Random random = random();
       checkAnalysisConsistency(random, a, false, "a b c d e f g h i j k");
@@ -301,7 +314,7 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   }
 
   public void testMockGraphTokenFilterRandom() throws Exception {
-    for(int iter=0;iter<3*RANDOM_MULTIPLIER;iter++) {
+    for (int iter = 0; iter < 3 * RANDOM_MULTIPLIER; iter++) {
 
       if (VERBOSE) {
         System.out.println("\nTEST: iter=" + iter);
@@ -309,15 +322,16 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
       // Make new analyzer each time, because MGTF has fixed
       // seed:
-      final Analyzer a = new Analyzer() {
-          @Override
-          protected TokenStreamComponents createComponents(String fieldName) {
-            final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
-            final TokenStream t2 = new MockGraphTokenFilter(random(), t);
-            return new TokenStreamComponents(t, t2);
-          }
-        };
-      
+      final Analyzer a =
+          new Analyzer() {
+            @Override
+            protected TokenStreamComponents createComponents(String fieldName) {
+              final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
+              final TokenStream t2 = new MockGraphTokenFilter(random(), t);
+              return new TokenStreamComponents(t, t2);
+            }
+          };
+
       Random random = random();
       checkRandomData(random, a, 5, atLeast(100));
     }
@@ -325,7 +339,7 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
   // Two MockGraphTokenFilters
   public void testDoubleMockGraphTokenFilterRandom() throws Exception {
-    for(int iter=0;iter<3*RANDOM_MULTIPLIER;iter++) {
+    for (int iter = 0; iter < 3 * RANDOM_MULTIPLIER; iter++) {
 
       if (VERBOSE) {
         System.out.println("\nTEST: iter=" + iter);
@@ -333,23 +347,24 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
       // Make new analyzer each time, because MGTF has fixed
       // seed:
-      final Analyzer a = new Analyzer() {
-          @Override
-          protected TokenStreamComponents createComponents(String fieldName) {
-            final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
-            final TokenStream t1 = new MockGraphTokenFilter(random(), t);
-            final TokenStream t2 = new MockGraphTokenFilter(random(), t1);
-            return new TokenStreamComponents(t, t2);
-          }
-        };
-      
+      final Analyzer a =
+          new Analyzer() {
+            @Override
+            protected TokenStreamComponents createComponents(String fieldName) {
+              final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
+              final TokenStream t1 = new MockGraphTokenFilter(random(), t);
+              final TokenStream t2 = new MockGraphTokenFilter(random(), t1);
+              return new TokenStreamComponents(t, t2);
+            }
+          };
+
       Random random = random();
       checkRandomData(random, a, 5, atLeast(100));
     }
   }
 
   public void testMockGraphTokenFilterBeforeHolesRandom() throws Exception {
-    for(int iter=0;iter<3*RANDOM_MULTIPLIER;iter++) {
+    for (int iter = 0; iter < 3 * RANDOM_MULTIPLIER; iter++) {
 
       if (VERBOSE) {
         System.out.println("\nTEST: iter=" + iter);
@@ -357,23 +372,24 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
       // Make new analyzer each time, because MGTF has fixed
       // seed:
-      final Analyzer a = new Analyzer() {
-          @Override
-          protected TokenStreamComponents createComponents(String fieldName) {
-            final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
-            final TokenStream t1 = new MockGraphTokenFilter(random(), t);
-            final TokenStream t2 = new MockHoleInjectingTokenFilter(random(), t1);
-            return new TokenStreamComponents(t, t2);
-          }
-        };
-      
+      final Analyzer a =
+          new Analyzer() {
+            @Override
+            protected TokenStreamComponents createComponents(String fieldName) {
+              final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
+              final TokenStream t1 = new MockGraphTokenFilter(random(), t);
+              final TokenStream t2 = new MockHoleInjectingTokenFilter(random(), t1);
+              return new TokenStreamComponents(t, t2);
+            }
+          };
+
       Random random = random();
       checkRandomData(random, a, 5, atLeast(100));
     }
   }
 
   public void testMockGraphTokenFilterAfterHolesRandom() throws Exception {
-    for(int iter=0;iter<3*RANDOM_MULTIPLIER;iter++) {
+    for (int iter = 0; iter < 3 * RANDOM_MULTIPLIER; iter++) {
 
       if (VERBOSE) {
         System.out.println("\nTEST: iter=" + iter);
@@ -381,16 +397,17 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
       // Make new analyzer each time, because MGTF has fixed
       // seed:
-      final Analyzer a = new Analyzer() {
-          @Override
-          protected TokenStreamComponents createComponents(String fieldName) {
-            final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
-            final TokenStream t1 = new MockHoleInjectingTokenFilter(random(), t);
-            final TokenStream t2 = new MockGraphTokenFilter(random(), t1);
-            return new TokenStreamComponents(t, t2);
-          }
-        };
-      
+      final Analyzer a =
+          new Analyzer() {
+            @Override
+            protected TokenStreamComponents createComponents(String fieldName) {
+              final Tokenizer t = new MockTokenizer(MockTokenizer.WHITESPACE, false);
+              final TokenStream t1 = new MockHoleInjectingTokenFilter(random(), t);
+              final TokenStream t2 = new MockGraphTokenFilter(random(), t1);
+              return new TokenStreamComponents(t, t2);
+            }
+          };
+
       Random random = random();
       checkRandomData(random, a, 5, atLeast(100));
     }
@@ -403,7 +420,8 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
     return t;
   }
 
-  private static Token token(String term, int posInc, int posLength, int startOffset, int endOffset) {
+  private static Token token(
+      String term, int posInc, int posLength, int startOffset, int endOffset) {
     final Token t = new Token(term, startOffset, endOffset);
     t.setPositionIncrement(posInc);
     t.setPositionLength(posLength);
@@ -411,31 +429,31 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   }
 
   public void testSingleToken() throws Exception {
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("abc", 1, 1),
-      });
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("abc", 1, 1),
+            });
     assertSameLanguage(s2a("abc"), ts);
   }
 
   public void testMultipleHoles() throws Exception {
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("a", 1, 1),
-        token("b", 3, 1),
-      });
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("a", 1, 1), token("b", 3, 1),
+            });
     assertSameLanguage(join(s2a("a"), SEP_A, HOLE_A, SEP_A, HOLE_A, SEP_A, s2a("b")), ts);
   }
 
   public void testSynOverMultipleHoles() throws Exception {
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("a", 1, 1),
-        token("x", 0, 3),
-        token("b", 3, 1),
-      });
-    final Automaton a1 = join(s2a("a"), SEP_A, HOLE_A, SEP_A, HOLE_A, SEP_A, s2a("b")); 
-    final Automaton a2 = join(s2a("x"), SEP_A, s2a("b")); 
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("a", 1, 1), token("x", 0, 3), token("b", 3, 1),
+            });
+    final Automaton a1 = join(s2a("a"), SEP_A, HOLE_A, SEP_A, HOLE_A, SEP_A, s2a("b"));
+    final Automaton a2 = join(s2a("x"), SEP_A, s2a("b"));
     assertSameLanguage(Operations.union(a1, a2), ts);
   }
 
@@ -453,17 +471,17 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   private static final Automaton SEP_A = Automata.makeChar(TokenStreamToAutomaton.POS_SEP);
   private static final Automaton HOLE_A = Automata.makeChar(TokenStreamToAutomaton.HOLE);
 
-  private Automaton join(String ... strings) {
+  private Automaton join(String... strings) {
     List<Automaton> as = new ArrayList<>();
-    for(String s : strings) {
+    for (String s : strings) {
       as.add(s2a(s));
       as.add(SEP_A);
     }
-    as.remove(as.size()-1);
+    as.remove(as.size() - 1);
     return Operations.concatenate(as);
   }
 
-  private Automaton join(Automaton ... as) {
+  private Automaton join(Automaton... as) {
     return Operations.concatenate(Arrays.asList(as));
   }
 
@@ -472,32 +490,29 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   }
 
   public void testTwoTokens() throws Exception {
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("abc", 1, 1),
-        token("def", 1, 1),
-      });
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("abc", 1, 1), token("def", 1, 1),
+            });
     assertSameLanguage(join("abc", "def"), ts);
   }
 
   public void testHole() throws Exception {
 
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("abc", 1, 1),
-        token("def", 2, 1),
-      });
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("abc", 1, 1), token("def", 2, 1),
+            });
     assertSameLanguage(join(s2a("abc"), SEP_A, HOLE_A, SEP_A, s2a("def")), ts);
   }
 
   public void testOverlappedTokensSausage() throws Exception {
 
     // Two tokens on top of each other (sausage):
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("abc", 1, 1),
-        token("xyz", 0, 1)
-      });
+    final TokenStream ts =
+        new CannedTokenStream(new Token[] {token("abc", 1, 1), token("xyz", 0, 1)});
     final Automaton a1 = s2a("abc");
     final Automaton a2 = s2a("xyz");
     assertSameLanguage(Operations.union(a1, a2), ts);
@@ -505,12 +520,11 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
   public void testOverlappedTokensLattice() throws Exception {
 
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("abc", 1, 1),
-        token("xyz", 0, 2),
-        token("def", 1, 1),
-      });
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("abc", 1, 1), token("xyz", 0, 2), token("def", 1, 1),
+            });
     final Automaton a1 = s2a("xyz");
     final Automaton a2 = join("abc", "def");
     assertSameLanguage(Operations.union(a1, a2), ts);
@@ -518,12 +532,11 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
   public void testSynOverHole() throws Exception {
 
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("a", 1, 1),
-        token("X", 0, 2),
-        token("b", 2, 1),
-      });
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("a", 1, 1), token("X", 0, 2), token("b", 2, 1),
+            });
     final Automaton a1 = Operations.union(join(s2a("a"), SEP_A, HOLE_A), s2a("X"));
     final Automaton expected = Operations.concatenate(a1, join(SEP_A, s2a("b")));
     assertSameLanguage(expected, ts);
@@ -531,26 +544,23 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
 
   public void testSynOverHole2() throws Exception {
 
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("xyz", 1, 1),
-        token("abc", 0, 3),
-        token("def", 2, 1),
-      });
-    final Automaton expected = Operations.union(
-      join(s2a("xyz"), SEP_A, HOLE_A, SEP_A, s2a("def")), s2a("abc"));
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("xyz", 1, 1), token("abc", 0, 3), token("def", 2, 1),
+            });
+    final Automaton expected =
+        Operations.union(join(s2a("xyz"), SEP_A, HOLE_A, SEP_A, s2a("def")), s2a("abc"));
     assertSameLanguage(expected, ts);
   }
 
   public void testOverlappedTokensLattice2() throws Exception {
 
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("abc", 1, 1),
-        token("xyz", 0, 3),
-        token("def", 1, 1),
-        token("ghi", 1, 1),
-      });
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("abc", 1, 1), token("xyz", 0, 3), token("def", 1, 1), token("ghi", 1, 1),
+            });
     final Automaton a1 = s2a("xyz");
     final Automaton a2 = join("abc", "def", "ghi");
     assertSameLanguage(Operations.union(a1, a2), ts);
@@ -564,27 +574,31 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   }
 
   public void testStartsWithHole() throws Exception {
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("abc", 2, 1),
-      });
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("abc", 2, 1),
+            });
     assertSameLanguage(join(HOLE_A, SEP_A, s2a("abc")), ts);
   }
 
   public void testEndsWithHole() throws Exception {
-    final TokenStream ts = new CannedTokenStream(1, 0,
-                                                 new Token[] {
-                                                   token("abc", 2, 1),
-                                                 });
+    final TokenStream ts =
+        new CannedTokenStream(
+            1,
+            0,
+            new Token[] {
+              token("abc", 2, 1),
+            });
     assertSameLanguage(join(HOLE_A, SEP_A, s2a("abc"), SEP_A, HOLE_A), ts);
   }
 
   public void testSynHangingOverEnd() throws Exception {
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("a", 1, 1),
-        token("X", 0, 10),
-      });
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("a", 1, 1), token("X", 0, 10),
+            });
     assertSameLanguage(Operations.union(s2a("a"), s2a("X")), ts);
   }
 
@@ -592,8 +606,11 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   private Set<String> toPathStrings(Automaton a) {
     BytesRefBuilder scratchBytesRefBuilder = new BytesRefBuilder();
     Set<String> paths = new HashSet<>();
-    for (IntsRef ir: AutomatonTestUtil.getFiniteStringsRecursive(a, -1)) {
-      paths.add(Util.toBytesRef(ir, scratchBytesRefBuilder).utf8ToString().replace((char) TokenStreamToAutomaton.POS_SEP, ' '));
+    for (IntsRef ir : AutomatonTestUtil.getFiniteStringsRecursive(a, -1)) {
+      paths.add(
+          Util.toBytesRef(ir, scratchBytesRefBuilder)
+              .utf8ToString()
+              .replace((char) TokenStreamToAutomaton.POS_SEP, ' '));
     }
     return paths;
   }
@@ -603,14 +620,17 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   }
 
   private void assertSameLanguage(Automaton expected, Automaton actual) {
-    Automaton expectedDet = Operations.determinize(Operations.removeDeadStates(expected), DEFAULT_DETERMINIZE_WORK_LIMIT);
-    Automaton actualDet = Operations.determinize(Operations.removeDeadStates(actual), DEFAULT_DETERMINIZE_WORK_LIMIT);
-    if (Operations.sameLanguage(expectedDet, actualDet) == false) {
+    Automaton expectedDet =
+        Operations.determinize(
+            Operations.removeDeadStates(expected), DEFAULT_DETERMINIZE_WORK_LIMIT);
+    Automaton actualDet =
+        Operations.determinize(Operations.removeDeadStates(actual), DEFAULT_DETERMINIZE_WORK_LIMIT);
+    if (AutomatonTestUtil.sameLanguage(expectedDet, actualDet) == false) {
       Set<String> expectedPaths = toPathStrings(expectedDet);
       Set<String> actualPaths = toPathStrings(actualDet);
       StringBuilder b = new StringBuilder();
       b.append("expected:\n");
-      for(String path : expectedPaths) {
+      for (String path : expectedPaths) {
         b.append("  ");
         b.append(path);
         if (actualPaths.contains(path) == false) {
@@ -619,7 +639,7 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
         b.append('\n');
       }
       b.append("actual:\n");
-      for(String path : actualPaths) {
+      for (String path : actualPaths) {
         b.append("  ");
         b.append(path);
         if (expectedPaths.contains(path) == false) {
@@ -632,14 +652,15 @@ public class TestGraphTokenizers extends BaseTokenStreamTestCase {
   }
 
   public void testTokenStreamGraphWithHoles() throws Exception {
-    final TokenStream ts = new CannedTokenStream(
-      new Token[] {
-        token("abc", 1, 1),
-        token("xyz", 1, 8),
-        token("def", 1, 1),
-        token("ghi", 1, 1),
-      });
-    assertSameLanguage(Operations.union(join(s2a("abc"), SEP_A, s2a("xyz")),
-                                        join(s2a("abc"), SEP_A, HOLE_A, SEP_A, s2a("def"), SEP_A, s2a("ghi"))), ts);
+    final TokenStream ts =
+        new CannedTokenStream(
+            new Token[] {
+              token("abc", 1, 1), token("xyz", 1, 8), token("def", 1, 1), token("ghi", 1, 1),
+            });
+    assertSameLanguage(
+        Operations.union(
+            join(s2a("abc"), SEP_A, s2a("xyz")),
+            join(s2a("abc"), SEP_A, HOLE_A, SEP_A, s2a("def"), SEP_A, s2a("ghi"))),
+        ts);
   }
 }
