@@ -18,8 +18,7 @@ package org.apache.lucene.codecs.lucene90;
 
 import com.carrotsearch.randomizedtesting.generators.RandomPicks;
 import org.apache.lucene.codecs.Codec;
-import org.apache.lucene.codecs.lucene95.Lucene95Codec;
-import org.apache.lucene.codecs.lucene95.Lucene95Codec.Mode;
+import org.apache.lucene.codecs.lucene100.Lucene100Codec;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.StoredField;
 import org.apache.lucene.index.DirectoryReader;
@@ -32,7 +31,7 @@ import org.apache.lucene.tests.index.BaseStoredFieldsFormatTestCase;
 public class TestLucene90StoredFieldsFormatHighCompression extends BaseStoredFieldsFormatTestCase {
   @Override
   protected Codec getCodec() {
-    return new Lucene95Codec(Mode.BEST_COMPRESSION);
+    return new Lucene100Codec(Lucene100Codec.Mode.BEST_COMPRESSION);
   }
 
   /**
@@ -43,7 +42,7 @@ public class TestLucene90StoredFieldsFormatHighCompression extends BaseStoredFie
     for (int i = 0; i < 10; i++) {
       IndexWriterConfig iwc = newIndexWriterConfig();
       iwc.setCodec(
-          new Lucene95Codec(RandomPicks.randomFrom(random(), Lucene95Codec.Mode.values())));
+          new Lucene100Codec(RandomPicks.randomFrom(random(), Lucene100Codec.Mode.values())));
       IndexWriter iw = new IndexWriter(dir, newIndexWriterConfig());
       Document doc = new Document();
       doc.add(new StoredField("field1", "value1"));
@@ -73,7 +72,7 @@ public class TestLucene90StoredFieldsFormatHighCompression extends BaseStoredFie
     expectThrows(
         NullPointerException.class,
         () -> {
-          new Lucene95Codec(null);
+          new Lucene100Codec(null);
         });
 
     expectThrows(
@@ -82,8 +81,4 @@ public class TestLucene90StoredFieldsFormatHighCompression extends BaseStoredFie
           new Lucene90StoredFieldsFormat(null);
         });
   }
-
-  public void testShowJDKBugStatus() {
-    System.err.println("JDK is buggy (JDK-8252739): " + BugfixDeflater_JDK8252739.IS_BUGGY_JDK);
-  }
 }
