@@ -106,6 +106,7 @@ public class TestCodecs extends LuceneTestCase {
                     storePayloads,
                     indexOptions,
                     DocValuesType.NONE,
+                    DocValuesSkipIndexType.NONE,
                     -1,
                     new HashMap<>(),
                     0,
@@ -114,6 +115,7 @@ public class TestCodecs extends LuceneTestCase {
                     0,
                     VectorEncoding.FLOAT32,
                     VectorSimilarityFunction.EUCLIDEAN,
+                    false,
                     false));
       }
       this.terms = terms;
@@ -230,7 +232,7 @@ public class TestCodecs extends LuceneTestCase {
     }
 
     final FieldInfos.Builder builder =
-        new FieldInfos.Builder(new FieldInfos.FieldNumbers(null, Version.LATEST.major));
+        new FieldInfos.Builder(new FieldInfos.FieldNumbers(null, null));
 
     final FieldData field = new FieldData("field", builder, terms, true, false);
     final FieldData[] fields = new FieldData[] {field};
@@ -245,6 +247,7 @@ public class TestCodecs extends LuceneTestCase {
             SEGMENT,
             10000,
             false,
+            false,
             codec,
             Collections.emptyMap(),
             StringHelper.randomId(),
@@ -293,7 +296,7 @@ public class TestCodecs extends LuceneTestCase {
 
   public void testRandomPostings() throws Throwable {
     final FieldInfos.Builder builder =
-        new FieldInfos.Builder(new FieldInfos.FieldNumbers(null, Version.LATEST.major));
+        new FieldInfos.Builder(new FieldInfos.FieldNumbers(null, null));
 
     final FieldData[] fields = new FieldData[NUM_FIELDS];
     for (int i = 0; i < NUM_FIELDS; i++) {
@@ -324,6 +327,7 @@ public class TestCodecs extends LuceneTestCase {
             SEGMENT,
             10000,
             false,
+            false,
             codec,
             Collections.emptyMap(),
             StringHelper.randomId(),
