@@ -16,12 +16,11 @@
  */
 package org.apache.lucene.index;
 
-
+import com.carrotsearch.randomizedtesting.annotations.TimeoutSuite;
 import java.io.IOException;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.concurrent.CountDownLatch;
-
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
 import org.apache.lucene.search.IndexSearcher;
@@ -29,18 +28,18 @@ import org.apache.lucene.search.Sort;
 import org.apache.lucene.search.SortField;
 import org.apache.lucene.search.TermQuery;
 import org.apache.lucene.search.TopDocs;
+import org.apache.lucene.search.TopScoreDocCollectorManager;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.store.FilterDirectory;
-import org.apache.lucene.store.MockDirectoryWrapper;
 import org.apache.lucene.store.NoLockFactory;
-import org.apache.lucene.util.LuceneTestCase.SuppressCodecs;
-import org.apache.lucene.util.LuceneTestCase;
-import org.apache.lucene.util.TestUtil;
-import org.apache.lucene.util.TimeUnits;
-import com.carrotsearch.randomizedtesting.annotations.TimeoutSuite;
+import org.apache.lucene.tests.store.MockDirectoryWrapper;
+import org.apache.lucene.tests.util.LuceneTestCase;
+import org.apache.lucene.tests.util.LuceneTestCase.SuppressCodecs;
+import org.apache.lucene.tests.util.TestUtil;
+import org.apache.lucene.tests.util.TimeUnits;
 
-@SuppressCodecs({ "SimpleText", "Memory", "Direct" })
-@TimeoutSuite(millis = 8 * TimeUnits.HOUR)
+@SuppressCodecs({"SimpleText", "Direct"})
+@TimeoutSuite(millis = 24 * TimeUnits.HOUR)
 public class TestIndexWriterMaxDocs extends LuceneTestCase {
 
   // The two hour time was achieved on a Linux 3.13 system with these specs:
@@ -62,19 +61,25 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     iw.commit();
 
     // First unoptimized, then optimized:
-    for(int i=0;i<2;i++) {
+    for (int i = 0; i < 2; i++) {
       DirectoryReader ir = DirectoryReader.open(dir);
       assertEquals(IndexWriter.MAX_DOCS, ir.maxDoc());
       assertEquals(IndexWriter.MAX_DOCS, ir.numDocs());
       IndexSearcher searcher = new IndexSearcher(ir);
-      TopDocs hits = searcher.search(new TermQuery(new Term("field", "text")), 10);
-      assertEquals(IndexWriter.MAX_DOCS, hits.totalHits);
+      TopScoreDocCollectorManager collectorManager =
+          new TopScoreDocCollectorManager(10, null, Integer.MAX_VALUE, true);
+      TopDocs hits = searcher.search(new TermQuery(new Term("field", "text")), collectorManager);
+      assertEquals(IndexWriter.MAX_DOCS, hits.totalHits.value());
 
       // Sort by docID reversed:
-      hits = searcher.search(new TermQuery(new Term("field", "text")), 10, new Sort(new SortField(null, SortField.Type.DOC, true)));
-      assertEquals(IndexWriter.MAX_DOCS, hits.totalHits);
+      hits =
+          searcher.search(
+              new TermQuery(new Term("field", "text")),
+              10,
+              new Sort(new SortField(null, SortField.Type.DOC, true)));
+      assertEquals(IndexWriter.MAX_DOCS, hits.totalHits.value());
       assertEquals(10, hits.scoreDocs.length);
-      assertEquals(IndexWriter.MAX_DOCS-1, hits.scoreDocs[0].doc);
+      assertEquals(IndexWriter.MAX_DOCS - 1, hits.scoreDocs[0].doc);
       ir.close();
 
       iw.forceMerge(1);
@@ -89,14 +94,16 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     try {
       Directory dir = newDirectory();
       IndexWriter w = new IndexWriter(dir, new IndexWriterConfig(null));
-      for(int i=0;i<10;i++) {
+      for (int i = 0; i < 10; i++) {
         w.addDocument(new Document());
       }
 
       // 11th document should fail:
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.close();
       dir.close();
@@ -110,14 +117,16 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     try {
       Directory dir = newDirectory();
       IndexWriter w = new IndexWriter(dir, new IndexWriterConfig(null));
-      for(int i=0;i<10;i++) {
+      for (int i = 0; i < 10; i++) {
         w.addDocument(new Document());
       }
 
       // 11th document should fail:
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocuments(Collections.singletonList(new Document()));
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocuments(Collections.singletonList(new Document()));
+          });
 
       w.close();
       dir.close();
@@ -131,14 +140,16 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     try {
       Directory dir = newDirectory();
       IndexWriter w = new IndexWriter(dir, new IndexWriterConfig(null));
-      for(int i=0;i<10;i++) {
+      for (int i = 0; i < 10; i++) {
         w.addDocument(new Document());
       }
 
       // 11th document should fail:
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.updateDocument(new Term("field", "foo"), new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.updateDocument(new Term("field", "foo"), new Document());
+          });
 
       w.close();
       dir.close();
@@ -152,14 +163,16 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     try {
       Directory dir = newDirectory();
       IndexWriter w = new IndexWriter(dir, new IndexWriterConfig(null));
-      for(int i=0;i<10;i++) {
+      for (int i = 0; i < 10; i++) {
         w.addDocument(new Document());
       }
 
       // 11th document should fail:
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.updateDocuments(new Term("field", "foo"), Collections.singletonList(new Document()));
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.updateDocuments(new Term("field", "foo"), Collections.singletonList(new Document()));
+          });
 
       w.close();
       dir.close();
@@ -173,15 +186,15 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     try {
       Directory dir = newDirectory();
       IndexWriter w = new IndexWriter(dir, new IndexWriterConfig(null));
-      for(int i=0;i<10;i++) {
+      for (int i = 0; i < 10; i++) {
         Document doc = new Document();
-        doc.add(newStringField("id", ""+i, Field.Store.NO));
+        doc.add(newStringField("id", "" + i, Field.Store.NO));
         w.addDocument(doc);
       }
 
       // Delete 5 of them:
-      for(int i=0;i<5;i++) {
-        w.deleteDocuments(new Term("id", ""+i));
+      for (int i = 0; i < 5; i++) {
+        w.deleteDocuments(new Term("id", "" + i));
       }
 
       w.forceMerge(1);
@@ -189,14 +202,16 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
       assertEquals(5, w.getDocStats().maxDoc);
 
       // Add 5 more docs
-      for(int i=0;i<5;i++) {
+      for (int i = 0; i < 5; i++) {
         w.addDocument(new Document());
       }
 
       // 11th document should fail:
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.close();
       dir.close();
@@ -205,7 +220,8 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     }
   }
 
-  // Tests that 100% deleted segments (which IW "specializes" by dropping entirely) are not mis-counted
+  // Tests that 100% deleted segments (which IW "specializes" by dropping entirely) are not
+  // mis-counted
   public void testReclaimedDeletesWholeSegments() throws Exception {
     setIndexWriterMaxDocs(10);
     try {
@@ -213,9 +229,9 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
       IndexWriterConfig iwc = new IndexWriterConfig(null);
       iwc.setMergePolicy(NoMergePolicy.INSTANCE);
       IndexWriter w = new IndexWriter(dir, iwc);
-      for(int i=0;i<10;i++) {
+      for (int i = 0; i < 10; i++) {
         Document doc = new Document();
-        doc.add(newStringField("id", ""+i, Field.Store.NO));
+        doc.add(newStringField("id", "" + i, Field.Store.NO));
         w.addDocument(doc);
         if (i % 2 == 0) {
           // Make a new segment every 2 docs:
@@ -224,8 +240,8 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
       }
 
       // Delete 5 of them:
-      for(int i=0;i<5;i++) {
-        w.deleteDocuments(new Term("id", ""+i));
+      for (int i = 0; i < 5; i++) {
+        w.deleteDocuments(new Term("id", "" + i));
       }
 
       w.forceMerge(1);
@@ -233,14 +249,16 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
       assertEquals(5, w.getDocStats().maxDoc);
 
       // Add 5 more docs
-      for(int i=0;i<5;i++) {
+      for (int i = 0; i < 5; i++) {
         w.addDocument(new Document());
       }
 
       // 11th document should fail:
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.close();
       dir.close();
@@ -254,7 +272,7 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     try {
       Directory dir = newDirectory();
       IndexWriter w = new IndexWriter(dir, new IndexWriterConfig(null));
-      for(int i=0;i<10;i++) {
+      for (int i = 0; i < 10; i++) {
         w.addDocument(new Document());
       }
       w.close();
@@ -262,15 +280,19 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
       Directory dir2 = newDirectory();
       IndexWriter w2 = new IndexWriter(dir2, new IndexWriterConfig(null));
       w2.addDocument(new Document());
-      expectThrows(IllegalArgumentException.class, () -> {
-        w2.addIndexes(new Directory[] {dir});
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w2.addIndexes(new Directory[] {dir});
+          });
 
       assertEquals(1, w2.getDocStats().maxDoc);
       DirectoryReader ir = DirectoryReader.open(dir);
-      expectThrows(IllegalArgumentException.class, () -> {
-        TestUtil.addIndexesSlowly(w2, ir);
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            TestUtil.addIndexesSlowly(w2, ir);
+          });
 
       w2.close();
       ir.close();
@@ -303,9 +325,9 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
 
     DirectoryReader ir = DirectoryReader.open(dir);
     DirectoryReader ir2 = DirectoryReader.open(dir2);
-    IndexReader subReaders[] = new IndexReader[copies+1];
+    IndexReader[] subReaders = new IndexReader[copies + 1];
     Arrays.fill(subReaders, ir);
-    subReaders[subReaders.length-1] = ir2;
+    subReaders[subReaders.length - 1] = ir2;
 
     MultiReader mr = new MultiReader(subReaders);
     assertEquals(IndexWriter.MAX_DOCS, mr.maxDoc());
@@ -342,23 +364,25 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
 
     DirectoryReader ir = DirectoryReader.open(dir);
     DirectoryReader ir2 = DirectoryReader.open(dir2);
-    IndexReader subReaders[] = new IndexReader[copies+1];
+    IndexReader[] subReaders = new IndexReader[copies + 1];
     Arrays.fill(subReaders, ir);
-    subReaders[subReaders.length-1] = ir2;
+    subReaders[subReaders.length - 1] = ir2;
 
-    expectThrows(IllegalArgumentException.class, () -> {
-      new MultiReader(subReaders);
-    });
+    expectThrows(
+        IllegalArgumentException.class,
+        () -> {
+          new MultiReader(subReaders);
+        });
 
     ir.close();
     ir2.close();
     dir.close();
     dir2.close();
   }
-  
-  /** 
-   * LUCENE-6299: Test if addindexes(Dir[]) prevents exceeding max docs.
-   */
+
+  /** LUCENE-6299: Test if addindexes(Dir[]) prevents exceeding max docs. */
+  // TODO: can we use the setter to lower the amount of docs to be written here?
+  @Nightly
   public void testAddTooManyIndexesDir() throws Exception {
     // we cheat and add the same one over again... IW wants a write lock on each
     Directory dir = newDirectory(random(), NoLockFactory.INSTANCE);
@@ -370,13 +394,13 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     w.forceMerge(1);
     w.commit();
     w.close();
-    
+
     // wrap this with disk full, so test fails faster and doesn't fill up real disks.
     MockDirectoryWrapper dir2 = newMockDirectory();
     w = new IndexWriter(dir2, new IndexWriterConfig(null));
     w.commit(); // don't confuse checkindex
     dir2.setMaxSizeInBytes(dir2.sizeInBytes() + 65536); // 64KB
-    Directory dirs[] = new Directory[1 + (IndexWriter.MAX_DOCS / 100000)];
+    Directory[] dirs = new Directory[1 + (IndexWriter.MAX_DOCS / 100000)];
     for (int i = 0; i < dirs.length; i++) {
       // bypass iw check for duplicate dirs
       dirs[i] = new FilterDirectory(dir) {};
@@ -385,27 +409,30 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     try {
       w.addIndexes(dirs);
       fail("didn't get expected exception");
-    } catch (IllegalArgumentException expected) {
+    } catch (
+        @SuppressWarnings("unused")
+        IllegalArgumentException expected) {
       // pass
     } catch (IOException fakeDiskFull) {
       final Exception e;
-      if (fakeDiskFull.getMessage() != null && fakeDiskFull.getMessage().startsWith("fake disk full")) {
-        e = new RuntimeException("test failed: IW checks aren't working and we are executing addIndexes");
+      if (fakeDiskFull.getMessage() != null
+          && fakeDiskFull.getMessage().startsWith("fake disk full")) {
+        e =
+            new RuntimeException(
+                "test failed: IW checks aren't working and we are executing addIndexes");
         e.addSuppressed(fakeDiskFull);
       } else {
         e = fakeDiskFull;
       }
       throw e;
     }
-    
+
     w.close();
     dir.close();
     dir2.close();
   }
 
-  /** 
-   * LUCENE-6299: Test if addindexes(CodecReader[]) prevents exceeding max docs.
-   */
+  /** LUCENE-6299: Test if addindexes(CodecReader[]) prevents exceeding max docs. */
   public void testAddTooManyIndexesCodecReader() throws Exception {
     // we cheat and add the same one over again... IW wants a write lock on each
     Directory dir = newDirectory(random(), NoLockFactory.INSTANCE);
@@ -417,7 +444,7 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     w.forceMerge(1);
     w.commit();
     w.close();
-    
+
     // wrap this with disk full, so test fails faster and doesn't fill up real disks.
     MockDirectoryWrapper dir2 = newMockDirectory();
     w = new IndexWriter(dir2, new IndexWriterConfig(null));
@@ -426,7 +453,7 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     IndexReader r = DirectoryReader.open(dir);
     CodecReader segReader = (CodecReader) r.leaves().get(0).reader();
 
-    CodecReader readers[] = new CodecReader[1 + (IndexWriter.MAX_DOCS / 100000)];
+    CodecReader[] readers = new CodecReader[1 + (IndexWriter.MAX_DOCS / 100000)];
     for (int i = 0; i < readers.length; i++) {
       readers[i] = segReader;
     }
@@ -434,12 +461,17 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     try {
       w.addIndexes(readers);
       fail("didn't get expected exception");
-    } catch (IllegalArgumentException expected) {
+    } catch (
+        @SuppressWarnings("unused")
+        IllegalArgumentException expected) {
       // pass
     } catch (IOException fakeDiskFull) {
       final Exception e;
-      if (fakeDiskFull.getMessage() != null && fakeDiskFull.getMessage().startsWith("fake disk full")) {
-        e = new RuntimeException("test failed: IW checks aren't working and we are executing addIndexes");
+      if (fakeDiskFull.getMessage() != null
+          && fakeDiskFull.getMessage().startsWith("fake disk full")) {
+        e =
+            new RuntimeException(
+                "test failed: IW checks aren't working and we are executing addIndexes");
         e.addSuppressed(fakeDiskFull);
       } else {
         e = fakeDiskFull;
@@ -453,10 +485,12 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
     dir2.close();
   }
 
-  public void testTooLargeMaxDocs() throws Exception {
-    expectThrows(IllegalArgumentException.class, () -> {
-      IndexWriter.setMaxDocs(Integer.MAX_VALUE);
-    });
+  public void testTooLargeMaxDocs() {
+    expectThrows(
+        IllegalArgumentException.class,
+        () -> {
+          IndexWriter.setMaxDocs(Integer.MAX_VALUE);
+        });
   }
 
   // LUCENE-6299
@@ -466,15 +500,19 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
       Directory dir = newDirectory();
       IndexWriter w = new IndexWriter(dir, new IndexWriterConfig(null));
       w.addDocument(new Document());
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.deleteAll();
       w.addDocument(new Document());
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.close();
       dir.close();
@@ -490,18 +528,22 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
       Directory dir = newDirectory();
       IndexWriter w = new IndexWriter(dir, new IndexWriterConfig(null));
       w.addDocument(new Document());
-      w.getReader().close();
+      DirectoryReader.open(w).close();
       w.addDocument(new Document());
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.deleteAll();
       w.addDocument(new Document());
       w.addDocument(new Document());
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.close();
       dir.close();
@@ -519,16 +561,20 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
       w.addDocument(new Document());
       w.commit();
       w.addDocument(new Document());
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.deleteAll();
       w.addDocument(new Document());
       w.addDocument(new Document());
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.close();
       dir.close();
@@ -547,38 +593,43 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
 
       CountDownLatch startingGun = new CountDownLatch(1);
       Thread[] threads = new Thread[limit];
-      for(int i=0;i<limit;i++) {
-        threads[i] = new Thread() {
-          @Override
-          public void run() {
-            try {
-              startingGun.await();
-              w.addDocument(new Document());
-            } catch (Exception e) {
-              throw new RuntimeException(e);
-            }
-          }
-          };
+      for (int i = 0; i < limit; i++) {
+        threads[i] =
+            new Thread() {
+              @Override
+              public void run() {
+                try {
+                  startingGun.await();
+                  w.addDocument(new Document());
+                } catch (Exception e) {
+                  throw new RuntimeException(e);
+                }
+              }
+            };
         threads[i].start();
       }
 
       startingGun.countDown();
 
-      for(Thread thread : threads) {
+      for (Thread thread : threads) {
         thread.join();
       }
 
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.deleteAll();
-      for(int i=0;i<limit;i++) {
+      for (int i = 0; i < limit; i++) {
         w.addDocument(new Document());
-      }        
-      expectThrows(IllegalArgumentException.class, () -> {
-        w.addDocument(new Document());
-      });
+      }
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w.addDocument(new Document());
+          });
 
       w.close();
       dir.close();
@@ -598,16 +649,20 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
 
       IndexWriter w2 = new IndexWriter(dir, new IndexWriterConfig(null));
       w2.addDocument(new Document());
-      expectThrows(IllegalArgumentException.class, () -> {
-        w2.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w2.addDocument(new Document());
+          });
 
       w2.deleteAll();
       w2.addDocument(new Document());
       w2.addDocument(new Document());
-      expectThrows(IllegalArgumentException.class, () -> {
-        w2.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w2.addDocument(new Document());
+          });
 
       w2.close();
       dir.close();
@@ -625,9 +680,11 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
       w.addDocument(new Document());
       w.close();
       IndexWriter w2 = new IndexWriter(dir, new IndexWriterConfig(null));
-      expectThrows(IllegalArgumentException.class, () -> {
-        w2.addDocument(new Document());
-      });
+      expectThrows(
+          IllegalArgumentException.class,
+          () -> {
+            w2.addDocument(new Document());
+          });
 
       w2.close();
       dir.close();
@@ -646,16 +703,18 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
 
     setIndexWriterMaxDocs(1);
     try {
-      expectThrows(CorruptIndexException.class, () -> {
-        DirectoryReader.open(dir);
-      });
+      expectThrows(
+          CorruptIndexException.class,
+          () -> {
+            DirectoryReader.open(dir);
+          });
     } finally {
       restoreIndexWriterMaxDocs();
     }
 
     dir.close();
   }
-  
+
   // LUCENE-6299
   public void testCorruptIndexExceptionTooLargeWriter() throws Exception {
     Directory dir = newDirectory();
@@ -666,9 +725,11 @@ public class TestIndexWriterMaxDocs extends LuceneTestCase {
 
     setIndexWriterMaxDocs(1);
     try {
-      expectThrows(CorruptIndexException.class, () -> {
-        new IndexWriter(dir, new IndexWriterConfig(null));
-      });
+      expectThrows(
+          CorruptIndexException.class,
+          () -> {
+            new IndexWriter(dir, new IndexWriterConfig(null));
+          });
     } finally {
       restoreIndexWriterMaxDocs();
     }
