@@ -16,21 +16,14 @@
  */
 package org.apache.lucene.index;
 
-
 import java.io.IOException;
 import java.io.StringReader;
-import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-
 import org.apache.lucene.analysis.Analyzer;
-import org.apache.lucene.analysis.CannedTokenStream;
-import org.apache.lucene.analysis.MockAnalyzer;
-import org.apache.lucene.analysis.MockTokenizer;
-import org.apache.lucene.analysis.Token;
 import org.apache.lucene.analysis.TokenFilter;
 import org.apache.lucene.analysis.TokenStream;
 import org.apache.lucene.analysis.Tokenizer;
@@ -42,23 +35,27 @@ import org.apache.lucene.document.TextField;
 import org.apache.lucene.index.IndexWriterConfig.OpenMode;
 import org.apache.lucene.search.DocIdSetIterator;
 import org.apache.lucene.store.Directory;
+import org.apache.lucene.tests.analysis.CannedTokenStream;
+import org.apache.lucene.tests.analysis.MockAnalyzer;
+import org.apache.lucene.tests.analysis.MockTokenizer;
+import org.apache.lucene.tests.analysis.Token;
+import org.apache.lucene.tests.index.RandomIndexWriter;
+import org.apache.lucene.tests.util.LuceneTestCase;
+import org.apache.lucene.tests.util.TestUtil;
 import org.apache.lucene.util.BytesRef;
-import org.apache.lucene.util.LuceneTestCase;
-import org.apache.lucene.util.TestUtil;
 
 public class TestPayloads extends LuceneTestCase {
-    
+
   // Simple tests to test the Payload class
   public void testPayload() throws Exception {
     BytesRef payload = new BytesRef("This is a test!");
     assertEquals("Wrong payload length.", "This is a test!".length(), payload.length);
-        
+
     BytesRef clone = payload.clone();
     assertEquals(payload.length, clone.length);
     for (int i = 0; i < payload.length; i++) {
       assertEquals(payload.bytes[i + payload.offset], clone.bytes[i + clone.offset]);
     }
-        
   }
 
   // Tests whether the DocumentWriter and SegmentMerger correctly enable the
@@ -75,7 +72,8 @@ public class TestPayloads extends LuceneTestCase {
     // even if only some term positions have payloads
     d.add(newTextField("f2", "This field has payloads in all docs", Field.Store.NO));
     d.add(newTextField("f2", "This field has payloads in all docs NO PAYLOAD", Field.Store.NO));
-    // this field is used to verify if the SegmentMerger enables payloads for a field if it has payloads 
+    // this field is used to verify if the SegmentMerger enables payloads for a field if it has
+    // payloads
     // enabled in only some documents
     d.add(newTextField("f3", "This field has payloads in some docs", Field.Store.NO));
     // only add payload data for field f2
@@ -90,12 +88,11 @@ public class TestPayloads extends LuceneTestCase {
     assertTrue("Payload field bit should be set.", fi.fieldInfo("f2").hasPayloads());
     assertFalse("Payload field bit should not be set.", fi.fieldInfo("f3").hasPayloads());
     reader.close();
-        
+
     // now we add another document which has payloads for field f3 and verify if the SegmentMerger
     // enabled payloads for that field
     analyzer = new PayloadAnalyzer(); // Clear payload state for each field
-    writer = new IndexWriter(ram, newIndexWriterConfig(analyzer)
-                             .setOpenMode(OpenMode.CREATE));
+    writer = new IndexWriter(ram, newIndexWriterConfig(analyzer).setOpenMode(OpenMode.CREATE));
     d = new Document();
     d.add(newTextField("f1", "This field has no payloads", Field.Store.NO));
     d.add(newTextField("f2", "This field has payloads in all docs", Field.Store.NO));
@@ -120,28 +117,31 @@ public class TestPayloads extends LuceneTestCase {
     ram.close();
   }
 
-  // Tests if payloads are correctly stored and loaded using both RamDirectory and FSDirectory
+  // Tests if payloads are correctly stored and loaded.
   public void testPayloadsEncoding() throws Exception {
     Directory dir = newDirectory();
     performTest(dir);
     dir.close();
   }
-    
+
   // builds an index with payloads in the given Directory and performs
   // different tests to verify the payload encoding
   private void performTest(Directory dir) throws Exception {
     PayloadAnalyzer analyzer = new PayloadAnalyzer();
-    IndexWriter writer = new IndexWriter(dir, newIndexWriterConfig(analyzer)
-                                         .setOpenMode(OpenMode.CREATE)
-                                         .setMergePolicy(newLogMergePolicy()));
-        
+    IndexWriter writer =
+        new IndexWriter(
+            dir,
+            newIndexWriterConfig(analyzer)
+                .setOpenMode(OpenMode.CREATE)
+                .setMergePolicy(newLogMergePolicy()));
+
     // should be in sync with value in TermInfosWriter
     final int skipInterval = 16;
-        
+
     final int numTerms = 5;
     final String fieldName = "f1";
-        
-    int numDocs = skipInterval + 1; 
+
+    int numDocs = skipInterval + 1;
     // create content for the test documents with just a few terms
     Term[] terms = generateTerms(fieldName, numTerms);
     StringBuilder sb = new StringBuilder();
@@ -150,11 +150,10 @@ public class TestPayloads extends LuceneTestCase {
       sb.append(" ");
     }
     String content = sb.toString();
-        
-        
+
     int payloadDataLength = numTerms * numDocs * 2 + numTerms * numDocs * (numDocs - 1) / 2;
     byte[] payloadData = generateRandomData(payloadDataLength);
-        
+
     Document d = new Document();
     d.add(newTextField(fieldName, content, Field.Store.NO));
     // add the same document multiple times to have the same payload lengths for all
@@ -165,35 +164,33 @@ public class TestPayloads extends LuceneTestCase {
       offset += numTerms;
       writer.addDocument(d);
     }
-        
+
     // make sure we create more than one segment to test merging
     writer.commit();
-        
-    // now we make sure to have different payload lengths next at the next skip point        
+
+    // now we make sure to have different payload lengths next at the next skip point
     for (int i = 0; i < numDocs; i++) {
       analyzer.setPayloadData(fieldName, payloadData, offset, i);
       offset += i * numTerms;
       writer.addDocument(d);
     }
-        
+
     writer.forceMerge(1);
     // flush
     writer.close();
-        
-        
+
     /*
      * Verify the index
      * first we test if all payloads are stored correctly
-     */        
+     */
     IndexReader reader = DirectoryReader.open(dir);
 
     byte[] verifyPayloadData = new byte[payloadDataLength];
     offset = 0;
     PostingsEnum[] tps = new PostingsEnum[numTerms];
     for (int i = 0; i < numTerms; i++) {
-      tps[i] = MultiFields.getTermPositionsEnum(reader,
-                                                terms[i].field(),
-                                                new BytesRef(terms[i].text()));
+      tps[i] =
+          MultiTerms.getTermPostingsEnum(reader, terms[i].field(), new BytesRef(terms[i].text()));
     }
 
     while (tps[0].nextDoc() != DocIdSetIterator.NO_MORE_DOCS) {
@@ -215,13 +212,12 @@ public class TestPayloads extends LuceneTestCase {
     }
 
     assertByteArrayEquals(payloadData, verifyPayloadData);
-        
+
     /*
      *  test lazy skipping
-     */        
-    PostingsEnum tp = MultiFields.getTermPositionsEnum(reader,
-                                                               terms[0].field(),
-                                                               new BytesRef(terms[0].text()));
+     */
+    PostingsEnum tp =
+        MultiTerms.getTermPostingsEnum(reader, terms[0].field(), new BytesRef(terms[0].text()));
     tp.nextDoc();
     tp.nextPosition();
     // NOTE: prior rev of this test was failing to first
@@ -234,20 +230,18 @@ public class TestPayloads extends LuceneTestCase {
     assertEquals(payload.bytes[payload.offset], payloadData[numTerms]);
     tp.nextDoc();
     tp.nextPosition();
-        
+
     // we don't read this payload and skip to a different document
     tp.advance(5);
     tp.nextPosition();
     payload = tp.getPayload();
     assertEquals("Wrong payload length.", 1, payload.length);
     assertEquals(payload.bytes[payload.offset], payloadData[5 * numTerms]);
-        
+
     /*
      * Test different lengths at skip points
      */
-    tp = MultiFields.getTermPositionsEnum(reader,
-                                          terms[1].field(),
-                                          new BytesRef(terms[1].text()));
+    tp = MultiTerms.getTermPostingsEnum(reader, terms[1].field(), new BytesRef(terms[1].text()));
     tp.nextDoc();
     tp.nextPosition();
     assertEquals("Wrong payload length.", 1, tp.getPayload().length);
@@ -259,16 +253,16 @@ public class TestPayloads extends LuceneTestCase {
     assertEquals("Wrong payload length.", 1, tp.getPayload().length);
     tp.advance(3 * skipInterval - 1);
     tp.nextPosition();
-    assertEquals("Wrong payload length.", 3 * skipInterval - 2 * numDocs - 1, tp.getPayload().length);
-        
+    assertEquals(
+        "Wrong payload length.", 3 * skipInterval - 2 * numDocs - 1, tp.getPayload().length);
+
     reader.close();
-        
+
     // test long payload
     analyzer = new PayloadAnalyzer();
-    writer = new IndexWriter(dir, newIndexWriterConfig(analyzer)
-                             .setOpenMode(OpenMode.CREATE));
+    writer = new IndexWriter(dir, newIndexWriterConfig(analyzer).setOpenMode(OpenMode.CREATE));
     String singleTerm = "lucene";
-        
+
     d = new Document();
     d.add(newTextField(fieldName, singleTerm, Field.Store.NO));
     // add a payload whose length is greater than the buffer size of BufferedIndexOutput
@@ -276,34 +270,28 @@ public class TestPayloads extends LuceneTestCase {
     analyzer.setPayloadData(fieldName, payloadData, 100, 1500);
     writer.addDocument(d);
 
-        
     writer.forceMerge(1);
     // flush
     writer.close();
-        
+
     reader = DirectoryReader.open(dir);
-    tp = MultiFields.getTermPositionsEnum(reader,
-                                          fieldName,
-                                          new BytesRef(singleTerm));
+    tp = MultiTerms.getTermPostingsEnum(reader, fieldName, new BytesRef(singleTerm));
     tp.nextDoc();
     tp.nextPosition();
-        
+
     BytesRef br = tp.getPayload();
     verifyPayloadData = new byte[br.length];
     byte[] portion = new byte[1500];
     System.arraycopy(payloadData, 100, portion, 0, 1500);
-        
+
     assertByteArrayEquals(portion, br.bytes, br.offset, br.length);
     reader.close();
-        
   }
-    
-  static final Charset utf8 = StandardCharsets.UTF_8;
-    
+
   private void generateRandomData(byte[] data) {
     // this test needs the random data to be valid unicode
     String s = TestUtil.randomFixedByteLengthUnicodeString(random(), data.length);
-    byte b[] = s.getBytes(utf8);
+    byte[] b = s.getBytes(StandardCharsets.UTF_8);
     assert b.length == data.length;
     System.arraycopy(b, 0, data, 0, b.length);
   }
@@ -313,7 +301,7 @@ public class TestPayloads extends LuceneTestCase {
     generateRandomData(data);
     return data;
   }
-    
+
   private Term[] generateTerms(String fieldName, int n) {
     int maxDigits = (int) (Math.log(n) / Math.log(10));
     Term[] terms = new Term[n];
@@ -331,32 +319,30 @@ public class TestPayloads extends LuceneTestCase {
     return terms;
   }
 
-
   void assertByteArrayEquals(byte[] b1, byte[] b2) {
     if (b1.length != b2.length) {
       fail("Byte arrays have different lengths: " + b1.length + ", " + b2.length);
     }
-        
+
     for (int i = 0; i < b1.length; i++) {
       if (b1[i] != b2[i]) {
         fail("Byte arrays different at index " + i + ": " + b1[i] + ", " + b2[i]);
       }
     }
-  }    
-    
+  }
+
   void assertByteArrayEquals(byte[] b1, byte[] b2, int b2offset, int b2length) {
     if (b1.length != b2length) {
       fail("Byte arrays have different lengths: " + b1.length + ", " + b2length);
     }
-        
+
     for (int i = 0; i < b1.length; i++) {
-      if (b1[i] != b2[b2offset+i]) {
-        fail("Byte arrays different at index " + i + ": " + b1[i] + ", " + b2[b2offset+i]);
+      if (b1[i] != b2[b2offset + i]) {
+        fail("Byte arrays different at index " + i + ": " + b1[i] + ", " + b2[b2offset + i]);
       }
     }
-  }    
-    
-    
+  }
+
   static class PayloadData {
     byte[] data;
     int offset;
@@ -369,55 +355,45 @@ public class TestPayloads extends LuceneTestCase {
     }
   }
 
-  /**
-   * This Analyzer uses an MockTokenizer and PayloadFilter.
-   */
+  /** This Analyzer uses an MockTokenizer and PayloadFilter. */
   private static class PayloadAnalyzer extends Analyzer {
-    Map<String,PayloadData> fieldToData = new HashMap<>();
+    Map<String, PayloadData> fieldToData = new HashMap<>();
 
     public PayloadAnalyzer() {
       super(PER_FIELD_REUSE_STRATEGY);
     }
-        
-    public PayloadAnalyzer(String field, byte[] data, int offset, int length) {
-      super(PER_FIELD_REUSE_STRATEGY);
-      setPayloadData(field, data, offset, length);
-    }
 
     void setPayloadData(String field, byte[] data, int offset, int length) {
       fieldToData.put(field, new PayloadData(data, offset, length));
     }
-        
+
     @Override
     public TokenStreamComponents createComponents(String fieldName) {
-      PayloadData payload =  fieldToData.get(fieldName);
+      PayloadData payload = fieldToData.get(fieldName);
       Tokenizer ts = new MockTokenizer(MockTokenizer.WHITESPACE, false);
-      TokenStream tokenStream = (payload != null) ?
-        new PayloadFilter(ts, fieldName, fieldToData) : ts;
+      TokenStream tokenStream =
+          (payload != null) ? new PayloadFilter(ts, fieldName, fieldToData) : ts;
       return new TokenStreamComponents(ts, tokenStream);
     }
   }
 
-    
-  /**
-   * This Filter adds payloads to the tokens.
-   */
+  /** This Filter adds payloads to the tokens. */
   private static class PayloadFilter extends TokenFilter {
     PayloadAttribute payloadAtt;
     CharTermAttribute termAttribute;
-    private Map<String,PayloadData> fieldToData;
+    private Map<String, PayloadData> fieldToData;
     private String fieldName;
     private PayloadData payloadData;
     private int offset;
-        
-    public PayloadFilter(TokenStream in, String fieldName, Map<String,PayloadData> fieldToData) {
+
+    public PayloadFilter(TokenStream in, String fieldName, Map<String, PayloadData> fieldToData) {
       super(in);
       this.fieldToData = fieldToData;
       this.fieldName = fieldName;
       payloadAtt = addAttribute(PayloadAttribute.class);
       termAttribute = addAttribute(CharTermAttribute.class);
     }
-        
+
     @Override
     public boolean incrementToken() throws IOException {
       boolean hasNext = input.incrementToken();
@@ -426,7 +402,8 @@ public class TestPayloads extends LuceneTestCase {
       }
 
       // Some values of the same field are to have payloads and others not
-      if (offset + payloadData.length <= payloadData.data.length && !termAttribute.toString().endsWith("NO PAYLOAD")) {
+      if (offset + payloadData.length <= payloadData.data.length
+          && !termAttribute.toString().endsWith("NO PAYLOAD")) {
         BytesRef p = new BytesRef(payloadData.data, offset, payloadData.length);
         payloadAtt.setPayload(p);
         offset += payloadData.length;
@@ -444,47 +421,49 @@ public class TestPayloads extends LuceneTestCase {
       this.offset = payloadData.offset;
     }
   }
-    
+
   public void testThreadSafety() throws Exception {
     final int numThreads = 5;
     final int numDocs = atLeast(50);
     final ByteArrayPool pool = new ByteArrayPool(numThreads, 5);
 
     Directory dir = newDirectory();
-    final IndexWriter writer = new IndexWriter(dir, newIndexWriterConfig(new MockAnalyzer(random())));
+    final IndexWriter writer =
+        new IndexWriter(dir, newIndexWriterConfig(new MockAnalyzer(random())));
     final String field = "test";
-        
+
     Thread[] ingesters = new Thread[numThreads];
     for (int i = 0; i < numThreads; i++) {
-      ingesters[i] = new Thread() {
-          @Override
-          public void run() {
-            try {
-              for (int j = 0; j < numDocs; j++) {
-                Document d = new Document();
-                d.add(new TextField(field, new PoolingPayloadTokenStream(pool)));
-                writer.addDocument(d);
+      ingesters[i] =
+          new Thread() {
+            @Override
+            public void run() {
+              try {
+                for (int j = 0; j < numDocs; j++) {
+                  Document d = new Document();
+                  d.add(new TextField(field, new PoolingPayloadTokenStream(pool)));
+                  writer.addDocument(d);
+                }
+              } catch (Exception e) {
+                e.printStackTrace();
+                fail(e.toString());
               }
-            } catch (Exception e) {
-              e.printStackTrace();
-              fail(e.toString());
             }
-          }
-        };
+          };
       ingesters[i].start();
     }
-        
+
     for (int i = 0; i < numThreads; i++) {
       ingesters[i].join();
     }
     writer.close();
     IndexReader reader = DirectoryReader.open(dir);
-    TermsEnum terms = MultiFields.getTerms(reader, field).iterator();
+    TermsEnum terms = MultiTerms.getTerms(reader, field).iterator();
     PostingsEnum tp = null;
     while (terms.next() != null) {
       String termText = terms.term().utf8ToString();
       tp = terms.postings(tp, PostingsEnum.PAYLOADS);
-      while(tp.nextDoc() != DocIdSetIterator.NO_MORE_DOCS) {
+      while (tp.nextDoc() != DocIdSetIterator.NO_MORE_DOCS) {
         int freq = tp.freq();
         for (int i = 0; i < freq; i++) {
           tp.nextPosition();
@@ -497,7 +476,7 @@ public class TestPayloads extends LuceneTestCase {
     dir.close();
     assertEquals(pool.size(), numThreads);
   }
-    
+
   private class PoolingPayloadTokenStream extends TokenStream {
     private byte[] payload;
     private boolean first;
@@ -506,17 +485,17 @@ public class TestPayloads extends LuceneTestCase {
 
     CharTermAttribute termAtt;
     PayloadAttribute payloadAtt;
-        
+
     PoolingPayloadTokenStream(ByteArrayPool pool) {
       this.pool = pool;
       payload = pool.get();
       generateRandomData(payload);
-      term = new String(payload, 0, payload.length, utf8);
+      term = new String(payload, StandardCharsets.UTF_8);
       first = true;
       payloadAtt = addAttribute(PayloadAttribute.class);
       termAtt = addAttribute(CharTermAttribute.class);
     }
-        
+
     @Override
     public boolean incrementToken() throws IOException {
       if (!first) return false;
@@ -526,32 +505,31 @@ public class TestPayloads extends LuceneTestCase {
       payloadAtt.setPayload(new BytesRef(payload));
       return true;
     }
-        
+
     @Override
     public void close() throws IOException {
       pool.release(payload);
     }
-        
   }
-    
+
   private static class ByteArrayPool {
     private List<byte[]> pool;
-        
+
     ByteArrayPool(int capacity, int size) {
       pool = new ArrayList<>();
       for (int i = 0; i < capacity; i++) {
         pool.add(new byte[size]);
       }
     }
-    
+
     synchronized byte[] get() {
       return pool.remove(0);
     }
-        
+
     synchronized void release(byte[] b) {
       pool.add(b);
     }
-        
+
     synchronized int size() {
       return pool.size();
     }
@@ -559,15 +537,17 @@ public class TestPayloads extends LuceneTestCase {
 
   public void testAcrossFields() throws Exception {
     Directory dir = newDirectory();
-    RandomIndexWriter writer = new RandomIndexWriter(random(), dir,
-                                                     new MockAnalyzer(random(), MockTokenizer.WHITESPACE, true));
+    RandomIndexWriter writer =
+        new RandomIndexWriter(
+            random(), dir, new MockAnalyzer(random(), MockTokenizer.WHITESPACE, true));
     Document doc = new Document();
     doc.add(new TextField("hasMaybepayload", "here we go", Field.Store.YES));
     writer.addDocument(doc);
     writer.close();
 
-    writer = new RandomIndexWriter(random(), dir,
-                                   new MockAnalyzer(random(), MockTokenizer.WHITESPACE, true));
+    writer =
+        new RandomIndexWriter(
+            random(), dir, new MockAnalyzer(random(), MockTokenizer.WHITESPACE, true));
     doc = new Document();
     doc.add(new TextField("hasMaybepayload2", "here we go", Field.Store.YES));
     writer.addDocument(doc);
@@ -577,7 +557,7 @@ public class TestPayloads extends LuceneTestCase {
 
     dir.close();
   }
-  
+
   /** some docs have payload att, some not */
   public void testMixupDocs() throws Exception {
     Directory dir = newDirectory();
@@ -585,10 +565,9 @@ public class TestPayloads extends LuceneTestCase {
     iwc.setMergePolicy(newLogMergePolicy());
     RandomIndexWriter writer = new RandomIndexWriter(random(), dir, iwc);
     Document doc = new Document();
-    Field field = new TextField("field", "", Field.Store.NO);
     TokenStream ts = new MockTokenizer(MockTokenizer.WHITESPACE, true);
-    ((Tokenizer)ts).setReader(new StringReader("here we go"));
-    field.setTokenStream(ts);
+    ((Tokenizer) ts).setReader(new StringReader("here we go"));
+    Field field = new Field("field", ts, TextField.TYPE_NOT_STORED);
     doc.add(field);
     writer.addDocument(doc);
     Token withPayload = new Token("withPayload", 0, 11);
@@ -598,11 +577,11 @@ public class TestPayloads extends LuceneTestCase {
     field.setTokenStream(ts);
     writer.addDocument(doc);
     ts = new MockTokenizer(MockTokenizer.WHITESPACE, true);
-    ((Tokenizer)ts).setReader(new StringReader("another"));
+    ((Tokenizer) ts).setReader(new StringReader("another"));
     field.setTokenStream(ts);
     writer.addDocument(doc);
     DirectoryReader reader = writer.getReader();
-    TermsEnum te = MultiFields.getTerms(reader, "field").iterator();
+    TermsEnum te = MultiTerms.getTerms(reader, "field").iterator();
     assertTrue(te.seekExact(new BytesRef("withPayload")));
     PostingsEnum de = te.postings(null, PostingsEnum.PAYLOADS);
     de.nextDoc();
@@ -612,28 +591,26 @@ public class TestPayloads extends LuceneTestCase {
     reader.close();
     dir.close();
   }
-  
+
   /** some field instances have payload att, some not */
   public void testMixupMultiValued() throws Exception {
     Directory dir = newDirectory();
     RandomIndexWriter writer = new RandomIndexWriter(random(), dir);
     Document doc = new Document();
-    Field field = new TextField("field", "", Field.Store.NO);
     TokenStream ts = new MockTokenizer(MockTokenizer.WHITESPACE, true);
-    ((Tokenizer)ts).setReader(new StringReader("here we go"));
+    Field field = new Field("field", ts, TextField.TYPE_NOT_STORED);
+    ((Tokenizer) ts).setReader(new StringReader("here we go"));
     field.setTokenStream(ts);
     doc.add(field);
-    Field field2 = new TextField("field", "", Field.Store.NO);
     Token withPayload = new Token("withPayload", 0, 11);
     withPayload.setPayload(new BytesRef("test"));
     ts = new CannedTokenStream(withPayload);
     assertTrue(ts.hasAttribute(PayloadAttribute.class));
-    field2.setTokenStream(ts);
+    Field field2 = new Field("field", ts, TextField.TYPE_NOT_STORED);
     doc.add(field2);
-    Field field3 = new TextField("field", "", Field.Store.NO);
     ts = new MockTokenizer(MockTokenizer.WHITESPACE, true);
-    ((Tokenizer)ts).setReader(new StringReader("nopayload"));
-    field3.setTokenStream(ts);
+    ((Tokenizer) ts).setReader(new StringReader("nopayload"));
+    Field field3 = new Field("field", ts, TextField.TYPE_NOT_STORED);
     doc.add(field3);
     writer.addDocument(doc);
     DirectoryReader reader = writer.getReader();
