@@ -226,7 +226,7 @@ public class TestBooleanQuery extends LuceneTestCase {
     // PhraseQuery w/ no terms added returns a null scorer
     PhraseQuery pq = new PhraseQuery("field", new String[0]);
     q.add(pq, BooleanClause.Occur.SHOULD);
-    assertEquals(1, s.search(q.build(), 10).totalHits.value);
+    assertEquals(1, s.search(q.build(), 10).totalHits.value());
 
     // A required clause which returns null scorer should return null scorer to
     // IndexSearcher.
@@ -234,11 +234,11 @@ public class TestBooleanQuery extends LuceneTestCase {
     pq = new PhraseQuery("field", new String[0]);
     q.add(new TermQuery(new Term("field", "a")), BooleanClause.Occur.SHOULD);
     q.add(pq, BooleanClause.Occur.MUST);
-    assertEquals(0, s.search(q.build(), 10).totalHits.value);
+    assertEquals(0, s.search(q.build(), 10).totalHits.value());
 
     DisjunctionMaxQuery dmq =
         new DisjunctionMaxQuery(Arrays.asList(new TermQuery(new Term("field", "a")), pq), 1.0f);
-    assertEquals(1, s.search(dmq, 10).totalHits.value);
+    assertEquals(1, s.search(dmq, 10).totalHits.value());
 
     r.close();
     w.close();
@@ -273,13 +273,13 @@ public class TestBooleanQuery extends LuceneTestCase {
 
     MultiReader multireader = new MultiReader(reader1, reader2);
     IndexSearcher searcher = newSearcher(multireader);
-    assertEquals(0, searcher.search(query.build(), 10).totalHits.value);
+    assertEquals(0, searcher.search(query.build(), 10).totalHits.value());
 
     final ExecutorService es =
         Executors.newCachedThreadPool(new NamedThreadFactory("NRT search threads"));
     searcher = new IndexSearcher(multireader, es);
     if (VERBOSE) System.out.println("rewritten form: " + searcher.rewrite(query.build()));
-    assertEquals(0, searcher.search(query.build(), 10).totalHits.value);
+    assertEquals(0, searcher.search(query.build(), 10).totalHits.value());
     es.shutdown();
     es.awaitTermination(1, TimeUnit.SECONDS);
 
@@ -419,7 +419,7 @@ public class TestBooleanQuery extends LuceneTestCase {
 
     // No doc can match: BQ has only 2 clauses and we are asking for minShouldMatch=4
     bq.setMinimumNumberShouldMatch(4);
-    assertEquals(0, s.search(bq.build(), 1).totalHits.value);
+    assertEquals(0, s.search(bq.build(), 1).totalHits.value());
     r.close();
     w.close();
     dir.close();
@@ -474,7 +474,7 @@ public class TestBooleanQuery extends LuceneTestCase {
       throws IOException {
     final BooleanQuery.Builder bq2Builder = new BooleanQuery.Builder();
     for (BooleanClause c : bq) {
-      if (c.getOccur() != Occur.FILTER) {
+      if (c.occur() != Occur.FILTER) {
         bq2Builder.add(c);
       }
     }
@@ -523,6 +523,7 @@ public class TestBooleanQuery extends LuceneTestCase {
             return null;
           }
         });
+
     assertTrue(matched.get());
   }
 
@@ -961,6 +962,143 @@ public class TestBooleanQuery extends LuceneTestCase {
     dir.close();
   }
 
+  public void testTwoClauseTermDisjunctionCountOptimization() throws Exception {
+    int largerTermCount = RandomNumbers.randomIntBetween(random(), 11, 100);
+    int smallerTermCount = RandomNumbers.randomIntBetween(random(), 1, (largerTermCount - 1) / 10);
+
+    List<String[]> docContent = new ArrayList<>(largerTermCount + smallerTermCount);
+
+    for (int i = 0; i < largerTermCount; i++) {
+      docContent.add(new String[] {"large"});
+    }
+
+    for (int i = 0; i < smallerTermCount; i++) {
+      docContent.add(new String[] {"small", "also small"});
+    }
+
+    try (Directory dir = newDirectory()) {
+      try (IndexWriter w =
+          new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(newLogMergePolicy()))) {
+
+        for (String[] values : docContent) {
+          Document doc = new Document();
+          for (String value : values) {
+            doc.add(new StringField("foo", value, Field.Store.NO));
+          }
+          w.addDocument(doc);
+        }
+        w.forceMerge(1);
+      }
+
+      try (IndexReader reader = DirectoryReader.open(dir)) {
+        final int[] countInvocations = new int[] {0};
+        IndexSearcher countingIndexSearcher =
+            new IndexSearcher(reader) {
+              @Override
+              public int count(Query query) throws IOException {
+                countInvocations[0]++;
+                return super.count(query);
+              }
+            };
+
+        {
+          // Test no matches in either term
+          countInvocations[0] = 0;
+          BooleanQuery query =
+              new BooleanQuery.Builder()
+                  .add(new TermQuery(new Term("foo", "no match")), BooleanClause.Occur.SHOULD)
+                  .add(new TermQuery(new Term("foo", "also no match")), BooleanClause.Occur.SHOULD)
+                  .build();
+
+          assertEquals(0, countingIndexSearcher.count(query));
+          assertEquals(3, countInvocations[0]);
+        }
+        {
+          // Test match no match in first term
+          countInvocations[0] = 0;
+          BooleanQuery query =
+              new BooleanQuery.Builder()
+                  .add(new TermQuery(new Term("foo", "no match")), BooleanClause.Occur.SHOULD)
+                  .add(new TermQuery(new Term("foo", "small")), BooleanClause.Occur.SHOULD)
+                  .build();
+
+          assertEquals(smallerTermCount, countingIndexSearcher.count(query));
+          assertEquals(3, countInvocations[0]);
+        }
+        {
+          // Test match no match in second term
+          countInvocations[0] = 0;
+          BooleanQuery query =
+              new BooleanQuery.Builder()
+                  .add(new TermQuery(new Term("foo", "small")), BooleanClause.Occur.SHOULD)
+                  .add(new TermQuery(new Term("foo", "no match")), BooleanClause.Occur.SHOULD)
+                  .build();
+
+          assertEquals(smallerTermCount, countingIndexSearcher.count(query));
+          assertEquals(3, countInvocations[0]);
+        }
+        {
+          // Test match in both terms that hits optimization threshold with small term first
+          countInvocations[0] = 0;
+
+          BooleanQuery query =
+              new BooleanQuery.Builder()
+                  .add(new TermQuery(new Term("foo", "small")), BooleanClause.Occur.SHOULD)
+                  .add(new TermQuery(new Term("foo", "large")), BooleanClause.Occur.SHOULD)
+                  .build();
+
+          int count = countingIndexSearcher.count(query);
+
+          assertEquals(largerTermCount + smallerTermCount, count);
+          assertEquals(4, countInvocations[0]);
+
+          assertTrue(query.isTwoClausePureDisjunctionWithTerms());
+          Query[] queries =
+              query.rewriteTwoClauseDisjunctionWithTermsForCount(countingIndexSearcher);
+          assertEquals(queries.length, 3);
+          assertEquals(smallerTermCount, countingIndexSearcher.count(queries[0]));
+          assertEquals(largerTermCount, countingIndexSearcher.count(queries[1]));
+        }
+        {
+          // Test match in both terms that hits optimization threshold with large term first
+          countInvocations[0] = 0;
+
+          BooleanQuery query =
+              new BooleanQuery.Builder()
+                  .add(new TermQuery(new Term("foo", "large")), BooleanClause.Occur.SHOULD)
+                  .add(new TermQuery(new Term("foo", "small")), BooleanClause.Occur.SHOULD)
+                  .build();
+
+          int count = countingIndexSearcher.count(query);
+
+          assertEquals(largerTermCount + smallerTermCount, count);
+          assertEquals(4, countInvocations[0]);
+
+          assertTrue(query.isTwoClausePureDisjunctionWithTerms());
+          Query[] queries =
+              query.rewriteTwoClauseDisjunctionWithTermsForCount(countingIndexSearcher);
+          assertEquals(queries.length, 3);
+          assertEquals(largerTermCount, countingIndexSearcher.count(queries[0]));
+          assertEquals(smallerTermCount, countingIndexSearcher.count(queries[1]));
+        }
+        {
+          // Test match in both terms that doesn't hit optimization threshold
+          countInvocations[0] = 0;
+          BooleanQuery query =
+              new BooleanQuery.Builder()
+                  .add(new TermQuery(new Term("foo", "small")), BooleanClause.Occur.SHOULD)
+                  .add(new TermQuery(new Term("foo", "also small")), BooleanClause.Occur.SHOULD)
+                  .build();
+
+          int count = countingIndexSearcher.count(query);
+
+          assertEquals(smallerTermCount, count);
+          assertEquals(3, countInvocations[0]);
+        }
+      }
+    }
+  }
+
   // test BlockMaxMaxscoreScorer
   public void testDisjunctionTwoClausesMatchesCountAndScore() throws Exception {
     List<String[]> docContent =
