@@ -83,7 +83,7 @@ public class TestBooleanRewrites extends LuceneTestCase {
     query1.add(new TermQuery(new Term("field", "a")), Occur.FILTER);
 
     // Single clauses rewrite to a term query
-    final Query rewritten1 = query1.build().rewrite(reader);
+    final Query rewritten1 = query1.build().rewrite(searcher);
     assertTrue(rewritten1 instanceof BoostQuery);
     assertEquals(0f, ((BoostQuery) rewritten1).getBoost(), 0f);
 
@@ -92,7 +92,7 @@ public class TestBooleanRewrites extends LuceneTestCase {
     // make sure to set score=0
     BooleanQuery.Builder query2 = new BooleanQuery.Builder();
     query2.add(new TermQuery(new Term("field", "a")), Occur.FILTER);
-    query2.add(new TermQuery(new Term("field", "b")), Occur.SHOULD);
+    query2.add(new TermQuery(new Term("missing_field", "b")), Occur.SHOULD);
     final Weight weight =
         searcher.createWeight(searcher.rewrite(query2.build()), ScoreMode.COMPLETE, 1);
     final Scorer scorer = weight.scorer(reader.leaves().get(0));
@@ -541,7 +541,7 @@ public class TestBooleanRewrites extends LuceneTestCase {
   }
 
   private void assertEquals(TopDocs td1, TopDocs td2) {
-    assertEquals(td1.totalHits.value, td2.totalHits.value);
+    assertEquals(td1.totalHits.value(), td2.totalHits.value());
     assertEquals(td1.scoreDocs.length, td2.scoreDocs.length);
     Map<Integer, Float> expectedScores =
         Arrays.stream(td1.scoreDocs).collect(Collectors.toMap(sd -> sd.doc, sd -> sd.score));
@@ -946,7 +946,7 @@ public class TestBooleanRewrites extends LuceneTestCase {
     }
 
     @Override
-    public Query rewrite(IndexReader indexReader) {
+    public Query rewrite(IndexSearcher indexSearcher) {
       numRewrites++;
       return this;
     }
