@@ -16,7 +16,7 @@
  */
 package org.apache.lucene.search;
 
-import static org.hamcrest.CoreMatchers.instanceOf;
+import static org.hamcrest.Matchers.instanceOf;
 
 import java.io.IOException;
 import java.util.Random;
@@ -36,6 +36,7 @@ import org.apache.lucene.tests.search.DummyTotalHitCountCollector;
 import org.apache.lucene.tests.search.QueryUtils;
 import org.apache.lucene.tests.util.LuceneTestCase;
 import org.apache.lucene.tests.util.TestUtil;
+import org.hamcrest.MatcherAssert;
 
 @LuceneTestCase.SuppressCodecs(value = "SimpleText")
 public class TestIndexSortSortedNumericDocValuesRangeQuery extends LuceneTestCase {
@@ -98,7 +99,7 @@ public class TestIndexSortSortedNumericDocValuesRangeQuery extends LuceneTestCas
     final int maxDoc = searcher.getIndexReader().maxDoc();
     final TopDocs td1 = searcher.search(q1, maxDoc, scores ? Sort.RELEVANCE : Sort.INDEXORDER);
     final TopDocs td2 = searcher.search(q2, maxDoc, scores ? Sort.RELEVANCE : Sort.INDEXORDER);
-    assertEquals(td1.totalHits.value, td2.totalHits.value);
+    assertEquals(td1.totalHits.value(), td2.totalHits.value());
     for (int i = 0; i < td1.scoreDocs.length; ++i) {
       assertEquals(td1.scoreDocs[i].doc, td2.scoreDocs[i].doc);
       if (scores) {
@@ -339,7 +340,7 @@ public class TestIndexSortSortedNumericDocValuesRangeQuery extends LuceneTestCas
     IndexReader reader = writer.getReader();
 
     Query query = createQuery("field", Long.MIN_VALUE, Long.MAX_VALUE);
-    Query rewrittenQuery = query.rewrite(reader);
+    Query rewrittenQuery = query.rewrite(newSearcher(reader));
     assertEquals(new FieldExistsQuery("field"), rewrittenQuery);
 
     writer.close();
@@ -357,9 +358,10 @@ public class TestIndexSortSortedNumericDocValuesRangeQuery extends LuceneTestCas
     Query fallbackQuery = new BooleanQuery.Builder().build();
     Query query = new IndexSortSortedNumericDocValuesRangeQuery("field", 1, 42, fallbackQuery);
 
-    Query rewrittenQuery = query.rewrite(reader);
+    Query rewrittenQuery = query.rewrite(newSearcher(reader));
     assertNotEquals(query, rewrittenQuery);
-    assertThat(rewrittenQuery, instanceOf(IndexSortSortedNumericDocValuesRangeQuery.class));
+    MatcherAssert.assertThat(
+        rewrittenQuery, instanceOf(IndexSortSortedNumericDocValuesRangeQuery.class));
 
     IndexSortSortedNumericDocValuesRangeQuery rangeQuery =
         (IndexSortSortedNumericDocValuesRangeQuery) rewrittenQuery;
@@ -561,9 +563,9 @@ public class TestIndexSortSortedNumericDocValuesRangeQuery extends LuceneTestCas
     RandomIndexWriter writer = new RandomIndexWriter(random(), dir, iwc);
 
     writer.addDocument(
-        createSNDVAndPointDocument("field", TestUtil.nextLong(random(), lowerValue, upperValue)));
+        createSNDVAndPointDocument("field", random().nextLong(lowerValue, upperValue)));
     writer.addDocument(
-        createSNDVAndPointDocument("field", TestUtil.nextLong(random(), lowerValue, upperValue)));
+        createSNDVAndPointDocument("field", random().nextLong(lowerValue, upperValue)));
     // missingValue
     writer.addDocument(createMissingValueDocument());
 
@@ -575,9 +577,11 @@ public class TestIndexSortSortedNumericDocValuesRangeQuery extends LuceneTestCas
         new IndexSortSortedNumericDocValuesRangeQuery(
             "field", lowerValue, upperValue, fallbackQuery);
     Weight weight = query.createWeight(searcher, ScoreMode.COMPLETE, 1.0f);
+    int count = 0;
     for (LeafReaderContext context : searcher.getLeafContexts()) {
-      assertEquals(2, weight.count(context));
+      count += weight.count(context);
     }
+    assertEquals(2, count);
 
     writer.close();
     reader.close();
