@@ -42,7 +42,6 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.IntConsumer;
-import java.util.stream.Collectors;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field.Store;
 import org.apache.lucene.document.LongPoint;
@@ -74,6 +73,7 @@ import org.apache.lucene.tests.util.TestUtil;
 import org.apache.lucene.util.Accountable;
 import org.apache.lucene.util.BytesRef;
 import org.apache.lucene.util.IOUtils;
+import org.apache.lucene.util.SuppressForbidden;
 
 public class TestLRUQueryCache extends LuceneTestCase {
 
@@ -198,7 +198,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
                                         .map(
                                             filterCollector ->
                                                 (DummyTotalHitCountCollector) filterCollector.in)
-                                        .collect(Collectors.toList()));
+                                        .toList());
                               }
                             });
                     assertEquals(totalHits2, totalHits1);
@@ -234,6 +234,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
     }
   }
 
+  @SuppressForbidden(reason = "Thread sleep")
   public void testLRUEviction() throws Exception {
     Directory dir = newDirectory();
     final RandomIndexWriter w = new RandomIndexWriter(random(), dir);
@@ -461,7 +462,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
         throws IOException {
       return new ConstantScoreWeight(this, boost) {
         @Override
-        public Scorer scorer(LeafReaderContext context) throws IOException {
+        public ScorerSupplier scorerSupplier(LeafReaderContext context) throws IOException {
           return null;
         }
 
@@ -555,7 +556,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
   }
 
   /** DummyQuery with Accountable, pretending to be a memory-eating query */
-  private class AccountableDummyQuery extends DummyQuery implements Accountable {
+  private static class AccountableDummyQuery extends DummyQuery implements Accountable {
 
     @Override
     public long ramBytesUsed() {
@@ -593,6 +594,43 @@ public class TestLRUQueryCache extends LuceneTestCase {
     dir.close();
   }
 
+  public void testConsistencyWithAccountableQueries() throws IOException {
+    var queryCache = new LRUQueryCache(1, 10000000, context -> true, Float.POSITIVE_INFINITY);
+
+    var dir = newDirectory();
+    var writer = new RandomIndexWriter(random(), dir);
+    writer.addDocument(new Document());
+    var reader = writer.getReader();
+    var searcher = new IndexSearcher(reader);
+    searcher.setQueryCache(queryCache);
+    searcher.setQueryCachingPolicy(ALWAYS_CACHE);
+
+    queryCache.assertConsistent();
+
+    var accountableQuery = new AccountableDummyQuery();
+    searcher.count(accountableQuery);
+    var expectedRamBytesUsed =
+        HASHTABLE_RAM_BYTES_PER_ENTRY
+            + LINKED_HASHTABLE_RAM_BYTES_PER_ENTRY
+            + accountableQuery.ramBytesUsed()
+            + queryCache.getChildResources().iterator().next().ramBytesUsed();
+    assertEquals(expectedRamBytesUsed, queryCache.ramBytesUsed());
+    queryCache.assertConsistent();
+
+    queryCache.clearQuery(accountableQuery);
+    assertEquals(HASHTABLE_RAM_BYTES_PER_ENTRY, queryCache.ramBytesUsed());
+    queryCache.assertConsistent();
+
+    queryCache.clearCoreCacheKey(
+        reader.getContext().leaves().get(0).reader().getCoreCacheHelper().getKey());
+    assertEquals(0, queryCache.ramBytesUsed());
+    queryCache.assertConsistent();
+
+    reader.close();
+    writer.close();
+    dir.close();
+  }
+
   public void testOnUse() throws IOException {
     final LRUQueryCache queryCache =
         new LRUQueryCache(
@@ -804,7 +842,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
           @Override
           protected void onHit(Object readerCoreKey, Query query) {
             super.onHit(readerCoreKey, query);
-            switch (indexId.get(readerCoreKey).intValue()) {
+            switch (indexId.get(readerCoreKey)) {
               case 1:
                 hitCount1.incrementAndGet();
                 break;
@@ -819,7 +857,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
           @Override
           protected void onMiss(Object readerCoreKey, Query query) {
             super.onMiss(readerCoreKey, query);
-            switch (indexId.get(readerCoreKey).intValue()) {
+            switch (indexId.get(readerCoreKey)) {
               case 1:
                 missCount1.incrementAndGet();
                 break;
@@ -1140,7 +1178,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
         throws IOException {
       return new ConstantScoreWeight(this, boost) {
         @Override
-        public Scorer scorer(LeafReaderContext context) throws IOException {
+        public ScorerSupplier scorerSupplier(LeafReaderContext context) throws IOException {
           return null;
         }
 
@@ -1294,11 +1332,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
     public void onUse(final Query query) {
       AtomicInteger count;
       synchronized (counts) {
-        count = counts.get(query);
-        if (count == null) {
-          count = new AtomicInteger();
-          counts.put(query, count);
-        }
+        count = counts.computeIfAbsent(query, k -> new AtomicInteger());
       }
       count.incrementAndGet();
     }
@@ -1321,15 +1355,30 @@ public class TestLRUQueryCache extends LuceneTestCase {
     }
 
     @Override
-    public Scorer scorer(LeafReaderContext context) throws IOException {
-      scorerCalled.set(true);
-      return in.scorer(context);
-    }
+    public ScorerSupplier scorerSupplier(LeafReaderContext context) throws IOException {
+      final var scorerSupplier = in.scorerSupplier(context);
+      if (scorerSupplier == null) {
+        return null;
+      }
+      final var scorer = scorerSupplier.get(Long.MAX_VALUE);
+      return new ScorerSupplier() {
+        @Override
+        public Scorer get(long leadCost) throws IOException {
+          scorerCalled.set(true);
+          return scorer;
+        }
 
-    @Override
-    public BulkScorer bulkScorer(LeafReaderContext context) throws IOException {
-      bulkScorerCalled.set(true);
-      return in.bulkScorer(context);
+        @Override
+        public BulkScorer bulkScorer() throws IOException {
+          bulkScorerCalled.set(true);
+          return in.bulkScorer(context);
+        }
+
+        @Override
+        public long cost() {
+          return scorer.iterator().cost();
+        }
+      };
     }
   }
 
@@ -1351,8 +1400,8 @@ public class TestLRUQueryCache extends LuceneTestCase {
     weight = new WeightWrapper(weight, scorerCalled, bulkScorerCalled);
     weight = cache.doCache(weight, NEVER_CACHE);
     weight.bulkScorer(leaf);
-    assertEquals(true, bulkScorerCalled.get());
-    assertEquals(false, scorerCalled.get());
+    assertTrue(bulkScorerCalled.get());
+    assertFalse(scorerCalled.get());
     assertEquals(0, cache.getCacheCount());
 
     searcher.getIndexReader().close();
@@ -1502,7 +1551,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
         }
 
         @Override
-        public Scorer scorer(LeafReaderContext context) throws IOException {
+        public ScorerSupplier scorerSupplier(LeafReaderContext context) throws IOException {
           return null;
         }
 
@@ -1573,11 +1622,6 @@ public class TestLRUQueryCache extends LuceneTestCase {
     public Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost)
         throws IOException {
       return new ConstantScoreWeight(this, boost) {
-        @Override
-        public Scorer scorer(LeafReaderContext context) throws IOException {
-          return scorerSupplier(context).get(Long.MAX_VALUE);
-        }
-
         @Override
         public boolean isCacheable(LeafReaderContext ctx) {
           return true;
@@ -1585,12 +1629,12 @@ public class TestLRUQueryCache extends LuceneTestCase {
 
         @Override
         public ScorerSupplier scorerSupplier(LeafReaderContext context) throws IOException {
-          final Weight weight = this;
+          final var scorer = new ConstantScoreScorer(boost, scoreMode, DocIdSetIterator.all(1));
           return new ScorerSupplier() {
             @Override
             public Scorer get(long leadCost) throws IOException {
               scorerCreated.set(true);
-              return new ConstantScoreScorer(weight, boost, scoreMode, DocIdSetIterator.all(1));
+              return scorer;
             }
 
             @Override
@@ -1675,12 +1719,21 @@ public class TestLRUQueryCache extends LuceneTestCase {
     public Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost)
         throws IOException {
       return new ConstantScoreWeight(this, 1) {
-
         @Override
-        public Scorer scorer(LeafReaderContext context) throws IOException {
-          scorerCreatedCount.incrementAndGet();
-          return new ConstantScoreScorer(
-              this, 1, scoreMode, DocIdSetIterator.all(context.reader().maxDoc()));
+        public ScorerSupplier scorerSupplier(LeafReaderContext context) throws IOException {
+          return new ScorerSupplier() {
+            @Override
+            public Scorer get(long leadCost) throws IOException {
+              scorerCreatedCount.incrementAndGet();
+              return new ConstantScoreScorer(
+                  1, scoreMode, DocIdSetIterator.all(context.reader().maxDoc()));
+            }
+
+            @Override
+            public long cost() {
+              return context.reader().maxDoc();
+            }
+          };
         }
 
         @Override
@@ -1742,7 +1795,7 @@ public class TestLRUQueryCache extends LuceneTestCase {
     assertEquals(2, searcher.count(query));
     assertEquals(2, query.scorerCreatedCount.get()); // both segments cached
 
-    w.updateNumericDocValue(new Term("text", "text"), "field", 2l);
+    w.updateNumericDocValue(new Term("text", "text"), "field", 2L);
     reader.close();
     reader = DirectoryReader.open(w);
     searcher =
