@@ -30,6 +30,7 @@ import org.apache.lucene.document.NumericDocValuesField;
 import org.apache.lucene.document.SortedDocValuesField;
 import org.apache.lucene.document.StoredField;
 import org.apache.lucene.index.IndexReader;
+import org.apache.lucene.index.StoredFields;
 import org.apache.lucene.index.Term;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.tests.index.RandomIndexWriter;
@@ -230,13 +231,13 @@ public class TestSearchAfter extends LuceneTestCase {
     final boolean doScores;
     final CollectorManager<?, ? extends TopDocs> allManager;
     if (sort == null) {
-      allManager = TopScoreDocCollector.createSharedManager(maxDoc, null, Integer.MAX_VALUE);
+      allManager = new TopScoreDocCollectorManager(maxDoc, null, Integer.MAX_VALUE);
       doScores = false;
     } else if (sort == Sort.RELEVANCE) {
-      allManager = TopFieldCollector.createSharedManager(sort, maxDoc, null, Integer.MAX_VALUE);
+      allManager = new TopFieldCollectorManager(sort, maxDoc, null, Integer.MAX_VALUE, true);
       doScores = true;
     } else {
-      allManager = TopFieldCollector.createSharedManager(sort, maxDoc, null, Integer.MAX_VALUE);
+      allManager = new TopFieldCollectorManager(sort, maxDoc, null, Integer.MAX_VALUE, true);
       doScores = random().nextBoolean();
     }
     all = searcher.search(query, allManager);
@@ -245,21 +246,22 @@ public class TestSearchAfter extends LuceneTestCase {
     }
 
     if (VERBOSE) {
-      System.out.println("  all.totalHits.value=" + all.totalHits.value);
+      System.out.println("  all.totalHits.value()=" + all.totalHits.value());
       int upto = 0;
+      StoredFields storedFields = searcher.storedFields();
       for (ScoreDoc scoreDoc : all.scoreDocs) {
         System.out.println(
             "    hit "
                 + (upto++)
                 + ": id="
-                + searcher.doc(scoreDoc.doc).get("id")
+                + storedFields.document(scoreDoc.doc).get("id")
                 + " "
                 + scoreDoc);
       }
     }
     int pageStart = 0;
     ScoreDoc lastBottom = null;
-    while (pageStart < all.totalHits.value) {
+    while (pageStart < all.totalHits.value()) {
       TopDocs paged;
       final CollectorManager<?, ? extends TopDocs> pagedManager;
       if (sort == null) {
@@ -267,20 +269,14 @@ public class TestSearchAfter extends LuceneTestCase {
           System.out.println("  iter lastBottom=" + lastBottom);
         }
         pagedManager =
-            TopScoreDocCollector.createSharedManager(pageSize, lastBottom, Integer.MAX_VALUE);
+            new TopScoreDocCollectorManager(pageSize, lastBottom, Integer.MAX_VALUE, true);
       } else {
         if (VERBOSE) {
           System.out.println("  iter lastBottom=" + lastBottom);
         }
-        if (sort == Sort.RELEVANCE) {
-          pagedManager =
-              TopFieldCollector.createSharedManager(
-                  sort, pageSize, (FieldDoc) lastBottom, Integer.MAX_VALUE);
-        } else {
-          pagedManager =
-              TopFieldCollector.createSharedManager(
-                  sort, pageSize, (FieldDoc) lastBottom, Integer.MAX_VALUE);
-        }
+        pagedManager =
+            new TopFieldCollectorManager(
+                sort, pageSize, (FieldDoc) lastBottom, Integer.MAX_VALUE, true);
       }
       paged = searcher.search(query, pagedManager);
       if (doScores) {
@@ -302,14 +298,17 @@ public class TestSearchAfter extends LuceneTestCase {
   }
 
   void assertPage(int pageStart, TopDocs all, TopDocs paged) throws IOException {
-    assertEquals(all.totalHits.value, paged.totalHits.value);
+    assertEquals(all.totalHits.value(), paged.totalHits.value());
+    StoredFields storedFields = searcher.storedFields();
     for (int i = 0; i < paged.scoreDocs.length; i++) {
       ScoreDoc sd1 = all.scoreDocs[pageStart + i];
       ScoreDoc sd2 = paged.scoreDocs[i];
       if (VERBOSE) {
         System.out.println("    hit " + (pageStart + i));
-        System.out.println("      expected id=" + searcher.doc(sd1.doc).get("id") + " " + sd1);
-        System.out.println("        actual id=" + searcher.doc(sd2.doc).get("id") + " " + sd2);
+        System.out.println(
+            "      expected id=" + storedFields.document(sd1.doc).get("id") + " " + sd1);
+        System.out.println(
+            "        actual id=" + storedFields.document(sd2.doc).get("id") + " " + sd2);
       }
       assertEquals(sd1.doc, sd2.doc);
       assertEquals(sd1.score, sd2.score, 0f);
