@@ -16,11 +16,12 @@
  */
 package org.apache.lucene.search;
 
+import static org.apache.lucene.search.SortField.FIELD_SCORE;
 
 import java.io.IOException;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Comparator;
-
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
 import org.apache.lucene.document.Field.Store;
@@ -30,23 +31,23 @@ import org.apache.lucene.document.TextField;
 import org.apache.lucene.index.DirectoryReader;
 import org.apache.lucene.index.IndexReader;
 import org.apache.lucene.index.IndexWriter;
+import org.apache.lucene.index.IndexWriterConfig;
 import org.apache.lucene.index.LeafReaderContext;
 import org.apache.lucene.index.NoMergePolicy;
-import org.apache.lucene.index.RandomIndexWriter;
 import org.apache.lucene.index.Term;
 import org.apache.lucene.search.BooleanClause.Occur;
 import org.apache.lucene.search.FieldValueHitQueue.Entry;
 import org.apache.lucene.store.Directory;
-import org.apache.lucene.util.LuceneTestCase;
-import org.apache.lucene.util.TestUtil;
-
-import static org.apache.lucene.search.SortField.FIELD_SCORE;
+import org.apache.lucene.tests.index.RandomIndexWriter;
+import org.apache.lucene.tests.search.CheckHits;
+import org.apache.lucene.tests.util.LuceneTestCase;
+import org.apache.lucene.tests.util.TestUtil;
 
 public class TestTopFieldCollector extends LuceneTestCase {
   private IndexSearcher is;
   private IndexReader ir;
   private Directory dir;
-  
+
   @Override
   public void setUp() throws Exception {
     super.setUp();
@@ -59,9 +60,10 @@ public class TestTopFieldCollector extends LuceneTestCase {
     }
     ir = iw.getReader();
     iw.close();
-    is = newSearcher(ir);
+    // cannot use threads with this IndexSearcher since some tests rely on no threads
+    is = newSearcher(ir, true, true, false);
   }
-  
+
   @Override
   public void tearDown() throws Exception {
     ir.close();
@@ -69,56 +71,61 @@ public class TestTopFieldCollector extends LuceneTestCase {
     super.tearDown();
   }
 
-  private TopFieldCollector doSearchWithThreshold(int numResults, int thresHold, Query q, Sort sort, IndexReader indexReader) throws IOException {
-    IndexSearcher searcher = newSearcher(indexReader, true, true, false);
-    TopFieldCollector tdc = TopFieldCollector.create(sort, numResults, thresHold);
-    searcher.search(q, tdc);
-    return tdc;
+  private static TopDocs doSearchWithThreshold(
+      int numResults, int thresHold, Query q, Sort sort, IndexReader indexReader)
+      throws IOException {
+    IndexSearcher searcher = newSearcher(indexReader);
+    TopFieldCollectorManager manager =
+        new TopFieldCollectorManager(
+            sort, numResults, null, thresHold, searcher.getSlices().length > 1);
+    return searcher.search(q, manager);
   }
 
-  private TopDocs doConcurrentSearchWithThreshold(int numResults, int threshold, Query q, Sort sort, IndexReader indexReader) throws IOException {
+  private static TopDocs doConcurrentSearchWithThreshold(
+      int numResults, int threshold, Query q, Sort sort, IndexReader indexReader)
+      throws IOException {
     IndexSearcher searcher = newSearcher(indexReader, true, true, true);
-    CollectorManager<TopFieldCollector,TopFieldDocs> collectorManager = TopFieldCollector.createSharedManager(sort, numResults,
-            null, threshold);
-    TopDocs tdc = searcher.search(q, collectorManager);
-    return tdc;
+
+    TopFieldCollectorManager collectorManager =
+        new TopFieldCollectorManager(
+            sort, numResults, null, threshold, searcher.getSlices().length > 1);
+
+    TopDocs topDoc = searcher.search(q, collectorManager);
+
+    return topDoc;
   }
-  
+
   public void testSortWithoutFillFields() throws Exception {
-    
+
     // There was previously a bug in TopFieldCollector when fillFields was set
     // to false - the same doc and score was set in ScoreDoc[] array. This test
     // asserts that if fillFields is false, the documents are set properly. It
     // does not use Searcher's default search methods (with Sort) since all set
     // fillFields to true.
-    Sort[] sort = new Sort[] { new Sort(SortField.FIELD_DOC), new Sort() };
-    for(int i = 0; i < sort.length; i++) {
+    Sort[] sort = new Sort[] {new Sort(SortField.FIELD_DOC), new Sort()};
+    for (int i = 0; i < sort.length; i++) {
       Query q = new MatchAllDocsQuery();
-      TopDocsCollector<Entry> tdc = TopFieldCollector.create(sort[i], 10, Integer.MAX_VALUE);
-      
-      is.search(q, tdc);
-      
-      ScoreDoc[] sd = tdc.topDocs().scoreDocs;
-      for(int j = 1; j < sd.length; j++) {
+      TopFieldCollectorManager collectorManager =
+          new TopFieldCollectorManager(sort[i], 10, Integer.MAX_VALUE);
+
+      ScoreDoc[] sd = is.search(q, collectorManager).scoreDocs;
+      for (int j = 1; j < sd.length; j++) {
         assertTrue(sd[j].doc != sd[j - 1].doc);
       }
-      
     }
   }
 
   public void testSort() throws Exception {
 
     // Two Sort criteria to instantiate the multi/single comparators.
-    Sort[] sort = new Sort[] {new Sort(SortField.FIELD_DOC), new Sort() };
-    for(int i = 0; i < sort.length; i++) {
+    Sort[] sort = new Sort[] {new Sort(SortField.FIELD_DOC), new Sort()};
+    for (int i = 0; i < sort.length; i++) {
       Query q = new MatchAllDocsQuery();
-      TopDocsCollector<Entry> tdc = TopFieldCollector.create(sort[i], 10, Integer.MAX_VALUE);
-      
-      is.search(q, tdc);
-      
-      TopDocs td = tdc.topDocs();
+      TopFieldCollectorManager tdc =
+          new TopFieldCollectorManager(sort[i], 10, null, Integer.MAX_VALUE, false);
+      TopDocs td = is.search(q, tdc);
       ScoreDoc[] sd = td.scoreDocs;
-      for(int j = 0; j < sd.length; j++) {
+      for (int j = 0; j < sd.length; j++) {
         assertTrue(Float.isNaN(sd[j].score));
       }
     }
@@ -126,21 +133,20 @@ public class TestTopFieldCollector extends LuceneTestCase {
 
   public void testSharedHitcountCollector() throws Exception {
     IndexSearcher concurrentSearcher = newSearcher(ir, true, true, true);
+    IndexSearcher singleThreadedSearcher = newSearcher(ir, true, true, false);
 
     // Two Sort criteria to instantiate the multi/single comparators.
-    Sort[] sort = new Sort[] {new Sort(SortField.FIELD_DOC), new Sort() };
-    for(int i = 0; i < sort.length; i++) {
+    Sort[] sort = new Sort[] {new Sort(SortField.FIELD_DOC), new Sort()};
+    for (int i = 0; i < sort.length; i++) {
       Query q = new MatchAllDocsQuery();
-      TopDocsCollector<Entry> tdc = TopFieldCollector.create(sort[i], 10, Integer.MAX_VALUE);
+      TopFieldCollectorManager tdc = new TopFieldCollectorManager(sort[i], 10, Integer.MAX_VALUE);
+      TopDocs td = singleThreadedSearcher.search(q, tdc);
 
-      is.search(q, tdc);
-
-      CollectorManager<TopFieldCollector,TopFieldDocs> tsdc = TopFieldCollector.createSharedManager(sort[i], 10, null, Integer.MAX_VALUE);
-
-      TopDocs td = tdc.topDocs();
+      TopFieldCollectorManager tsdc = new TopFieldCollectorManager(sort[i], 10, Integer.MAX_VALUE);
       TopDocs td2 = concurrentSearcher.search(q, tsdc);
+
       ScoreDoc[] sd = td.scoreDocs;
-      for(int j = 0; j < sd.length; j++) {
+      for (int j = 0; j < sd.length; j++) {
         assertTrue(Float.isNaN(sd[j].score));
       }
 
@@ -150,23 +156,21 @@ public class TestTopFieldCollector extends LuceneTestCase {
 
   public void testSortWithoutTotalHitTracking() throws Exception {
     Sort sort = new Sort(SortField.FIELD_DOC);
-    for(int i = 0; i < 2; i++) {
+    for (int i = 0; i < 2; i++) {
       Query q = new MatchAllDocsQuery();
       // check that setting trackTotalHits to false does not throw an NPE because
       // the index is not sorted
-      TopDocsCollector<Entry> tdc;
+      TopFieldCollectorManager manager;
       if (i % 2 == 0) {
-        tdc =  TopFieldCollector.create(sort, 10, 1);
+        manager = new TopFieldCollectorManager(sort, 10, 1);
       } else {
-        FieldDoc fieldDoc = new FieldDoc(1, Float.NaN, new Object[] { 1 });
-        tdc = TopFieldCollector.create(sort, 10, fieldDoc, 1);
+        FieldDoc fieldDoc = new FieldDoc(1, Float.NaN, new Object[] {1});
+        manager = new TopFieldCollectorManager(sort, 10, fieldDoc, 1);
       }
 
-      is.search(q, tdc);
-
-      TopDocs td = tdc.topDocs();
+      TopDocs td = is.search(q, manager);
       ScoreDoc[] sd = td.scoreDocs;
-      for(int j = 0; j < sd.length; j++) {
+      for (int j = 0; j < sd.length; j++) {
         assertTrue(Float.isNaN(sd[j].score));
       }
     }
@@ -175,55 +179,63 @@ public class TestTopFieldCollector extends LuceneTestCase {
   public void testTotalHits() throws Exception {
     Directory dir = newDirectory();
     Sort sort = new Sort(new SortField("foo", SortField.Type.LONG));
-    IndexWriter w = new IndexWriter(dir, newIndexWriterConfig()
-        .setMergePolicy(NoMergePolicy.INSTANCE)
-        .setIndexSort(sort));
+    IndexWriter w =
+        new IndexWriter(
+            dir,
+            newIndexWriterConfig()
+                .setMergePolicy(NoMergePolicy.INSTANCE)
+                .setIndexSort(sort)
+                .setMaxBufferedDocs(7)
+                .setRAMBufferSizeMB(IndexWriterConfig.DISABLE_AUTO_FLUSH));
     Document doc = new Document();
     doc.add(new NumericDocValuesField("foo", 3));
-    w.addDocuments(Arrays.asList(doc, doc, doc, doc));
+    for (Document d : Arrays.asList(doc, doc, doc, doc)) {
+      w.addDocument(d);
+    }
     w.flush();
-    w.addDocuments(Arrays.asList(doc, doc, doc, doc, doc, doc));
+    for (Document d : Arrays.asList(doc, doc, doc, doc, doc, doc)) {
+      w.addDocument(d);
+    }
     w.flush();
     IndexReader reader = DirectoryReader.open(w);
     assertEquals(2, reader.leaves().size());
     w.close();
 
-    for (int totalHitsThreshold = 0; totalHitsThreshold < 20; ++ totalHitsThreshold) {
-      for (FieldDoc after : new FieldDoc[] { null, new FieldDoc(4, Float.NaN, new Object[] { 2L })}) {
-        TopFieldCollector collector = TopFieldCollector.create(sort, 2, after, totalHitsThreshold);
-        ScoreAndDoc scorer = new ScoreAndDoc();
+    for (int totalHitsThreshold = 0; totalHitsThreshold < 20; ++totalHitsThreshold) {
+      for (FieldDoc after : new FieldDoc[] {null, new FieldDoc(4, Float.NaN, new Object[] {2L})}) {
+        TopFieldCollector collector =
+            new TopFieldCollectorManager(sort, 2, after, totalHitsThreshold).newCollector();
+        Score scorer = new Score();
 
         LeafCollector leafCollector1 = collector.getLeafCollector(reader.leaves().get(0));
         leafCollector1.setScorer(scorer);
 
-        scorer.doc = 0;
         scorer.score = 3;
         leafCollector1.collect(0);
 
-        scorer.doc = 1;
         scorer.score = 3;
         leafCollector1.collect(1);
 
         LeafCollector leafCollector2 = collector.getLeafCollector(reader.leaves().get(1));
         leafCollector2.setScorer(scorer);
 
-        scorer.doc = 1;
         scorer.score = 3;
         if (totalHitsThreshold < 3) {
           expectThrows(CollectionTerminatedException.class, () -> leafCollector2.collect(1));
           TopDocs topDocs = collector.topDocs();
-          assertEquals(new TotalHits(3, TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO), topDocs.totalHits);
+          assertEquals(
+              new TotalHits(3, TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO), topDocs.totalHits);
           continue;
         } else {
           leafCollector2.collect(1);
         }
 
-        scorer.doc = 5;
         scorer.score = 4;
         if (totalHitsThreshold == 3) {
           expectThrows(CollectionTerminatedException.class, () -> leafCollector2.collect(1));
           TopDocs topDocs = collector.topDocs();
-          assertEquals(new TotalHits(4, TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO), topDocs.totalHits);
+          assertEquals(
+              new TotalHits(4, TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO), topDocs.totalHits);
           continue;
         } else {
           leafCollector2.collect(1);
@@ -238,8 +250,7 @@ public class TestTopFieldCollector extends LuceneTestCase {
     dir.close();
   }
 
-  private static class ScoreAndDoc extends Scorable {
-    int doc = -1;
+  private static class Score extends Scorable {
     float score;
     Float minCompetitiveScore = null;
 
@@ -248,11 +259,6 @@ public class TestTopFieldCollector extends LuceneTestCase {
       this.minCompetitiveScore = minCompetitiveScore;
     }
 
-    @Override
-    public int docID() {
-      return doc;
-    }
-
     @Override
     public float score() throws IOException {
       return score;
@@ -261,7 +267,8 @@ public class TestTopFieldCollector extends LuceneTestCase {
 
   public void testSetMinCompetitiveScore() throws Exception {
     Directory dir = newDirectory();
-    IndexWriter w = new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(NoMergePolicy.INSTANCE));
+    IndexWriter w =
+        new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(NoMergePolicy.INSTANCE));
     Document doc = new Document();
     w.addDocuments(Arrays.asList(doc, doc, doc, doc));
     w.flush();
@@ -272,52 +279,45 @@ public class TestTopFieldCollector extends LuceneTestCase {
     w.close();
 
     Sort sort = new Sort(FIELD_SCORE, new SortField("foo", SortField.Type.LONG));
-    TopFieldCollector collector = TopFieldCollector.create(sort, 2, null, 2);
-    ScoreAndDoc scorer = new ScoreAndDoc();
+    TopFieldCollector collector = new TopFieldCollectorManager(sort, 2, 2).newCollector();
+    Score scorer = new Score();
 
     LeafCollector leafCollector = collector.getLeafCollector(reader.leaves().get(0));
     leafCollector.setScorer(scorer);
     assertNull(scorer.minCompetitiveScore);
 
-    scorer.doc = 0;
     scorer.score = 1;
     leafCollector.collect(0);
     assertNull(scorer.minCompetitiveScore);
 
-    scorer.doc = 1;
     scorer.score = 2;
     leafCollector.collect(1);
     assertNull(scorer.minCompetitiveScore);
-    
-    scorer.doc = 2;
+
     scorer.score = 3;
     leafCollector.collect(2);
     assertEquals(2f, scorer.minCompetitiveScore, 0f);
 
-    scorer.doc = 3;
     scorer.score = 0.5f;
     // Make sure we do not call setMinCompetitiveScore for non-competitive hits
     scorer.minCompetitiveScore = Float.NaN;
     leafCollector.collect(3);
     assertTrue(Float.isNaN(scorer.minCompetitiveScore));
 
-    scorer.doc = 4;
     scorer.score = 4;
     leafCollector.collect(4);
     assertEquals(3f, scorer.minCompetitiveScore, 0f);
 
     // Make sure the min score is set on scorers on new segments
-    scorer = new ScoreAndDoc();
+    scorer = new Score();
     leafCollector = collector.getLeafCollector(reader.leaves().get(1));
     leafCollector.setScorer(scorer);
     assertEquals(3f, scorer.minCompetitiveScore, 0f);
 
-    scorer.doc = 0;
     scorer.score = 1;
     leafCollector.collect(0);
     assertEquals(3f, scorer.minCompetitiveScore, 0f);
 
-    scorer.doc = 1;
     scorer.score = 4;
     leafCollector.collect(1);
     assertEquals(4f, scorer.minCompetitiveScore, 0f);
@@ -328,7 +328,8 @@ public class TestTopFieldCollector extends LuceneTestCase {
 
   public void testTotalHitsWithScore() throws Exception {
     Directory dir = newDirectory();
-    IndexWriter w = new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(NoMergePolicy.INSTANCE));
+    IndexWriter w =
+        new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(NoMergePolicy.INSTANCE));
     Document doc = new Document();
     w.addDocuments(Arrays.asList(doc, doc, doc, doc));
     w.flush();
@@ -338,37 +339,39 @@ public class TestTopFieldCollector extends LuceneTestCase {
     assertEquals(2, reader.leaves().size());
     w.close();
 
-    for (int totalHitsThreshold = 0; totalHitsThreshold < 20; ++ totalHitsThreshold) {
+    for (int totalHitsThreshold = 0; totalHitsThreshold < 20; ++totalHitsThreshold) {
       Sort sort = new Sort(FIELD_SCORE, new SortField("foo", SortField.Type.LONG));
-      TopFieldCollector collector = TopFieldCollector.create(sort, 2, null, totalHitsThreshold);
-      ScoreAndDoc scorer = new ScoreAndDoc();
+      TopFieldCollector collector =
+          new TopFieldCollectorManager(sort, 2, totalHitsThreshold).newCollector();
+      Score scorer = new Score();
 
       LeafCollector leafCollector = collector.getLeafCollector(reader.leaves().get(0));
       leafCollector.setScorer(scorer);
 
-      scorer.doc = 0;
       scorer.score = 3;
       leafCollector.collect(0);
 
-      scorer.doc = 1;
       scorer.score = 3;
       leafCollector.collect(1);
 
       leafCollector = collector.getLeafCollector(reader.leaves().get(1));
       leafCollector.setScorer(scorer);
 
-      scorer.doc = 1;
       scorer.score = 3;
       leafCollector.collect(1);
 
-      scorer.doc = 5;
       scorer.score = 4;
       leafCollector.collect(1);
 
       TopDocs topDocs = collector.topDocs();
-      assertEquals(4, topDocs.totalHits.value);
       assertEquals(totalHitsThreshold < 4, scorer.minCompetitiveScore != null);
-      assertEquals(totalHitsThreshold < 4 ? TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO : TotalHits.Relation.EQUAL_TO, topDocs.totalHits.relation);
+      assertEquals(
+          new TotalHits(
+              4,
+              totalHitsThreshold < 4
+                  ? TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO
+                  : TotalHits.Relation.EQUAL_TO),
+          topDocs.totalHits);
     }
 
     reader.close();
@@ -376,13 +379,14 @@ public class TestTopFieldCollector extends LuceneTestCase {
   }
 
   public void testSortNoResults() throws Exception {
-    
+
     // Two Sort criteria to instantiate the multi/single comparators.
-    Sort[] sort = new Sort[] {new Sort(SortField.FIELD_DOC), new Sort() };
-    for(int i = 0; i < sort.length; i++) {
-      TopDocsCollector<Entry> tdc = TopFieldCollector.create(sort[i], 10, Integer.MAX_VALUE);
+    Sort[] sort = new Sort[] {new Sort(SortField.FIELD_DOC), new Sort()};
+    for (int i = 0; i < sort.length; i++) {
+      TopDocsCollector<Entry> tdc =
+          new TopFieldCollectorManager(sort[i], 10, null, Integer.MAX_VALUE, false).newCollector();
       TopDocs td = tdc.topDocs();
-      assertEquals(0, td.totalHits.value);
+      assertEquals(0, td.totalHits.value());
     }
   }
 
@@ -405,45 +409,72 @@ public class TestTopFieldCollector extends LuceneTestCase {
     foo = new BoostQuery(foo, 2);
     Query baz = new TermQuery(new Term("text", "baz"));
     baz = new BoostQuery(baz, 3);
-    Query query = new BooleanQuery.Builder()
-        .add(foo, Occur.SHOULD)
-        .add(bar, Occur.SHOULD)
-        .add(baz, Occur.SHOULD)
-        .build();
-    final IndexSearcher searcher = new IndexSearcher(reader);
-    for (Sort sort : new Sort[] {new Sort(FIELD_SCORE), new Sort(new SortField("f", SortField.Type.SCORE))}) {
-      final TopFieldCollector topCollector = TopFieldCollector.create(sort, TestUtil.nextInt(random(), 1, 2), Integer.MAX_VALUE);
-      final Collector assertingCollector = new Collector() {
-        @Override
-        public LeafCollector getLeafCollector(LeafReaderContext context) throws IOException {
-          final LeafCollector in = topCollector.getLeafCollector(context);
-          return new FilterLeafCollector(in) {
+    Query query =
+        new BooleanQuery.Builder()
+            .add(foo, Occur.SHOULD)
+            .add(bar, Occur.SHOULD)
+            .add(baz, Occur.SHOULD)
+            .build();
+    final IndexSearcher searcher = newSearcher(reader);
+    for (Sort sort :
+        new Sort[] {new Sort(FIELD_SCORE), new Sort(new SortField("f", SortField.Type.SCORE))}) {
+      searcher.search(
+          query,
+          new CollectorManager<>() {
+            TopFieldCollectorManager topFieldCollectorManager =
+                new TopFieldCollectorManager(
+                    sort, TestUtil.nextInt(random(), 1, 2), Integer.MAX_VALUE);
+
             @Override
-            public void setScorer(final Scorable scorer) throws IOException {
-              Scorable s = new FilterScorable(scorer) {
-
-                int lastComputedDoc = -1;
-
+            public Collector newCollector() throws IOException {
+              TopFieldCollector topCollector = topFieldCollectorManager.newCollector();
+              return new Collector() {
                 @Override
-                public float score() throws IOException {
-                  if (lastComputedDoc == docID()) {
-                    throw new AssertionError("Score computed twice on " + docID());
-                  }
-                  lastComputedDoc = docID();
-                  return scorer.score();
+                public LeafCollector getLeafCollector(LeafReaderContext context)
+                    throws IOException {
+                  final LeafCollector in = topCollector.getLeafCollector(context);
+                  return new FilterLeafCollector(in) {
+                    int currentDoc = -1;
+
+                    @Override
+                    public void collect(int doc) throws IOException {
+                      currentDoc = doc;
+                      super.collect(doc);
+                    }
+
+                    @Override
+                    public void setScorer(final Scorable scorer) throws IOException {
+                      Scorable s =
+                          new FilterScorable(scorer) {
+
+                            int lastComputedDoc = -1;
+
+                            @Override
+                            public float score() throws IOException {
+                              if (lastComputedDoc == currentDoc) {
+                                throw new AssertionError("Score computed twice on " + currentDoc);
+                              }
+                              lastComputedDoc = currentDoc;
+                              return scorer.score();
+                            }
+                          };
+                      super.setScorer(s);
+                    }
+                  };
                 }
 
+                @Override
+                public ScoreMode scoreMode() {
+                  return ScoreMode.COMPLETE;
+                }
               };
-              super.setScorer(s);
             }
-          };
-        }
-        @Override
-        public ScoreMode scoreMode() {
-          return topCollector.scoreMode();
-        }
-      };
-      searcher.search(query, assertingCollector);
+
+            @Override
+            public Void reduce(Collection<Collector> collectors) {
+              return null;
+            }
+          });
     }
     reader.close();
     w.close();
@@ -482,7 +513,7 @@ public class TestTopFieldCollector extends LuceneTestCase {
     w.close();
     IndexSearcher searcher = newSearcher(reader);
 
-    for (String queryText : new String[] { "foo", "bar" }) {
+    for (String queryText : new String[] {"foo", "bar"}) {
       Query query = new TermQuery(new Term("f", queryText));
       for (boolean reverse : new boolean[] {false, true}) {
         ScoreDoc[] sortedByDoc = searcher.search(query, 10).scoreDocs;
@@ -494,9 +525,15 @@ public class TestTopFieldCollector extends LuceneTestCase {
         TopFieldCollector.populateScores(sortedByFieldClone, searcher, query);
         for (int i = 0; i < sortedByFieldClone.length; ++i) {
           assertEquals(sortedByFieldClone[i].doc, sortedByField[i].doc);
-          assertSame(((FieldDoc) sortedByFieldClone[i]).fields, ((FieldDoc) sortedByField[i]).fields);
-          assertEquals(sortedByFieldClone[i].score,
-              sortedByDoc[Arrays.binarySearch(sortedByDoc, sortedByFieldClone[i], Comparator.comparingInt(sd -> sd.doc))].score, 0f);
+          assertSame(
+              ((FieldDoc) sortedByFieldClone[i]).fields, ((FieldDoc) sortedByField[i]).fields);
+          assertEquals(
+              sortedByFieldClone[i].score,
+              sortedByDoc[
+                  Arrays.binarySearch(
+                      sortedByDoc, sortedByFieldClone[i], Comparator.comparingInt(sd -> sd.doc))]
+                  .score,
+              0f);
         }
       }
     }
@@ -507,7 +544,8 @@ public class TestTopFieldCollector extends LuceneTestCase {
 
   public void testConcurrentMinScore() throws Exception {
     Directory dir = newDirectory();
-    IndexWriter w = new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(NoMergePolicy.INSTANCE));
+    IndexWriter w =
+        new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(NoMergePolicy.INSTANCE));
     Document doc = new Document();
     w.addDocuments(Arrays.asList(doc, doc, doc, doc, doc));
     w.flush();
@@ -521,7 +559,7 @@ public class TestTopFieldCollector extends LuceneTestCase {
 
     Sort sort = new Sort(SortField.FIELD_SCORE, SortField.FIELD_DOC);
     CollectorManager<TopFieldCollector, TopFieldDocs> manager =
-        TopFieldCollector.createSharedManager(sort, 2, null, 0);
+        new TopFieldCollectorManager(sort, 2, 0);
     TopFieldCollector collector = manager.newCollector();
     TopFieldCollector collector2 = manager.newCollector();
     assertTrue(collector.minScoreAcc == collector2.minScoreAcc);
@@ -529,99 +567,87 @@ public class TestTopFieldCollector extends LuceneTestCase {
     // force the check of the global minimum score on every round
     minValueChecker.modInterval = 0;
 
-    ScoreAndDoc scorer = new ScoreAndDoc();
-    ScoreAndDoc scorer2 = new ScoreAndDoc();
+    Score scorer = new Score();
+    Score scorer2 = new Score();
 
     LeafCollector leafCollector = collector.getLeafCollector(reader.leaves().get(0));
     leafCollector.setScorer(scorer);
     LeafCollector leafCollector2 = collector2.getLeafCollector(reader.leaves().get(1));
     leafCollector2.setScorer(scorer2);
 
-    scorer.doc = 0;
     scorer.score = 3;
     leafCollector.collect(0);
     assertNull(minValueChecker.get());
     assertNull(scorer.minCompetitiveScore);
 
-    scorer2.doc = 0;
     scorer2.score = 6;
     leafCollector2.collect(0);
     assertNull(minValueChecker.get());
     assertNull(scorer2.minCompetitiveScore);
 
-    scorer.doc = 1;
     scorer.score = 2;
     leafCollector.collect(1);
-    assertEquals(2f, minValueChecker.get().score, 0f);
+    assertEquals(2f, minValueChecker.get().score(), 0f);
     assertEquals(2f, scorer.minCompetitiveScore, 0f);
     assertNull(scorer2.minCompetitiveScore);
 
-    scorer2.doc = 1;
     scorer2.score = 9;
     leafCollector2.collect(1);
-    assertEquals(6f, minValueChecker.get().score, 0f);
+    assertEquals(6f, minValueChecker.get().score(), 0f);
     assertEquals(2f, scorer.minCompetitiveScore, 0f);
     assertEquals(6f, scorer2.minCompetitiveScore, 0f);
 
-    scorer2.doc = 2;
     scorer2.score = 7;
     leafCollector2.collect(2);
-    assertEquals(7f, minValueChecker.get().score, 0f);
+    assertEquals(7f, minValueChecker.get().score(), 0f);
     assertEquals(2f, scorer.minCompetitiveScore, 0f);
     assertEquals(7f, scorer2.minCompetitiveScore, 0f);
 
-    scorer2.doc = 3;
     scorer2.score = 1;
     leafCollector2.collect(3);
-    assertEquals(7f, minValueChecker.get().score, 0f);
+    assertEquals(7f, minValueChecker.get().score(), 0f);
     assertEquals(2f, scorer.minCompetitiveScore, 0f);
     assertEquals(7f, scorer2.minCompetitiveScore, 0f);
 
-    scorer.doc = 2;
     scorer.score = 10;
     leafCollector.collect(2);
-    assertEquals(7f, minValueChecker.get().score, 0f);
+    assertEquals(7f, minValueChecker.get().score(), 0f);
     assertEquals(7f, scorer.minCompetitiveScore, 0f);
     assertEquals(7f, scorer2.minCompetitiveScore, 0f);
 
-    scorer.doc = 3;
     scorer.score = 11;
     leafCollector.collect(3);
-    assertEquals(10f, minValueChecker.get().score, 0f);
+    assertEquals(10f, minValueChecker.get().score(), 0f);
     assertEquals(10f, scorer.minCompetitiveScore, 0f);
     assertEquals(7f, scorer2.minCompetitiveScore, 0f);
 
     TopFieldCollector collector3 = manager.newCollector();
     LeafCollector leafCollector3 = collector3.getLeafCollector(reader.leaves().get(2));
-    ScoreAndDoc scorer3 = new ScoreAndDoc();
+    Score scorer3 = new Score();
     leafCollector3.setScorer(scorer3);
     assertEquals(10f, scorer3.minCompetitiveScore, 0f);
 
-    scorer3.doc = 0;
     scorer3.score = 1f;
     leafCollector3.collect(0);
-    assertEquals(10f, minValueChecker.get().score, 0f);
+    assertEquals(10f, minValueChecker.get().score(), 0f);
     assertEquals(10f, scorer3.minCompetitiveScore, 0f);
 
-    scorer.doc = 4;
     scorer.score = 11;
     leafCollector.collect(4);
-    assertEquals(11f, minValueChecker.get().score, 0f);
+    assertEquals(11f, minValueChecker.get().score(), 0f);
     assertEquals(11f, scorer.minCompetitiveScore, 0f);
     assertEquals(7f, scorer2.minCompetitiveScore, 0f);
     assertEquals(10f, scorer3.minCompetitiveScore, 0f);
 
-    scorer3.doc = 1;
     scorer3.score = 2f;
     leafCollector3.collect(1);
-    assertEquals(11f, minValueChecker.get().score, 0f);
+    assertEquals(11f, minValueChecker.get().score(), 0f);
     assertEquals(11f, scorer.minCompetitiveScore, 0f);
     assertEquals(7f, scorer2.minCompetitiveScore, 0f);
     assertEquals(11f, scorer3.minCompetitiveScore, 0f);
 
-
     TopFieldDocs topDocs = manager.reduce(Arrays.asList(collector, collector2, collector3));
-    assertEquals(11, topDocs.totalHits.value);
+    assertEquals(11, topDocs.totalHits.value());
     assertEquals(new TotalHits(11, TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO), topDocs.totalHits);
 
     leafCollector.setScorer(scorer);
@@ -638,8 +664,8 @@ public class TestTopFieldCollector extends LuceneTestCase {
     int numDocs = atLeast(1000);
     for (int i = 0; i < numDocs; ++i) {
       int numAs = 1 + random().nextInt(5);
-      int numBs = random().nextFloat() < 0.5f ?  0 : 1 + random().nextInt(5);
-      int numCs = random().nextFloat() < 0.1f ?  0 : 1 + random().nextInt(5);
+      int numBs = random().nextFloat() < 0.5f ? 0 : 1 + random().nextInt(5);
+      int numCs = random().nextFloat() < 0.1f ? 0 : 1 + random().nextInt(5);
       Document doc = new Document();
       for (int j = 0; j < numAs; ++j) {
         doc.add(new StringField("f", "A", Field.Store.NO));
@@ -654,59 +680,60 @@ public class TestTopFieldCollector extends LuceneTestCase {
     }
     IndexReader indexReader = w.getReader();
     w.close();
-    Query[] queries = new Query[]{
-        new TermQuery(new Term("f", "A")),
-        new TermQuery(new Term("f", "B")),
-        new TermQuery(new Term("f", "C")),
-        new BooleanQuery.Builder()
-            .add(new TermQuery(new Term("f", "A")), BooleanClause.Occur.MUST)
-            .add(new TermQuery(new Term("f", "B")), BooleanClause.Occur.SHOULD)
-            .build()
-    };
+    Query[] queries =
+        new Query[] {
+          new TermQuery(new Term("f", "A")),
+          new TermQuery(new Term("f", "B")),
+          new TermQuery(new Term("f", "C")),
+          new BooleanQuery.Builder()
+              .add(new TermQuery(new Term("f", "A")), BooleanClause.Occur.MUST)
+              .add(new TermQuery(new Term("f", "B")), BooleanClause.Occur.SHOULD)
+              .build()
+        };
     for (Query query : queries) {
-      Sort sort = new Sort(new SortField[]{SortField.FIELD_SCORE, SortField.FIELD_DOC});
-      TopFieldCollector fieldCollector = doSearchWithThreshold(5, 0, query, sort, indexReader);
+      Sort sort = new Sort(new SortField[] {SortField.FIELD_SCORE, SortField.FIELD_DOC});
       TopDocs tdc = doConcurrentSearchWithThreshold(5, 0, query, sort, indexReader);
-      TopDocs tdc2 = fieldCollector.topDocs();
+      TopDocs tdc2 = doSearchWithThreshold(5, 0, query, sort, indexReader);
 
-      assertTrue(tdc.totalHits.value > 0);
-      assertTrue(tdc2.totalHits.value > 0);
+      assertTrue(tdc.totalHits.value() > 0);
+      assertTrue(tdc2.totalHits.value() > 0);
       CheckHits.checkEqual(query, tdc.scoreDocs, tdc2.scoreDocs);
     }
 
     indexReader.close();
     dir.close();
   }
-  
+
   public void testRelationVsTopDocsCount() throws Exception {
     Sort sort = new Sort(SortField.FIELD_SCORE, SortField.FIELD_DOC);
     try (Directory dir = newDirectory();
-        IndexWriter w = new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(NoMergePolicy.INSTANCE))) {
+        IndexWriter w =
+            new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(NoMergePolicy.INSTANCE))) {
       Document doc = new Document();
       doc.add(new TextField("f", "foo bar", Store.NO));
       w.addDocuments(Arrays.asList(doc, doc, doc, doc, doc));
       w.flush();
       w.addDocuments(Arrays.asList(doc, doc, doc, doc, doc));
       w.flush();
-      
+
       try (IndexReader reader = DirectoryReader.open(w)) {
         IndexSearcher searcher = new IndexSearcher(reader);
-        TopFieldCollector collector = TopFieldCollector.create(sort, 2, 10);
-        searcher.search(new TermQuery(new Term("f", "foo")), collector);
-        assertEquals(10, collector.totalHits);
-        assertEquals(TotalHits.Relation.EQUAL_TO, collector.totalHitsRelation);
-        
-        collector = TopFieldCollector.create(sort, 2, 2);
-        searcher.search(new TermQuery(new Term("f", "foo")), collector);
-        assertTrue(10 >= collector.totalHits);
-        assertEquals(TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO, collector.totalHitsRelation);
-        
-        collector = TopFieldCollector.create(sort, 10, 2);
-        searcher.search(new TermQuery(new Term("f", "foo")), collector);
-        assertEquals(10, collector.totalHits);
-        assertEquals(TotalHits.Relation.EQUAL_TO, collector.totalHitsRelation);
+        TopFieldCollectorManager collectorManager =
+            new TopFieldCollectorManager(sort, 2, null, 10, true);
+        TopDocs topDocs = searcher.search(new TermQuery(new Term("f", "foo")), collectorManager);
+        assertEquals(10, topDocs.totalHits.value());
+        assertEquals(TotalHits.Relation.EQUAL_TO, topDocs.totalHits.relation());
+
+        collectorManager = new TopFieldCollectorManager(sort, 2, null, 2, true);
+        topDocs = searcher.search(new TermQuery(new Term("f", "foo")), collectorManager);
+        assertTrue(10 >= topDocs.totalHits.value());
+        assertEquals(TotalHits.Relation.GREATER_THAN_OR_EQUAL_TO, topDocs.totalHits.relation());
+
+        collectorManager = new TopFieldCollectorManager(sort, 10, null, 2, true);
+        topDocs = searcher.search(new TermQuery(new Term("f", "foo")), collectorManager);
+        assertEquals(10, topDocs.totalHits.value());
+        assertEquals(TotalHits.Relation.EQUAL_TO, topDocs.totalHits.relation());
       }
     }
   }
-
 }
