@@ -20,11 +20,12 @@ import java.io.IOException;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.store.Lock;
 import org.apache.lucene.store.LockFactory;
-
+import org.apache.pylucene.ref.PythonObject;
+import org.apache.pylucene.ref.PythonObjectManager;
 
 public class PythonLockFactory extends LockFactory {
 
-    private long pythonObject;
+    private PythonObject pythonObject;
 
     public PythonLockFactory()
     {
@@ -32,21 +33,13 @@ public class PythonLockFactory extends LockFactory {
 
     public void pythonExtension(long pythonObject)
     {
-        this.pythonObject = pythonObject;
+        this.pythonObject = PythonObjectManager.register(pythonObject);
     }
     public long pythonExtension()
     {
-        return this.pythonObject;
+        return this.pythonObject.pythonObject;
     }
 
-    public void finalize()
-        throws Throwable
-    {
-        pythonDecRef();
-    }
-
-    public native void pythonDecRef();
-
     @Override
     public native Lock obtainLock(Directory dir, String lockName)
         throws IOException;
