@@ -16,11 +16,12 @@
 package org.apache.pylucene.util;
 
 import java.util.Iterator;
-
+import org.apache.pylucene.ref.PythonObject;
+import org.apache.pylucene.ref.PythonObjectManager;
 
 public class PythonIterator implements Iterator<Object> {
 
-    private long pythonObject;
+    private PythonObject pythonObject;
 
     public PythonIterator()
     {
@@ -28,21 +29,13 @@ public class PythonIterator implements Iterator<Object> {
 
     public void pythonExtension(long pythonObject)
     {
-        this.pythonObject = pythonObject;
+        this.pythonObject = PythonObjectManager.register(pythonObject);
     }
     public long pythonExtension()
     {
-        return this.pythonObject;
+        return this.pythonObject.pythonObject;
     }
 
-    public void finalize()
-        throws Throwable
-    {
-        pythonDecRef();
-    }
-
-    public native void pythonDecRef();
-
     @Override
     public native boolean hasNext();
     @Override
