@@ -32,36 +32,16 @@ import org.apache.lucene.facet.FacetsCollector;
 import org.apache.lucene.facet.FacetsConfig;
 import org.apache.lucene.facet.FacetsConfig.DimConfig;
 import org.apache.lucene.facet.LabelAndValue;
-import org.apache.lucene.facet.TopOrdAndFloatNumberQueue;
-import org.apache.lucene.facet.TopOrdAndIntNumberQueue;
+import org.apache.lucene.facet.TopOrdAndIntQueue;
 import org.apache.lucene.facet.TopOrdAndNumberQueue;
 import org.apache.lucene.internal.hppc.IntArrayList;
 import org.apache.lucene.internal.hppc.IntIntHashMap;
 import org.apache.lucene.util.PriorityQueue;
 
-/**
- * Base class for all taxonomy-based facets impls.
- *
- * @deprecated Visibility of this class will be reduced to pkg-private in a future version. This
- *     class is meant to host common code as an internal implementation detail to taxonomy
- *     faceting,and is not intended as an extension point for user-created {@code Facets}
- *     implementations. If your code is relying on this, please migrate necessary functionality down
- *     into your own class.
- */
-@Deprecated
-public abstract class TaxonomyFacets extends Facets {
+/** Base class for all taxonomy-based facets impls. */
+abstract class TaxonomyFacets extends Facets {
   /** Intermediate result to store top children for a given path before resolving labels, etc. */
-  static class TopChildrenForPath {
-    Number pathValue;
-    int childCount;
-    TopOrdAndNumberQueue childQueue;
-
-    public TopChildrenForPath(Number pathValue, int childCount, TopOrdAndNumberQueue childQueue) {
-      this.pathValue = pathValue;
-      this.childCount = childCount;
-      this.childQueue = childQueue;
-    }
-  }
+  record TopChildrenForPath(Number pathValue, int childCount, TopOrdAndNumberQueue childQueue) {}
 
   private static class DimValue {
     String dim;
@@ -76,13 +56,13 @@ public abstract class TaxonomyFacets extends Facets {
   }
 
   /** Index field name provided to the constructor. */
-  protected final String indexFieldName;
+  final String indexFieldName;
 
   /** {@code TaxonomyReader} provided to the constructor. */
-  protected final TaxonomyReader taxoReader;
+  final TaxonomyReader taxoReader;
 
   /** {@code FacetsConfig} provided to the constructor. */
-  protected final FacetsConfig config;
+  final FacetsConfig config;
 
   /** {@code FacetsCollector} provided to the constructor. */
   final FacetsCollector fc;
@@ -96,18 +76,6 @@ public abstract class TaxonomyFacets extends Facets {
   /** Maps an ordinal to its parent, or -1 if there is no parent (root node). */
   final ParallelTaxonomyArrays.IntArray parents;
 
-  /**
-   * Constructor without a {@link FacetsCollector} - we don't have access to the hits, so we have to
-   * assume there are hits when initializing internal data structures.
-   *
-   * @deprecated To be removed in Lucene 10.
-   */
-  @Deprecated
-  protected TaxonomyFacets(String indexFieldName, TaxonomyReader taxoReader, FacetsConfig config)
-      throws IOException {
-    this(indexFieldName, taxoReader, config, null);
-  }
-
   /** Dense ordinal counts. */
   int[] counts;
 
@@ -117,13 +85,9 @@ public abstract class TaxonomyFacets extends Facets {
   /** Have value counters been initialized. */
   boolean initialized;
 
-  /** Defines comparison between aggregated values. */
   protected Comparator<Number> valueComparator;
 
-  /**
-   * Constructor with a {@link FacetsCollector}, allowing lazy initialization of internal data
-   * structures.
-   */
+  /** Sole constructor. */
   TaxonomyFacets(
       String indexFieldName, TaxonomyReader taxoReader, FacetsConfig config, FacetsCollector fc)
       throws IOException {
@@ -136,7 +100,7 @@ public abstract class TaxonomyFacets extends Facets {
   }
 
   /** Return true if a sparse hash table should be used for counting, instead of a dense int[]. */
-  protected boolean useHashTable(FacetsCollector fc, TaxonomyReader taxoReader) {
+  private boolean useHashTable(FacetsCollector fc, TaxonomyReader taxoReader) {
     if (taxoReader.getSize() < 1024) {
       // small number of unique values: use an array
       return false;
@@ -150,15 +114,14 @@ public abstract class TaxonomyFacets extends Facets {
     int maxDoc = 0;
     int sumTotalHits = 0;
     for (FacetsCollector.MatchingDocs docs : fc.getMatchingDocs()) {
-      sumTotalHits += docs.totalHits;
-      maxDoc += docs.context.reader().maxDoc();
+      sumTotalHits += docs.totalHits();
+      maxDoc += docs.context().reader().maxDoc();
     }
 
     // if our result set is < 10% of the index, we collect sparsely (use hash map):
     return sumTotalHits < maxDoc / 10;
   }
 
-  /** If not done already, initialize the data structures storing counts. */
   protected void initializeValueCounters() {
     if (initialized) {
       return;
@@ -255,7 +218,7 @@ public abstract class TaxonomyFacets extends Facets {
    * @throws IllegalArgumentException if the provided dimension was manually configured, but its
    *     {@link DimConfig#indexFieldName} does not match {@link #indexFieldName}.
    */
-  protected DimConfig verifyDim(String dim) {
+  DimConfig verifyDim(String dim) {
     FacetsConfig.DimConfig dimConfig = config.getDimConfig(dim);
     if (config.isDimConfigured(dim) == true
         && dimConfig.indexFieldName.equals(indexFieldName) == false) {
@@ -281,11 +244,11 @@ public abstract class TaxonomyFacets extends Facets {
   }
 
   /**
-   * Return a {@link TopOrdAndNumberQueue} of the appropriate type, i.e. a {@link
-   * TopOrdAndIntNumberQueue} or a {@link TopOrdAndFloatNumberQueue}.
+   * Return a {@link TopOrdAndNumberQueue} of the appropriate type, i.e. a {@link TopOrdAndIntQueue}
+   * or a {@link org.apache.lucene.facet.TopOrdAndFloatQueue}.
    */
   protected TopOrdAndNumberQueue makeTopOrdAndNumberQueue(int topN) {
-    return new TopOrdAndIntNumberQueue(Math.min(taxoReader.getSize(), topN));
+    return new TopOrdAndIntQueue(Math.min(taxoReader.getSize(), topN));
   }
 
   // TODO: We don't need this if we're okay with having an integer -1 in the results even for float
@@ -296,7 +259,7 @@ public abstract class TaxonomyFacets extends Facets {
   }
 
   /** Rolls up any single-valued hierarchical dimensions. */
-  protected void rollup() throws IOException {
+  void rollup() throws IOException {
     if (initialized == false) {
       return;
     }
@@ -450,15 +413,10 @@ public abstract class TaxonomyFacets extends Facets {
     return new FacetResult(dim, path, aggregatedValue, labelValues, ordinals.size());
   }
 
-  /**
-   * Set the value for a {@link org.apache.lucene.facet.TopOrdAndNumberQueue.OrdAndValue} to the one
-   * corresponding to the given ordinal.
-   */
   protected void setIncomingValue(TopOrdAndNumberQueue.OrdAndValue incomingOrdAndValue, int ord) {
-    ((TopOrdAndIntNumberQueue.OrdAndInt) incomingOrdAndValue).value = getCount(ord);
+    ((TopOrdAndIntQueue.OrdAndInt) incomingOrdAndValue).value = getCount(ord);
   }
 
-  /** Insert an ordinal and the value corresponding to it into the queue. */
   protected TopOrdAndNumberQueue.OrdAndValue insertIntoQueue(
       TopOrdAndNumberQueue q, TopOrdAndNumberQueue.OrdAndValue incomingOrdAndValue, int ord) {
     if (incomingOrdAndValue == null) {
@@ -471,16 +429,12 @@ public abstract class TaxonomyFacets extends Facets {
     return incomingOrdAndValue;
   }
 
-  /** An accumulator for an aggregated value. */
   protected abstract static class AggregatedValue {
     /** Aggregate the value corresponding to the given ordinal into this value. */
     public abstract void aggregate(int ord);
 
     /** Retrieve the encapsulated value. */
     public abstract Number get();
-
-    /** Default constructor. */
-    public AggregatedValue() {}
   }
 
   private class AggregatedCount extends AggregatedValue {
@@ -501,7 +455,6 @@ public abstract class TaxonomyFacets extends Facets {
     }
   }
 
-  /** Initialize an accumulator. */
   protected AggregatedValue newAggregatedValue() {
     return new AggregatedCount(0);
   }
@@ -702,7 +655,7 @@ public abstract class TaxonomyFacets extends Facets {
               intermediateResults = new HashMap<>();
             }
             intermediateResults.put(dim, topChildrenForPath);
-            dimValue = topChildrenForPath.pathValue;
+            dimValue = topChildrenForPath.pathValue();
           }
           if (valueComparator.compare(dimValue, 0) != 0) {
             if (pq.size() < topNDims) {
