@@ -43,7 +43,6 @@ import org.apache.lucene.index.IndexReader;
 import org.apache.lucene.index.LeafReaderContext;
 import org.apache.lucene.index.Term;
 import org.apache.lucene.search.DocIdSetIterator;
-import org.apache.lucene.search.DoubleValuesSource;
 import org.apache.lucene.search.IndexSearcher;
 import org.apache.lucene.search.MatchAllDocsQuery;
 import org.apache.lucene.search.Query;
@@ -215,7 +214,9 @@ public class TestTaxonomyFacetAssociations extends FacetTestCase {
     FacetsCollector fc =
         searcher.search(new TermQuery(new Term("match", "yes")), new FacetsCollectorManager());
 
-    Facets facets = getIntSumFacets("$facets.int", taxoReader, config, fc);
+    Facets facets =
+        new TaxonomyFacetIntAssociations(
+            "$facets.int", taxoReader, config, fc, AssociationAggregationFunction.SUM);
     assertEquals(
         "dim=int path=[] value=-1 childCount=2\n  a (200)\n  b (150)\n",
         facets.getTopChildren(10, "int").toString());
@@ -294,7 +295,9 @@ public class TestTaxonomyFacetAssociations extends FacetTestCase {
     FacetsCollector fc =
         searcher.search(new TermQuery(new Term("match", "yes")), new FacetsCollectorManager());
 
-    Facets facets = getFloatSumFacets("$facets.float", taxoReader, config, fc, null);
+    Facets facets =
+        new TaxonomyFacetFloatAssociations(
+            "$facets.float", taxoReader, config, fc, AssociationAggregationFunction.SUM);
     assertEquals(
         "dim=float path=[] value=-1.0 childCount=2\n  a (50.0)\n  b (9.999995)\n",
         facets.getTopChildren(10, "float").toString());
@@ -388,7 +391,9 @@ public class TestTaxonomyFacetAssociations extends FacetTestCase {
     FacetsCollector fc =
         searcher.search(new TermQuery(new Term("match", "yes")), new FacetsCollectorManager());
 
-    Facets facets = getFloatSumFacets("$facets.float", taxoReader, config, fc, null);
+    Facets facets =
+        new TaxonomyFacetFloatAssociations(
+            "$facets.float", taxoReader, config, fc, AssociationAggregationFunction.SUM);
     assertEquals(
         "Wrong count for category 'a'!",
         50f,
@@ -400,7 +405,9 @@ public class TestTaxonomyFacetAssociations extends FacetTestCase {
         facets.getSpecificValue("float", "b").floatValue(),
         0.00001);
 
-    facets = getIntSumFacets("$facets.int", taxoReader, config, fc);
+    facets =
+        new TaxonomyFacetIntAssociations(
+            "$facets.int", taxoReader, config, fc, AssociationAggregationFunction.SUM);
     assertEquals(
         "Wrong count for category 'a'!", 200, facets.getSpecificValue("int", "a").intValue());
     assertEquals(
@@ -411,7 +418,9 @@ public class TestTaxonomyFacetAssociations extends FacetTestCase {
     IndexSearcher searcher = newSearcher(reader);
     FacetsCollector fc =
         searcher.search(new TermQuery(new Term("match", "yes")), new FacetsCollectorManager());
-    Facets facets = getFloatSumFacets("wrong_field", taxoReader, config, fc, null);
+    Facets facets =
+        new TaxonomyFacetFloatAssociations(
+            "wrong_field", taxoReader, config, fc, AssociationAggregationFunction.SUM);
     expectThrows(IllegalArgumentException.class, () -> facets.getSpecificValue("float"));
 
     expectThrows(IllegalArgumentException.class, () -> facets.getTopChildren(10, "float"));
@@ -478,7 +487,9 @@ public class TestTaxonomyFacetAssociations extends FacetTestCase {
     q.add("int", "b");
     FacetsCollector fc = searcher.search(q, new FacetsCollectorManager());
 
-    Facets facets = getIntSumFacets("$facets.int", taxoReader, config, fc);
+    Facets facets =
+        new TaxonomyFacetIntAssociations(
+            "$facets.int", taxoReader, config, fc, AssociationAggregationFunction.SUM);
     assertEquals(
         "dim=int path=[] value=-1 childCount=2\n  b (150)\n  a (100)\n",
         facets.getTopChildren(10, "int").toString());
@@ -502,41 +513,6 @@ public class TestTaxonomyFacetAssociations extends FacetTestCase {
         "Wrong count for category 'b'!", 150, facets.getSpecificValue("int", "b").intValue());
   }
 
-  private Facets getIntSumFacets(
-      String fieldName, TaxonomyReader taxoReader, FacetsConfig facetsConfig, FacetsCollector fc)
-      throws IOException {
-    if (random().nextBoolean()) {
-      return new TaxonomyFacetSumIntAssociations(fieldName, taxoReader, facetsConfig, fc);
-    } else {
-      return new TaxonomyFacetIntAssociations(
-          fieldName, taxoReader, facetsConfig, fc, AssociationAggregationFunction.SUM);
-    }
-  }
-
-  private Facets getFloatSumFacets(
-      String fieldName,
-      TaxonomyReader taxoReader,
-      FacetsConfig facetsConfig,
-      FacetsCollector fc,
-      DoubleValuesSource dvs)
-      throws IOException {
-    if (random().nextBoolean()) {
-      if (dvs == null) {
-        return new TaxonomyFacetSumFloatAssociations(fieldName, taxoReader, facetsConfig, fc);
-      } else {
-        return new TaxonomyFacetSumValueSource(fieldName, taxoReader, facetsConfig, fc, dvs);
-      }
-    } else {
-      if (dvs == null) {
-        return new TaxonomyFacetFloatAssociations(
-            fieldName, taxoReader, facetsConfig, fc, AssociationAggregationFunction.SUM);
-      } else {
-        return new TaxonomyFacetFloatAssociations(
-            fieldName, taxoReader, facetsConfig, fc, AssociationAggregationFunction.SUM, dvs);
-      }
-    }
-  }
-
   public void testNonPositiveAggregations() throws IOException {
     Directory dir = newDirectory();
     Directory taxoDir = newDirectory();
