@@ -24,7 +24,6 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-
 import org.apache.lucene.analysis.CachingTokenFilter;
 import org.apache.lucene.analysis.TokenStream;
 import org.apache.lucene.index.BinaryDocValues;
@@ -40,11 +39,21 @@ import org.apache.lucene.index.Terms;
 import org.apache.lucene.index.memory.MemoryIndex;
 import org.apache.lucene.queries.CommonTermsQuery;
 import org.apache.lucene.queries.function.FunctionScoreQuery;
+import org.apache.lucene.queries.spans.FieldMaskingSpanQuery;
+import org.apache.lucene.queries.spans.SpanFirstQuery;
+import org.apache.lucene.queries.spans.SpanNearQuery;
+import org.apache.lucene.queries.spans.SpanNotQuery;
+import org.apache.lucene.queries.spans.SpanOrQuery;
+import org.apache.lucene.queries.spans.SpanQuery;
+import org.apache.lucene.queries.spans.SpanTermQuery;
+import org.apache.lucene.queries.spans.SpanWeight;
+import org.apache.lucene.queries.spans.Spans;
 import org.apache.lucene.search.BooleanClause;
 import org.apache.lucene.search.BooleanQuery;
 import org.apache.lucene.search.BoostQuery;
 import org.apache.lucene.search.ConstantScoreQuery;
 import org.apache.lucene.search.DisjunctionMaxQuery;
+import org.apache.lucene.search.FieldExistsQuery;
 import org.apache.lucene.search.IndexSearcher;
 import org.apache.lucene.search.MatchAllDocsQuery;
 import org.apache.lucene.search.MultiPhraseQuery;
@@ -55,25 +64,17 @@ import org.apache.lucene.search.QueryVisitor;
 import org.apache.lucene.search.ScoreMode;
 import org.apache.lucene.search.SynonymQuery;
 import org.apache.lucene.search.TermQuery;
-import org.apache.lucene.search.spans.FieldMaskingSpanQuery;
-import org.apache.lucene.search.spans.SpanFirstQuery;
-import org.apache.lucene.search.spans.SpanNearQuery;
-import org.apache.lucene.search.spans.SpanNotQuery;
-import org.apache.lucene.search.spans.SpanOrQuery;
-import org.apache.lucene.search.spans.SpanQuery;
-import org.apache.lucene.search.spans.SpanTermQuery;
-import org.apache.lucene.search.spans.SpanWeight;
-import org.apache.lucene.search.spans.Spans;
 import org.apache.lucene.util.Bits;
 import org.apache.lucene.util.IOUtils;
 
 /**
- * Class used to extract {@link WeightedSpanTerm}s from a {@link Query} based on whether 
- * {@link Term}s from the {@link Query} are contained in a supplied {@link TokenStream}.
+ * Class used to extract {@link WeightedSpanTerm}s from a {@link Query} based on whether {@link
+ * Term}s from the {@link Query} are contained in a supplied {@link TokenStream}.
+ *
+ * <p>In order to support additional, by default unsupported queries, subclasses can override {@link
+ * #extract(Query, float, Map)} for extracting wrapped or delegate queries and {@link
+ * #extractUnknownQuery(Query, Map)} to process custom leaf queries:
  *
- * In order to support additional, by default unsupported queries, subclasses can override
- * {@link #extract(Query, float, Map)} for extracting wrapped or delegate queries and
- * {@link #extractUnknownQuery(Query, Map)} to process custom leaf queries:
  * <pre>
  * <code>
  *    WeightedSpanTermExtractor extractor = new WeightedSpanTermExtractor() {
@@ -99,8 +100,8 @@ import org.apache.lucene.util.IOUtils;
 public class WeightedSpanTermExtractor {
 
   private String fieldName;
-  private TokenStream tokenStream;//set subsequent to getWeightedSpanTerms* methods
-  private String defaultField;
+  private TokenStream tokenStream; // set after getWeightedSpanTerms* methods
+  private final String defaultField;
   private boolean expandMultiTermQuery;
   private boolean cachedTokenStream;
   private boolean wrapToCaching = true;
@@ -117,22 +118,22 @@ public class WeightedSpanTermExtractor {
   }
 
   /**
-   * Fills a <code>Map</code> with {@link WeightedSpanTerm}s using the terms from the supplied <code>Query</code>.
-   * 
-   * @param query
-   *          Query to extract Terms from
-   * @param terms
-   *          Map to place created WeightedSpanTerms in
+   * Fills a <code>Map</code> with {@link WeightedSpanTerm}s using the terms from the supplied
+   * <code>Query</code>.
+   *
+   * @param query Query to extract Terms from
+   * @param terms Map to place created WeightedSpanTerms in
    * @throws IOException If there is a low-level I/O error
    */
-  protected void extract(Query query, float boost, Map<String,WeightedSpanTerm> terms) throws IOException {
+  protected void extract(Query query, float boost, Map<String, WeightedSpanTerm> terms)
+      throws IOException {
     if (query instanceof BoostQuery) {
       BoostQuery boostQuery = (BoostQuery) query;
       extract(boostQuery.getQuery(), boost * boostQuery.getBoost(), terms);
     } else if (query instanceof BooleanQuery) {
       for (BooleanClause clause : (BooleanQuery) query) {
         if (!clause.isProhibited()) {
-          extract(clause.getQuery(), boost, terms);
+          extract(clause.query(), boost, terms);
         }
       }
     } else if (query instanceof PhraseQuery) {
@@ -151,13 +152,15 @@ public class WeightedSpanTermExtractor {
         int[] positions = phraseQuery.getPositions();
         if (positions.length >= 2) {
           // positions are in increasing order.   max(0,...) is just a safeguard.
-          positionGaps = Math.max(0, positions[positions.length - 1] - positions[0] - positions.length + 1);
+          positionGaps =
+              Math.max(0, positions[positions.length - 1] - positions[0] - positions.length + 1);
         }
 
-        //if original slop is 0 then require inOrder
+        // if original slop is 0 then require inOrder
         boolean inorder = (phraseQuery.getSlop() == 0);
 
-        SpanNearQuery sp = new SpanNearQuery(clauses, phraseQuery.getSlop() + positionGaps, inorder);
+        SpanNearQuery sp =
+            new SpanNearQuery(clauses, phraseQuery.getSlop() + positionGaps, inorder);
         extractWeightedSpanTerms(terms, sp, boost);
       }
     } else if (query instanceof TermQuery || query instanceof SynonymQuery) {
@@ -170,7 +173,7 @@ public class WeightedSpanTermExtractor {
         extract(q, boost, terms);
       }
     } else if (query instanceof CommonTermsQuery) {
-      // specialized since rewriting would change the result query 
+      // specialized since rewriting would change the result query
       // this query is index sensitive.
       extractWeightedTerms(terms, query, boost);
     } else if (query instanceof DisjunctionMaxQuery) {
@@ -190,7 +193,7 @@ public class WeightedSpanTermExtractor {
           }
         }
 
-        @SuppressWarnings({"unchecked","rawtypes"})
+        @SuppressWarnings({"unchecked", "rawtypes"})
         final List<SpanQuery>[] disjunctLists = new List[maxPosition + 1];
         int distinctPositions = 0;
 
@@ -211,8 +214,8 @@ public class WeightedSpanTermExtractor {
         final SpanQuery[] clauses = new SpanQuery[distinctPositions];
         for (List<SpanQuery> disjuncts : disjunctLists) {
           if (disjuncts != null) {
-            clauses[position++] = new SpanOrQuery(disjuncts
-                .toArray(new SpanQuery[disjuncts.size()]));
+            clauses[position++] =
+                new SpanOrQuery(disjuncts.toArray(new SpanQuery[disjuncts.size()]));
           } else {
             ++positionGaps;
           }
@@ -229,27 +232,30 @@ public class WeightedSpanTermExtractor {
         }
       }
     } else if (query instanceof MatchAllDocsQuery) {
-      //nothing
+      // nothing
+    } else if (query instanceof FieldExistsQuery) {
+      // nothing
     } else if (query instanceof FunctionScoreQuery) {
       extract(((FunctionScoreQuery) query).getWrappedQuery(), boost, terms);
     } else if (isQueryUnsupported(query.getClass())) {
       // nothing
     } else {
-      if (query instanceof MultiTermQuery &&
-          (!expandMultiTermQuery || !fieldNameComparator(((MultiTermQuery)query).getField()))) {
+      if (query instanceof MultiTermQuery
+          && (!expandMultiTermQuery || !fieldNameComparator(((MultiTermQuery) query).getField()))) {
         return;
       }
-      Query origQuery = query;
       final IndexReader reader = getLeafContext().reader();
       Query rewritten;
       if (query instanceof MultiTermQuery) {
-        rewritten = MultiTermQuery.SCORING_BOOLEAN_REWRITE.rewrite(reader, (MultiTermQuery) query);
+        rewritten =
+            MultiTermQuery.SCORING_BOOLEAN_REWRITE.rewrite(
+                new IndexSearcher(reader), (MultiTermQuery) query);
       } else {
-        rewritten = origQuery.rewrite(reader);
+        rewritten = query.rewrite(new IndexSearcher(reader));
       }
-      if (rewritten != origQuery) {
-        // only rewrite once and then flatten again - the rewritten query could have a special treatment
-        // if this method is overwritten in a subclass or above in the next recursion
+      if (rewritten != query) {
+        // only rewrite once and then flatten again - the rewritten query could have a special
+        // treatment if this method is overwritten in a subclass or above in the next recursion
         extract(rewritten, boost, terms);
       } else {
         extractUnknownQuery(query, terms);
@@ -269,86 +275,69 @@ public class WeightedSpanTermExtractor {
     return false;
   }
 
-  protected void extractUnknownQuery(Query query,
-      Map<String, WeightedSpanTerm> terms) throws IOException {
-    
+  protected void extractUnknownQuery(Query query, Map<String, WeightedSpanTerm> terms)
+      throws IOException {
+
     // for sub-classing to extract custom queries
   }
 
   /**
-   * Fills a <code>Map</code> with {@link WeightedSpanTerm}s using the terms from the supplied <code>SpanQuery</code>.
-   * 
-   * @param terms
-   *          Map to place created WeightedSpanTerms in
-   * @param spanQuery
-   *          SpanQuery to extract Terms from
+   * Fills a <code>Map</code> with {@link WeightedSpanTerm}s using the terms from the supplied
+   * <code>SpanQuery</code>.
+   *
+   * @param terms Map to place created WeightedSpanTerms in
+   * @param spanQuery SpanQuery to extract Terms from
    * @throws IOException If there is a low-level I/O error
    */
-  protected void extractWeightedSpanTerms(Map<String,WeightedSpanTerm> terms, SpanQuery spanQuery, float boost) throws IOException {
-    Set<String> fieldNames;
+  protected void extractWeightedSpanTerms(
+      Map<String, WeightedSpanTerm> terms, SpanQuery spanQuery, float boost) throws IOException {
 
-    if (fieldName == null) {
-      fieldNames = new HashSet<>();
-      collectSpanQueryFields(spanQuery, fieldNames);
-    } else {
-      fieldNames = new HashSet<>(1);
-      fieldNames.add(fieldName);
-    }
-    // To support the use of the default field name
-    if (defaultField != null) {
-      fieldNames.add(defaultField);
-    }
-    
-    Map<String, SpanQuery> queries = new HashMap<>();
- 
-    Set<Term> nonWeightedTerms = new HashSet<>();
-    final boolean mustRewriteQuery = mustRewriteQuery(spanQuery);
-    final IndexSearcher searcher = new IndexSearcher(getLeafContext());
-    searcher.setQueryCache(null);
-    if (mustRewriteQuery) {
-      for (final String field : fieldNames) {
-        final SpanQuery rewrittenQuery = (SpanQuery) spanQuery.rewrite(getLeafContext().reader());
-        queries.put(field, rewrittenQuery);
-        rewrittenQuery.visit(QueryVisitor.termCollector(nonWeightedTerms));
-      }
-    } else {
-      spanQuery.visit(QueryVisitor.termCollector(nonWeightedTerms));
-    }
-
-    List<PositionSpan> spanPositions = new ArrayList<>();
-
-    for (final String field : fieldNames) {
-      final SpanQuery q;
-      if (mustRewriteQuery) {
-        q = queries.get(field);
-      } else {
-        q = spanQuery;
-      }
-      LeafReaderContext context = getLeafContext();
-      SpanWeight w = (SpanWeight) searcher.createWeight(searcher.rewrite(q), ScoreMode.COMPLETE_NO_SCORES, 1);
-      Bits acceptDocs = context.reader().getLiveDocs();
-      final Spans spans = w.getSpans(context, SpanWeight.Postings.POSITIONS);
-      if (spans == null) {
-        return;
-      }
-
-      // collect span positions
-      while (spans.nextDoc() != Spans.NO_MORE_DOCS) {
-        if (acceptDocs != null && acceptDocs.get(spans.docID()) == false) {
-          continue;
-        }
-        while (spans.nextStartPosition() != Spans.NO_MORE_POSITIONS) {
-          spanPositions.add(new PositionSpan(spans.startPosition(), spans.endPosition() - 1));
-        }
-      }
-    }
-
-    if (spanPositions.size() == 0) {
-      // no spans found
+    Set<String> queryFieldNames = new HashSet<>();
+    collectSpanQueryFields(spanQuery, queryFieldNames);
+    if (fieldName != null
+        && queryFieldNames.contains(fieldName) == false
+        && (defaultField == null || queryFieldNames.contains(defaultField) == false)) {
       return;
     }
 
-    for (final Term queryTerm :  nonWeightedTerms) {
+    final boolean mustRewriteQuery = mustRewriteQuery(spanQuery);
+    final IndexSearcher searcher = new IndexSearcher(getLeafContext());
+    searcher.setQueryCache(null);
+    final SpanQuery query = mustRewriteQuery ? (SpanQuery) searcher.rewrite(spanQuery) : spanQuery;
+
+    final Set<Term> nonWeightedTerms = new HashSet<>();
+    query.visit(QueryVisitor.termCollector(nonWeightedTerms));
+    if (nonWeightedTerms.isEmpty()) {
+      return;
+    }
+
+    final List<PositionSpan> spanPositions = new ArrayList<>();
+
+    LeafReaderContext context = getLeafContext();
+    SpanWeight w =
+        (SpanWeight)
+            searcher.createWeight(searcher.rewrite(query), ScoreMode.COMPLETE_NO_SCORES, 1);
+    final Spans spans = w.getSpans(context, SpanWeight.Postings.POSITIONS);
+    if (spans == null) {
+      return;
+    }
+
+    final Bits acceptDocs = context.reader().getLiveDocs();
+    // collect span positions
+    while (spans.nextDoc() != Spans.NO_MORE_DOCS) {
+      if (acceptDocs != null && acceptDocs.get(spans.docID()) == false) {
+        continue;
+      }
+      while (spans.nextStartPosition() != Spans.NO_MORE_POSITIONS) {
+        spanPositions.add(new PositionSpan(spans.startPosition(), spans.endPosition() - 1));
+      }
+    }
+
+    if (spanPositions.isEmpty()) {
+      return;
+    }
+
+    for (final Term queryTerm : nonWeightedTerms) {
 
       if (fieldNameComparator(queryTerm.field())) {
         WeightedSpanTerm weightedSpanTerm = terms.get(queryTerm.text());
@@ -368,15 +357,15 @@ public class WeightedSpanTermExtractor {
   }
 
   /**
-   * Fills a <code>Map</code> with {@link WeightedSpanTerm}s using the terms from the supplied <code>Query</code>.
-   * 
-   * @param terms
-   *          Map to place created WeightedSpanTerms in
-   * @param query
-   *          Query to extract Terms from
+   * Fills a <code>Map</code> with {@link WeightedSpanTerm}s using the terms from the supplied
+   * <code>Query</code>.
+   *
+   * @param terms Map to place created WeightedSpanTerms in
+   * @param query Query to extract Terms from
    * @throws IOException If there is a low-level I/O error
    */
-  protected void extractWeightedTerms(Map<String,WeightedSpanTerm> terms, Query query, float boost) throws IOException {
+  protected void extractWeightedTerms(Map<String, WeightedSpanTerm> terms, Query query, float boost)
+      throws IOException {
     Set<Term> nonWeightedTerms = new HashSet<>();
     final IndexSearcher searcher = new IndexSearcher(getLeafContext());
     searcher.rewrite(query).visit(QueryVisitor.termCollector(nonWeightedTerms));
@@ -390,13 +379,11 @@ public class WeightedSpanTermExtractor {
     }
   }
 
-  /**
-   * Necessary to implement matches for queries against <code>defaultField</code>
-   */
+  /** Necessary to implement matches for queries against <code>defaultField</code> */
   protected boolean fieldNameComparator(String fieldNameToCheck) {
-    boolean rv = fieldName == null || fieldName.equals(fieldNameToCheck)
-      || (defaultField != null && defaultField.equals(fieldNameToCheck));
-    return rv;
+    return fieldName == null
+        || fieldName.equals(fieldNameToCheck)
+        || (defaultField != null && defaultField.equals(fieldNameToCheck));
   }
 
   protected LeafReaderContext getLeafContext() throws IOException {
@@ -408,20 +395,23 @@ public class WeightedSpanTermExtractor {
         cacheIt = false;
         Terms termVectorTerms = ((TokenStreamFromTermVector) tokenStream).getTermVectorTerms();
         if (termVectorTerms.hasPositions() && termVectorTerms.hasOffsets()) {
-          internalReader = new TermVectorLeafReader(DelegatingLeafReader.FIELD_NAME, termVectorTerms);
+          internalReader =
+              new TermVectorLeafReader(DelegatingLeafReader.FIELD_NAME, termVectorTerms);
         }
       }
 
       // Use MemoryIndex (index/invert this tokenStream now)
       if (internalReader == null) {
-        final MemoryIndex indexer = new MemoryIndex(true, usePayloads);//offsets and payloads
+        final MemoryIndex indexer = new MemoryIndex(true, usePayloads); // offsets and payloads
         if (cacheIt) {
           assert !cachedTokenStream;
-          tokenStream = new CachingTokenFilter(new OffsetLimitTokenFilter(tokenStream, maxDocCharsToAnalyze));
+          tokenStream =
+              new CachingTokenFilter(new OffsetLimitTokenFilter(tokenStream, maxDocCharsToAnalyze));
           cachedTokenStream = true;
           indexer.addField(DelegatingLeafReader.FIELD_NAME, tokenStream);
         } else {
-          indexer.addField(DelegatingLeafReader.FIELD_NAME,
+          indexer.addField(
+              DelegatingLeafReader.FIELD_NAME,
               new OffsetLimitTokenFilter(tokenStream, maxDocCharsToAnalyze));
         }
         final IndexSearcher searcher = indexer.createSearcher();
@@ -429,13 +419,13 @@ public class WeightedSpanTermExtractor {
         internalReader = ((LeafReaderContext) searcher.getTopReaderContext()).reader();
       }
 
-      //Now wrap it so we always use a common field.
+      // Now wrap it so we always use a common field.
       this.internalReader = new DelegatingLeafReader(internalReader);
     }
 
     return internalReader.getContext();
   }
-  
+
   /*
    * This reader will just delegate every call to a single field in the wrapped
    * LeafReader. This way we only need to build this field once rather than
@@ -450,7 +440,7 @@ public class WeightedSpanTermExtractor {
 
     @Override
     public FieldInfos getFieldInfos() {
-      throw new UnsupportedOperationException();//TODO merge them
+      throw new UnsupportedOperationException(); // TODO merge them
     }
 
     @Override
@@ -462,17 +452,17 @@ public class WeightedSpanTermExtractor {
     public NumericDocValues getNumericDocValues(String field) throws IOException {
       return super.getNumericDocValues(FIELD_NAME);
     }
-    
+
     @Override
     public BinaryDocValues getBinaryDocValues(String field) throws IOException {
       return super.getBinaryDocValues(FIELD_NAME);
     }
-    
+
     @Override
     public SortedDocValues getSortedDocValues(String field) throws IOException {
       return super.getSortedDocValues(FIELD_NAME);
     }
-    
+
     @Override
     public NumericDocValues getNormValues(String field) throws IOException {
       return super.getNormValues(FIELD_NAME);
@@ -490,32 +480,34 @@ public class WeightedSpanTermExtractor {
   }
 
   /**
-   * Creates a Map of <code>WeightedSpanTerms</code> from the given <code>Query</code> and <code>TokenStream</code>.
-   * 
+   * Creates a Map of <code>WeightedSpanTerms</code> from the given <code>Query</code> and <code>
+   * TokenStream</code>.
+   *
    * @param query that caused hit
    * @param tokenStream of text to be highlighted
    * @return Map containing WeightedSpanTerms
    * @throws IOException If there is a low-level I/O error
    */
-  public Map<String,WeightedSpanTerm> getWeightedSpanTerms(Query query, float boost, TokenStream tokenStream)
-      throws IOException {
+  public Map<String, WeightedSpanTerm> getWeightedSpanTerms(
+      Query query, float boost, TokenStream tokenStream) throws IOException {
     return getWeightedSpanTerms(query, boost, tokenStream, null);
   }
 
   /**
-   * Creates a Map of <code>WeightedSpanTerms</code> from the given <code>Query</code> and <code>TokenStream</code>.
-   * 
+   * Creates a Map of <code>WeightedSpanTerms</code> from the given <code>Query</code> and <code>
+   * TokenStream</code>.
+   *
    * @param query that caused hit
    * @param tokenStream of text to be highlighted
    * @param fieldName restricts Term's used based on field name
    * @return Map containing WeightedSpanTerms
    * @throws IOException If there is a low-level I/O error
    */
-  public Map<String,WeightedSpanTerm> getWeightedSpanTerms(Query query, float boost, TokenStream tokenStream,
-      String fieldName) throws IOException {
+  public Map<String, WeightedSpanTerm> getWeightedSpanTerms(
+      Query query, float boost, TokenStream tokenStream, String fieldName) throws IOException {
     this.fieldName = fieldName;
 
-    Map<String,WeightedSpanTerm> terms = new PositionCheckingMap<>();
+    Map<String, WeightedSpanTerm> terms = new PositionCheckingMap<>();
     this.tokenStream = tokenStream;
     try {
       extract(query, boost, terms);
@@ -527,9 +519,10 @@ public class WeightedSpanTermExtractor {
   }
 
   /**
-   * Creates a Map of <code>WeightedSpanTerms</code> from the given <code>Query</code> and <code>TokenStream</code>. Uses a supplied
-   * <code>IndexReader</code> to properly weight terms (for gradient highlighting).
-   * 
+   * Creates a Map of <code>WeightedSpanTerms</code> from the given <code>Query</code> and <code>
+   * TokenStream</code>. Uses a supplied <code>IndexReader</code> to properly weight terms (for
+   * gradient highlighting).
+   *
    * @param query that caused hit
    * @param tokenStream of text to be highlighted
    * @param fieldName restricts Term's used based on field name
@@ -537,16 +530,13 @@ public class WeightedSpanTermExtractor {
    * @return Map of WeightedSpanTerms with quasi tf/idf scores
    * @throws IOException If there is a low-level I/O error
    */
-  public Map<String,WeightedSpanTerm> getWeightedSpanTermsWithScores(Query query, float boost, TokenStream tokenStream, String fieldName,
-      IndexReader reader) throws IOException {
-    if (fieldName != null) {
-      this.fieldName = fieldName;
-    } else {
-      this.fieldName = null;
-    }
+  public Map<String, WeightedSpanTerm> getWeightedSpanTermsWithScores(
+      Query query, float boost, TokenStream tokenStream, String fieldName, IndexReader reader)
+      throws IOException {
+    this.fieldName = fieldName;
     this.tokenStream = tokenStream;
 
-    Map<String,WeightedSpanTerm> terms = new PositionCheckingMap<>();
+    Map<String, WeightedSpanTerm> terms = new PositionCheckingMap<>();
     extract(query, boost, terms);
 
     int totalNumDocs = reader.maxDoc();
@@ -570,82 +560,78 @@ public class WeightedSpanTermExtractor {
 
   protected void collectSpanQueryFields(SpanQuery spanQuery, Set<String> fieldNames) {
     if (spanQuery instanceof FieldMaskingSpanQuery) {
-      collectSpanQueryFields(((FieldMaskingSpanQuery)spanQuery).getMaskedQuery(), fieldNames);
+      collectSpanQueryFields(((FieldMaskingSpanQuery) spanQuery).getMaskedQuery(), fieldNames);
     } else if (spanQuery instanceof SpanFirstQuery) {
-      collectSpanQueryFields(((SpanFirstQuery)spanQuery).getMatch(), fieldNames);
+      collectSpanQueryFields(((SpanFirstQuery) spanQuery).getMatch(), fieldNames);
     } else if (spanQuery instanceof SpanNearQuery) {
-      for (final SpanQuery clause : ((SpanNearQuery)spanQuery).getClauses()) {
+      for (final SpanQuery clause : ((SpanNearQuery) spanQuery).getClauses()) {
         collectSpanQueryFields(clause, fieldNames);
       }
     } else if (spanQuery instanceof SpanNotQuery) {
-      collectSpanQueryFields(((SpanNotQuery)spanQuery).getInclude(), fieldNames);
+      collectSpanQueryFields(((SpanNotQuery) spanQuery).getInclude(), fieldNames);
     } else if (spanQuery instanceof SpanOrQuery) {
-      for (final SpanQuery clause : ((SpanOrQuery)spanQuery).getClauses()) {
+      for (final SpanQuery clause : ((SpanOrQuery) spanQuery).getClauses()) {
         collectSpanQueryFields(clause, fieldNames);
       }
     } else {
       fieldNames.add(spanQuery.getField());
     }
   }
-  
+
   protected boolean mustRewriteQuery(SpanQuery spanQuery) {
     if (!expandMultiTermQuery) {
       return false; // Will throw UnsupportedOperationException in case of a SpanRegexQuery.
     } else if (spanQuery instanceof FieldMaskingSpanQuery) {
-      return mustRewriteQuery(((FieldMaskingSpanQuery)spanQuery).getMaskedQuery());
+      return mustRewriteQuery(((FieldMaskingSpanQuery) spanQuery).getMaskedQuery());
     } else if (spanQuery instanceof SpanFirstQuery) {
-      return mustRewriteQuery(((SpanFirstQuery)spanQuery).getMatch());
+      return mustRewriteQuery(((SpanFirstQuery) spanQuery).getMatch());
     } else if (spanQuery instanceof SpanNearQuery) {
-      for (final SpanQuery clause : ((SpanNearQuery)spanQuery).getClauses()) {
+      for (final SpanQuery clause : ((SpanNearQuery) spanQuery).getClauses()) {
         if (mustRewriteQuery(clause)) {
           return true;
         }
       }
-      return false; 
+      return false;
     } else if (spanQuery instanceof SpanNotQuery) {
-      SpanNotQuery spanNotQuery = (SpanNotQuery)spanQuery;
-      return mustRewriteQuery(spanNotQuery.getInclude()) || mustRewriteQuery(spanNotQuery.getExclude());
+      SpanNotQuery spanNotQuery = (SpanNotQuery) spanQuery;
+      return mustRewriteQuery(spanNotQuery.getInclude())
+          || mustRewriteQuery(spanNotQuery.getExclude());
     } else if (spanQuery instanceof SpanOrQuery) {
-      for (final SpanQuery clause : ((SpanOrQuery)spanQuery).getClauses()) {
+      for (final SpanQuery clause : ((SpanOrQuery) spanQuery).getClauses()) {
         if (mustRewriteQuery(clause)) {
           return true;
         }
       }
-      return false; 
+      return false;
     } else if (spanQuery instanceof SpanTermQuery) {
       return false;
     } else {
       return true;
     }
   }
-  
+
   /**
-   * This class makes sure that if both position sensitive and insensitive
-   * versions of the same term are added, the position insensitive one wins.
+   * This class makes sure that if both position sensitive and insensitive versions of the same term
+   * are added, the position insensitive one wins.
    */
-  @SuppressWarnings("serial")
-  protected static class PositionCheckingMap<K> extends HashMap<K,WeightedSpanTerm> {
+  protected static class PositionCheckingMap<K> extends HashMap<K, WeightedSpanTerm> {
 
     @Override
-    public void putAll(Map<? extends K,? extends WeightedSpanTerm> m) {
-      for (Map.Entry<? extends K,? extends WeightedSpanTerm> entry : m.entrySet())
+    public void putAll(Map<? extends K, ? extends WeightedSpanTerm> m) {
+      for (Map.Entry<? extends K, ? extends WeightedSpanTerm> entry : m.entrySet())
         this.put(entry.getKey(), entry.getValue());
     }
 
     @Override
-    public WeightedSpanTerm put(K key, WeightedSpanTerm value) {
-      WeightedSpanTerm prev = super.put(key, value);
-      if (prev == null) return prev;
-      WeightedSpanTerm prevTerm = prev;
-      WeightedSpanTerm newTerm = value;
-      if (!prevTerm.positionSensitive) {
+    public WeightedSpanTerm put(K key, WeightedSpanTerm newTerm) {
+      WeightedSpanTerm prevTerm = super.put(key, newTerm);
+      if (prevTerm != null && prevTerm.positionSensitive == false) {
         newTerm.positionSensitive = false;
       }
-      return prev;
+      return prevTerm;
     }
-    
   }
-  
+
   public boolean getExpandMultiTermQuery() {
     return expandMultiTermQuery;
   }
@@ -666,28 +652,30 @@ public class WeightedSpanTermExtractor {
     return cachedTokenStream;
   }
 
-  /** Returns the tokenStream which may have been wrapped in a CachingTokenFilter.
-   * getWeightedSpanTerms* sets the tokenStream, so don't call this before. */
+  /**
+   * Returns the tokenStream which may have been wrapped in a CachingTokenFilter.
+   * getWeightedSpanTerms* sets the tokenStream, so don't call this before.
+   */
   public TokenStream getTokenStream() {
     assert tokenStream != null;
     return tokenStream;
   }
-  
+
   /**
-   * By default, {@link TokenStream}s that are not of the type
-   * {@link CachingTokenFilter} are wrapped in a {@link CachingTokenFilter} to
-   * ensure an efficient reset - if you are already using a different caching
-   * {@link TokenStream} impl and you don't want it to be wrapped, set this to
-   * false. This setting is ignored when a term vector based TokenStream is supplied,
-   * since it can be reset efficiently.
+   * By default, {@link TokenStream}s that are not of the type {@link CachingTokenFilter} are
+   * wrapped in a {@link CachingTokenFilter} to ensure an efficient reset - if you are already using
+   * a different caching {@link TokenStream} impl and you don't want it to be wrapped, set this to
+   * false. This setting is ignored when a term vector based TokenStream is supplied, since it can
+   * be reset efficiently.
    */
   public void setWrapIfNotCachingTokenFilter(boolean wrap) {
     this.wrapToCaching = wrap;
   }
 
-  /** A threshold of number of characters to analyze. When a TokenStream based on
-   * term vectors with offsets and positions are supplied, this setting
-   * does not apply. */
+  /**
+   * A threshold of number of characters to analyze. When a TokenStream based on term vectors with
+   * offsets and positions are supplied, this setting does not apply.
+   */
   protected final void setMaxDocCharsToAnalyze(int maxDocCharsToAnalyze) {
     this.maxDocCharsToAnalyze = maxDocCharsToAnalyze;
   }
