@@ -57,7 +57,7 @@ import org.apache.lucene.search.SortField;
 import org.apache.lucene.search.SortedNumericSortField;
 import org.apache.lucene.search.SortedSetSortField;
 import org.apache.lucene.search.TopDocs;
-import org.apache.lucene.search.TopScoreDocCollector;
+import org.apache.lucene.search.TopScoreDocCollectorManager;
 import org.apache.lucene.search.TotalHits;
 import org.apache.lucene.search.similarities.BM25Similarity;
 import org.apache.lucene.search.similarities.ClassicSimilarity;
@@ -107,7 +107,7 @@ public final class SearchImpl extends LukeModel implements Search {
         .map(f -> IndexUtils.getFieldInfo(reader, f))
         .filter(info -> !info.getDocValuesType().equals(DocValuesType.NONE))
         .map(info -> info.name)
-        .collect(Collectors.toList());
+        .toList();
   }
 
   @Override
@@ -116,7 +116,7 @@ public final class SearchImpl extends LukeModel implements Search {
         .map(f -> IndexUtils.getFieldInfo(reader, f))
         .filter(info -> !info.getIndexOptions().equals(IndexOptions.NONE))
         .map(info -> info.name)
-        .collect(Collectors.toList());
+        .toList();
   }
 
   @Override
@@ -155,7 +155,7 @@ public final class SearchImpl extends LukeModel implements Search {
         query = query.rewrite(searcher);
       } catch (IOException e) {
         throw new LukeException(
-            String.format(Locale.ENGLISH, "Failed to rewrite query: %s", query.toString()), e);
+            String.format(Locale.ENGLISH, "Failed to rewrite query: %s", query), e);
       }
     }
 
@@ -314,9 +314,10 @@ public final class SearchImpl extends LukeModel implements Search {
       topDocs = searcher.searchAfter(after, query, pageSize, sort);
     } else {
       int hitsThreshold = exactHitsCount ? Integer.MAX_VALUE : DEFAULT_TOTAL_HITS_THRESHOLD;
-      TopScoreDocCollector collector = TopScoreDocCollector.create(pageSize, after, hitsThreshold);
-      searcher.search(query, collector);
-      topDocs = collector.topDocs();
+      TopScoreDocCollectorManager collectorManager =
+          new TopScoreDocCollectorManager(
+              pageSize, after, hitsThreshold, searcher.getSlices().length > 1);
+      topDocs = searcher.search(query, collectorManager);
     }
 
     // reset total hits for the current query
@@ -341,9 +342,9 @@ public final class SearchImpl extends LukeModel implements Search {
     // proceed to next page
     currentPage += 1;
 
-    if (totalHits.value == 0
-        || (totalHits.relation == TotalHits.Relation.EQUAL_TO
-            && currentPage * (long) pageSize >= totalHits.value)) {
+    if (totalHits.value() == 0
+        || (totalHits.relation() == TotalHits.Relation.EQUAL_TO
+            && currentPage * (long) pageSize >= totalHits.value())) {
       log.warning("No more next search results are available.");
       return Optional.empty();
     }
@@ -394,8 +395,7 @@ public final class SearchImpl extends LukeModel implements Search {
     Similarity similarity;
 
     if (config.isUseClassicSimilarity()) {
-      ClassicSimilarity tfidf = new ClassicSimilarity();
-      tfidf.setDiscountOverlaps(config.isDiscountOverlaps());
+      ClassicSimilarity tfidf = new ClassicSimilarity(config.isDiscountOverlaps());
       similarity = tfidf;
     } else {
       BM25Similarity bm25 =
@@ -427,7 +427,7 @@ public final class SearchImpl extends LukeModel implements Search {
                   new SortField(name, SortField.Type.FLOAT),
                   new SortField(name, SortField.Type.DOUBLE)
                 })
-            .collect(Collectors.toList());
+            .toList();
 
       case SORTED_NUMERIC:
         return Arrays.stream(
@@ -437,7 +437,7 @@ public final class SearchImpl extends LukeModel implements Search {
                   new SortedNumericSortField(name, SortField.Type.FLOAT),
                   new SortedNumericSortField(name, SortField.Type.DOUBLE)
                 })
-            .collect(Collectors.toList());
+            .toList();
 
       case SORTED:
         return Arrays.stream(
@@ -445,7 +445,7 @@ public final class SearchImpl extends LukeModel implements Search {
                   new SortField(name, SortField.Type.STRING),
                   new SortField(name, SortField.Type.STRING_VAL)
                 })
-            .collect(Collectors.toList());
+            .toList();
 
       case SORTED_SET:
         return Collections.singletonList(new SortedSetSortField(name, false));
