@@ -51,7 +51,9 @@ import org.apache.lucene.tests.index.RandomIndexWriter;
 import org.apache.lucene.tests.util.LuceneTestCase;
 import org.apache.lucene.util.BytesRef;
 import org.apache.lucene.util.IOUtils;
+import org.apache.lucene.util.automaton.Automaton;
 import org.apache.lucene.util.automaton.CompiledAutomaton;
+import org.apache.lucene.util.automaton.Operations;
 import org.apache.lucene.util.automaton.RegExp;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
@@ -61,7 +63,7 @@ public class TestIntervals extends LuceneTestCase {
   //   0         1         2         3         4         5         6         7         8         9
   //
   // 012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
-  private static String[] field1_docs = {
+  private static final String[] field1_docs = {
     "Nothing of interest to anyone here",
     "Pease porridge hot, pease porridge cold, pease porridge in the pot nine days old.  Some like it hot, some like it cold, some like it in the pot nine days old",
     "Pease porridge cold, pease porridge hot, pease porridge in the pot twelve days old.  Some like it cold, some like it hot, some like it in the fraggle",
@@ -74,7 +76,7 @@ public class TestIntervals extends LuceneTestCase {
   //   0         1         2         3         4         5         6         7         8         9
   //
   // 012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
-  private static String[] field2_docs = {
+  private static final String[] field2_docs = {
     "In Xanadu did Kubla Khan a stately pleasure dome decree",
     "Where Alph the sacred river ran through caverns measureless to man",
     "a b a c b a b c",
@@ -86,7 +88,7 @@ public class TestIntervals extends LuceneTestCase {
 
   private static Directory directory;
   private static IndexSearcher searcher;
-  private static Analyzer analyzer = new StandardAnalyzer(CharArraySet.EMPTY_SET);
+  private static final Analyzer analyzer = new StandardAnalyzer(CharArraySet.EMPTY_SET);
 
   private static final FieldType FIELD_TYPE = new FieldType(TextField.TYPE_STORED);
 
@@ -200,7 +202,7 @@ public class TestIntervals extends LuceneTestCase {
           @Override
           public void consumeTerms(Query query, Term... terms) {
             visitedSources[0]++;
-            actualTerms.addAll(Arrays.stream(terms).map(Term::text).collect(Collectors.toList()));
+            actualTerms.addAll(Arrays.stream(terms).map(Term::text).toList());
           }
 
           @Override
@@ -259,9 +261,9 @@ public class TestIntervals extends LuceneTestCase {
     IllegalArgumentException e =
         expectThrows(
             IllegalArgumentException.class,
-            () -> {
-              Intervals.term("wibble").intervals("id", searcher.getIndexReader().leaves().get(0));
-            });
+            () ->
+                Intervals.term("wibble")
+                    .intervals("id", searcher.getIndexReader().leaves().get(0)));
     assertEquals(
         "Cannot create an IntervalIterator over field id because it has no indexed positions",
         e.getMessage());
@@ -452,10 +454,7 @@ public class TestIntervals extends LuceneTestCase {
     Collections.shuffle(Arrays.asList(terms), random());
 
     IntervalsSource source =
-        Intervals.or(
-            Arrays.stream(terms)
-                .map((term) -> Intervals.term(term))
-                .toArray((sz) -> new IntervalsSource[sz]));
+        Intervals.or(Arrays.stream(terms).map(Intervals::term).toArray(IntervalsSource[]::new));
     assertEquals(expected, source.toString());
   }
 
@@ -1157,8 +1156,9 @@ public class TestIntervals extends LuceneTestCase {
 
   public void testMultiTerm() throws IOException {
     RegExp re = new RegExp("p.*e");
-    IntervalsSource source =
-        Intervals.multiterm(new CompiledAutomaton(re.toAutomaton()), re.toString());
+    Automaton automaton =
+        Operations.determinize(re.toAutomaton(), Operations.DEFAULT_DETERMINIZE_WORK_LIMIT);
+    IntervalsSource source = Intervals.multiterm(new CompiledAutomaton(automaton), re.toString());
 
     checkIntervals(
         source,
@@ -1178,7 +1178,7 @@ public class TestIntervals extends LuceneTestCase {
             IllegalStateException.class,
             () -> {
               IntervalsSource s =
-                  Intervals.multiterm(new CompiledAutomaton(re.toAutomaton()), 1, re.toString());
+                  Intervals.multiterm(new CompiledAutomaton(automaton), 1, re.toString());
               for (LeafReaderContext ctx : searcher.getIndexReader().leaves()) {
                 s.intervals("field1", ctx);
               }
@@ -1187,4 +1187,27 @@ public class TestIntervals extends LuceneTestCase {
 
     checkVisits(source, 1);
   }
+
+  // basic test for equality and inequality of instances created by the factories
+  public void testEquality() {
+    assertEquals(Intervals.term("wibble"), Intervals.term("wibble"));
+    assertEquals(Intervals.prefix(new BytesRef("p"), 1), Intervals.prefix(new BytesRef("p"), 1));
+    assertEquals(Intervals.fuzzyTerm("kot", 1), Intervals.fuzzyTerm("kot", 1));
+    assertEquals(Intervals.regexp(new BytesRef(".*ot")), Intervals.regexp(new BytesRef(".*ot")));
+    assertEquals(
+        Intervals.wildcard(new BytesRef("*.txt")), Intervals.wildcard(new BytesRef("*.txt")));
+    assertEquals(
+        Intervals.range(new BytesRef("cold"), new BytesRef("hot"), true, true),
+        Intervals.range(new BytesRef("cold"), new BytesRef("hot"), true, true));
+
+    assertNotEquals(Intervals.term("wibble"), Intervals.term("wobble"));
+    assertNotEquals(Intervals.prefix(new BytesRef("p"), 1), Intervals.prefix(new BytesRef("b"), 1));
+    assertNotEquals(Intervals.fuzzyTerm("kot", 1), Intervals.fuzzyTerm("kof", 1));
+    assertNotEquals(Intervals.regexp(new BytesRef(".*ot")), Intervals.regexp(new BytesRef(".*at")));
+    assertNotEquals(
+        Intervals.wildcard(new BytesRef("*.txt")), Intervals.wildcard(new BytesRef("*.tat")));
+    assertNotEquals(
+        Intervals.range(new BytesRef("warm"), new BytesRef("hot"), true, true),
+        Intervals.range(new BytesRef("cold"), new BytesRef("hot"), true, true));
+  }
 }
