@@ -17,14 +17,14 @@
 
 package org.apache.lucene.replicator.nrt;
 
+import static java.nio.charset.StandardCharsets.UTF_8;
+
 import com.carrotsearch.randomizedtesting.SeedUtils;
 import java.io.BufferedReader;
 import java.io.Closeable;
 import java.io.IOException;
 import java.io.InputStreamReader;
-import java.io.UnsupportedEncodingException;
 import java.io.Writer;
-import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
@@ -37,6 +37,7 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
@@ -94,8 +95,7 @@ import org.apache.lucene.util.ThreadInterruptedException;
 // hazardous window
 //   - replica comes up just as the primary is crashing / moving
 //   - electing a new primary when a replica is just finishing its nrt sync: we need to wait for it
-// so we are sure to get the "most up to
-//     date" replica
+// so we are sure to get the "most up to date" replica
 //   - replica comes up after merged segment finished so it doesn't copy over the merged segment
 // "promptly" (i.e. only sees it on NRT refresh)
 
@@ -120,6 +120,7 @@ import org.apache.lucene.util.ThreadInterruptedException;
 @SuppressCodecs({"MockRandom", "Direct", "SimpleText"})
 @SuppressSysoutChecks(bugUrl = "Stuff gets printed, important stuff for debugging a failure")
 public class TestStressNRTReplication extends LuceneTestCase {
+  public static final String CRASH_MSG_PREFIX = "will crash after ";
 
   // Test evilness controls:
 
@@ -189,6 +190,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
   final Set<Integer> crashingNodes = Collections.synchronizedSet(new HashSet<>());
 
   @Nightly
+  @SuppressForbidden(reason = "Thread sleep")
   public void test() throws Exception {
 
     Node.globalStartNS = System.nanoTime();
@@ -210,7 +212,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
     if (NUM_NODES == null) {
       numNodes = TestUtil.nextInt(random(), 2, 10);
     } else {
-      numNodes = NUM_NODES.intValue();
+      numNodes = NUM_NODES;
     }
 
     System.out.println("TEST: using " + numNodes + " nodes");
@@ -256,7 +258,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
     if (TEST_NIGHTLY) {
       runTimeSec = RANDOM_MULTIPLIER * TestUtil.nextInt(random(), 120, 240);
     } else {
-      runTimeSec = RANDOM_MULTIPLIER * TestUtil.nextInt(random(), 45, 120);
+      runTimeSec = RANDOM_MULTIPLIER * TestUtil.nextInt(random(), 20, 60);
     }
 
     System.out.println("TEST: will run for " + runTimeSec + " sec");
@@ -347,7 +349,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
     }
     restarter.join();
 
-    // Close replicas before primary so we cancel any in-progres replications:
+    // Close replicas before primary so we cancel any in-progress replications:
     System.out.println("TEST: top: now close replicas");
     List<Closeable> toClose = new ArrayList<>();
     for (NodeProcess node : nodes) {
@@ -484,7 +486,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
 
     // When the primary starts, the userData in its latest commit point tells us which version it
     // had indexed up to, so we know where to
-    // replay from in the xlog.  However, we forcefuly advance the version, and then IW on init (or
+    // replay from in the xlog.  However, we forcefully advance the version, and then IW on init (or
     // maybe getReader) also adds 1 to it.
     // Since we publish the primary in this state (before xlog replay is done), a replica can start
     // up at this point and pull this version,
@@ -620,10 +622,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
       message("logging to " + childOut);
       childLog =
           Files.newBufferedWriter(
-              childOut,
-              StandardCharsets.UTF_8,
-              StandardOpenOption.APPEND,
-              StandardOpenOption.CREATE);
+              childOut, UTF_8, StandardOpenOption.APPEND, StandardOpenOption.CREATE);
       childLog.write("\n\nSTART NEW CHILD:\n");
     } else {
       childLog = null;
@@ -638,18 +637,14 @@ public class TestStressNRTReplication extends LuceneTestCase {
 
     Process p = pb.start();
 
-    BufferedReader r;
-    try {
-      r = new BufferedReader(new InputStreamReader(p.getInputStream(), IOUtils.UTF_8));
-    } catch (UnsupportedEncodingException uee) {
-      throw new RuntimeException(uee);
-    }
+    BufferedReader r = new BufferedReader(new InputStreamReader(p.getInputStream(), UTF_8));
 
     int tcpPort = -1;
     long initCommitVersion = -1;
     long initInfosVersion = -1;
-    Pattern logTimeStart = Pattern.compile("^[0-9\\.]+s .*");
+    Pattern logTimeStart = Pattern.compile("^[0-9.]+s .*");
     boolean willCrash = false;
+    Optional<Thread> subprocessKiller = Optional.empty();
 
     while (true) {
       String l = r.readLine();
@@ -709,8 +704,40 @@ public class TestStressNRTReplication extends LuceneTestCase {
         initCommitVersion = Integer.parseInt(l.substring(16).trim());
       } else if (l.startsWith("INFOS VERSION: ")) {
         initInfosVersion = Integer.parseInt(l.substring(15).trim());
-      } else if (l.contains("will crash after")) {
+      } else if (l.contains(CRASH_MSG_PREFIX)) {
         willCrash = true;
+        Pattern crashMsg =
+            Pattern.compile(Pattern.quote(CRASH_MSG_PREFIX) + "\\s*(?<millis>[0-9]+)");
+        var m = crashMsg.matcher(l);
+        if (!m.find()) {
+          throw new AssertionError("Expected the crash message to include the timeout: " + l);
+        }
+        final long deadline =
+            System.nanoTime() + TimeUnit.MILLISECONDS.toNanos(Integer.parseInt(m.group("millis")));
+        // Fork a new thread that will attempt to terminate the subprocess after a certain delay.
+        // We don't keep track of these "killer" subprocesses; they will end gracefully if the
+        // subprocess is terminated prior to the timeout.
+        subprocessKiller =
+            Optional.of(
+                new Thread(
+                    () -> {
+                      while (System.nanoTime() < deadline && p.isAlive()) {
+                        try {
+                          Thread.sleep(250);
+                        } catch (
+                            @SuppressWarnings("unused")
+                            InterruptedException e) {
+                          // If we do get interrupted, it's likely we're being cleaned up. Do
+                          // proceed immediately then.
+                          break;
+                        }
+                      }
+                      if (p.isAlive()) {
+                        message("now killing process " + p);
+                        p.destroyForcibly();
+                      }
+                    }));
+        subprocessKiller.get().start();
       } else if (l.startsWith("NODE STARTED")) {
         break;
       }
@@ -723,48 +750,43 @@ public class TestStressNRTReplication extends LuceneTestCase {
     // nodeClosed once it exits:
     Thread pumper =
         ThreadPumper.start(
-            new Runnable() {
-              @Override
-              public void run() {
-                message("now wait for process " + p);
-                try {
-                  p.waitFor();
-                } catch (Throwable t) {
-                  throw new RuntimeException(t);
-                }
-
-                message("done wait for process " + p);
-                int exitValue = p.exitValue();
-                message("exit value=" + exitValue + " willCrash=" + finalWillCrash);
-                if (childLog != null) {
-                  try {
-                    childLog.write("process done; exitValue=" + exitValue + "\n");
-                    childLog.close();
-                  } catch (IOException ioe) {
-                    throw new RuntimeException(ioe);
-                  }
-                }
-                if (exitValue != 0
-                    && finalWillCrash == false
-                    && crashingNodes.remove(id) == false) {
-                  // should fail test
-                  failed.set(true);
-                  if (childLog != null) {
-                    throw new RuntimeException(
-                        "node "
-                            + id
-                            + " process had unexpected non-zero exit status="
-                            + exitValue
-                            + "; see "
-                            + childLog
-                            + " for details");
-                  } else {
-                    throw new RuntimeException(
-                        "node " + id + " process had unexpected non-zero exit status=" + exitValue);
-                  }
-                }
-                nodeClosed(id);
+            () -> {
+              message("now wait for process " + p);
+              try {
+                p.waitFor();
+              } catch (Throwable t) {
+                throw new RuntimeException(t);
               }
+
+              message("done wait for process " + p);
+              int exitValue = p.exitValue();
+              message("exit value=" + exitValue + " willCrash=" + finalWillCrash);
+              if (childLog != null) {
+                try {
+                  childLog.write("process done; exitValue=" + exitValue + "\n");
+                  childLog.close();
+                } catch (IOException ioe) {
+                  throw new RuntimeException(ioe);
+                }
+              }
+              if (exitValue != 0 && finalWillCrash == false && crashingNodes.remove(id) == false) {
+                // should fail test
+                failed.set(true);
+                if (childLog != null) {
+                  throw new RuntimeException(
+                      "node "
+                          + id
+                          + " process had unexpected non-zero exit status="
+                          + exitValue
+                          + "; see "
+                          + childLog
+                          + " for details");
+                } else {
+                  throw new RuntimeException(
+                      "node " + id + " process had unexpected non-zero exit status=" + exitValue);
+                }
+              }
+              nodeClosed(id);
             },
             r,
             System.out,
@@ -782,7 +804,15 @@ public class TestStressNRTReplication extends LuceneTestCase {
             + " initInfosVersion="
             + initInfosVersion);
     return new NodeProcess(
-        p, id, tcpPort, pumper, isPrimary, initCommitVersion, initInfosVersion, nodeIsClosing);
+        p,
+        id,
+        tcpPort,
+        pumper,
+        isPrimary,
+        initCommitVersion,
+        initInfosVersion,
+        nodeIsClosing,
+        subprocessKiller);
   }
 
   private void nodeClosed(int id) {
@@ -866,6 +896,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
   }
 
   // Periodically wakes up and starts up any down nodes:
+  @SuppressForbidden(reason = "Thread sleep")
   private class RestartThread extends Thread {
     @Override
     public void run() {
@@ -920,7 +951,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
             }
             b.append(String.format(Locale.ROOT, "%s%d(%.1fs)", prefix, i, sec));
           }
-          message("node status" + b.toString());
+          message("node status" + b);
           message("downNodes=" + downNodes);
 
           // If primary is down, promote a replica:
@@ -992,6 +1023,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
   }
 
   /** Randomly picks a node and runs a search against it */
+  @SuppressForbidden(reason = "Thread sleep")
   private class SearchThread extends Thread {
 
     @Override
@@ -1084,23 +1116,18 @@ public class TestStressNRTReplication extends LuceneTestCase {
             } else {
               // Just ensure that all nodes show the same hit count for
               // the same version, i.e. they really are replicas of one another:
-              if (oldHitCount.intValue() != hitCount) {
+              if (oldHitCount != hitCount) {
                 failed.set(true);
                 stop.set(true);
                 message(
                     "top: searcher: wrong version hitCount: version="
                         + version
                         + " oldHitCount="
-                        + oldHitCount.intValue()
+                        + oldHitCount
                         + " hitCount="
                         + hitCount);
                 fail(
-                    "version="
-                        + version
-                        + " oldHitCount="
-                        + oldHitCount.intValue()
-                        + " hitCount="
-                        + hitCount);
+                    "version=" + version + " oldHitCount=" + oldHitCount + " hitCount=" + hitCount);
               }
             }
           } catch (
@@ -1196,6 +1223,7 @@ public class TestStressNRTReplication extends LuceneTestCase {
     }
   }
 
+  @SuppressForbidden(reason = "Thread sleep")
   private class IndexThread extends Thread {
 
     @Override
@@ -1333,24 +1361,22 @@ public class TestStressNRTReplication extends LuceneTestCase {
 
   static void message(String message) {
     long now = System.nanoTime();
-    System.out.println(
-        String.format(
-            Locale.ROOT,
-            "%5.3fs       :     parent [%11s] %s",
-            (now - Node.globalStartNS) / (double) TimeUnit.SECONDS.toNanos(1),
-            Thread.currentThread().getName(),
-            message));
+    System.out.printf(
+        Locale.ROOT,
+        "%5.3fs       :     parent [%11s] %s%n",
+        (now - Node.globalStartNS) / (double) TimeUnit.SECONDS.toNanos(1),
+        Thread.currentThread().getName(),
+        message);
   }
 
   static void message(String message, long localStartNS) {
     long now = System.nanoTime();
-    System.out.println(
-        String.format(
-            Locale.ROOT,
-            "%5.3fs %5.1fs:     parent [%11s] %s",
-            (now - Node.globalStartNS) / (double) TimeUnit.SECONDS.toNanos(1),
-            (now - localStartNS) / (double) TimeUnit.SECONDS.toNanos(1),
-            Thread.currentThread().getName(),
-            message));
+    System.out.printf(
+        Locale.ROOT,
+        "%5.3fs %5.1fs:     parent [%11s] %s%n",
+        (now - Node.globalStartNS) / (double) TimeUnit.SECONDS.toNanos(1),
+        (now - localStartNS) / (double) TimeUnit.SECONDS.toNanos(1),
+        Thread.currentThread().getName(),
+        message);
   }
 }
