@@ -60,13 +60,9 @@ from lucene.ICUNormalizer2Filter import ICUNormalizer2Filter
 from icu import ResourceBundle, Normalizer2, UNormalizationMode2
 
 if sys.version_info < (3,):
-    utr30 = os.path.join(lucene.__dir__, 'resources',
-                         'org', 'apache', 'lucene', 'analysis', 'icu',
-                         'utr30.dat')
+    utr30 = os.path.join(lucene.__dir__, 'resources', 'utr30.dat')
 else:
-    utr30 = os.path.join(lucene.__module_dir__, 'resources',
-                         'org', 'apache', 'lucene', 'analysis', 'icu',
-                         'utr30.dat')
+    utr30 = os.path.join(lucene.__module_dir__, 'resources', 'utr30.dat')
 print(utr30)
 ResourceBundle.setAppData("utr30", utr30)
 
