@@ -39,30 +39,30 @@ def indexDirectory(dir):
 def indexFile(dir, filename):
 
     path = os.path.join(dir, filename)
-    print "  File: ", filename
+    print("  File: ", filename)
 
     if filename.endswith('.gz'):
         child = Popen('gunzip -c ' + path + ' | groff -t -e -E -mandoc -Tascii | col -bx', shell=True, stdout=PIPE, cwd=os.path.dirname(dir)).stdout
-        command, section = re.search('^(.*)\.(.*)\.gz$', filename).groups()
+        command, section = re.search(r'^(.*)\.(.*)\.gz$', filename).groups()
     else:
         child = Popen('groff -t -e -E -mandoc -Tascii ' + path + ' | col -bx',
                       shell=True, stdout=PIPE, cwd=os.path.dirname(dir)).stdout
-        command, section = re.search('^(.*)\.(.*)$', filename).groups()
+        command, section = re.search(r'^(.*)\.(.*)$', filename).groups()
 
-    data = child.read()
+    data = str(child.read(), 'utf-8')
     err = child.close()
     if err:
-        raise RuntimeError, '%s failed with exit code %d' %(command, err)
+        raise RuntimeError('%s failed with exit code %d' %(command, err))
 
-    matches = re.search('^NAME$(.*?)^\S', data,
+    matches = re.search(r'^NAME$(.*?)^\S', data,
                         re.MULTILINE | re.DOTALL)
     name = matches and matches.group(1) or ''
 
-    matches = re.search('^(?:SYNOPSIS|SYNOPSYS)$(.*?)^\S', data,
+    matches = re.search(r'^(?:SYNOPSIS|SYNOPSYS)$(.*?)^\S', data,
                         re.MULTILINE | re.DOTALL)
     synopsis = matches and matches.group(1) or ''
 
-    matches = re.search('^(?:DESCRIPTION|OVERVIEW)$(.*?)', data,
+    matches = re.search(r'^(?:DESCRIPTION|OVERVIEW)$(.*?)', data,
                         re.MULTILINE | re.DOTALL)
     description = matches and matches.group(1) or ''
 
@@ -81,11 +81,11 @@ def indexFile(dir, filename):
 if __name__ == '__main__':
 
     if len(sys.argv) != 2:
-        print "Usage: python manindex.py <index dir>"
+        print("Usage: python manindex.py <index dir>")
 
     else:
         lucene.initVM(vmargs=['-Djava.awt.headless=true'])
-        directory = SimpleFSDirectory(Paths.get(sys.argv[1]))
+        directory = NIOFSDirectory(Paths.get(sys.argv[1]))
         analyzer = StandardAnalyzer()
         analyzer = LimitTokenCountAnalyzer(analyzer, 10000)
         config = IndexWriterConfig(analyzer)
@@ -93,7 +93,7 @@ if __name__ == '__main__':
 
         manpath = os.environ.get('MANPATH', '/usr/share/man').split(os.pathsep)
         for dir in manpath:
-            print "Crawling", dir
+            print("Crawling", dir)
             for name in os.listdir(dir):
                 path = os.path.join(dir, name)
                 if os.path.isdir(path):
