@@ -44,7 +44,7 @@ class Test_Bug1564(PyLuceneTestCase):
         searcher = self.getSearcher()
         query = QueryParser('all', self.analyzer).parse('supreme')
         topDocs = searcher.search(query, 50)
-        self.assertEqual(topDocs.totalHits.value, 1)
+        self.assertEqual(topDocs.totalHits.value(), 1)
 
 
 if __name__ == '__main__':
